Package: faunabr
Title: Explore Catálogo Taxônomico da Fauna do Brasil Database
Version: 1.1.1
Authors@R: 
    person("Weverton", "Trindade", , "wevertonf1993@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-2045-4555"))
Description: A collection of functions designed to retrieve, filter and spatialize data from the Catálogo Taxônomico da Fauna do Brasil. For more information about the dataset, please visit <https://fauna.jbrj.gov.br/fauna/listaBrasil/>.
Imports: XML (>= 3.99.0.14), data.table (>= 1.14.8), httr (>= 1.4.6),
        terra (>= 1.7.39), stats (>= 4.2.3), utils(>= 4.2.3),
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 2.10)
LazyData: true
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
BugReports: https://github.com/wevertonbio/faunabr/issues
URL: https://wevertonbio.github.io/faunabr/
NeedsCompilation: no
Packaged: 2026-08-21 01:02:19 UTC; wever
Author: Weverton Trindade [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2045-4555>)
Maintainer: Weverton Trindade <wevertonf1993@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-21 13:30:25 UTC
Built: R 4.6.0; ; 2026-08-21 14:21:19 UTC; unix

# Generated by roxygen2: do not edit by hand

export(check_fauna_names)
export(extract_binomial)
export(fauna_attributes)
export(fauna_by_vernacular)
export(fauna_discrepancies)
export(fauna_pam)
export(fauna_spat_occ)
export(fauna_synonym)
export(fauna_version)
export(filter_faunabr)
export(get_faunabr)
export(load_faunabr)
export(merge_data)
export(select_fauna)
export(subset_fauna)
export(translate_faunabr)
importFrom(XML,htmlParse)
importFrom(XML,xmlGetAttr)
importFrom(XML,xpathSApply)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,is.data.table)
importFrom(data.table,rbindlist)
importFrom(data.table,setDT)
importFrom(data.table,setkey)
importFrom(data.table,setnames)
importFrom(grDevices,terrain.colors)
importFrom(httr,GET)
importFrom(httr,write_disk)
importFrom(stats,ave)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(terra,aggregate)
importFrom(terra,as.data.frame)
importFrom(terra,buffer)
importFrom(terra,intersect)
importFrom(terra,mask)
importFrom(terra,merge)
importFrom(terra,plot)
importFrom(terra,subset)
importFrom(terra,unwrap)
importFrom(terra,vect)
importFrom(utils,adist)
importFrom(utils,read.csv)
importFrom(utils,unzip)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# faunabr <img src="man/figures/logo.png" align="right" width="20%" height="20%"/>

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/faunabr)](https://CRAN.R-project.org/package=faunabr)
[![downloads](https://cranlogs.r-pkg.org/badges/grand-total/faunabr)](https://cranlogs.r-pkg.org:443/badges/grand-total/faunabr)
[![License](https://img.shields.io/badge/license-GPL%20(%3E=%203)-lightgrey.svg?style=flat)](http://www.gnu.org/licenses/gpl-3.0.html)
[![R-CMD-check](https://github.com/wevertonbio/faunabr/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/wevertonbio/faunabr/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

The [Catálogo Taxonômico da Fauna do Brasil](https://fauna.jbrj.gov.br/fauna) is the result of the collaborative efforts of over 500 zoologists, each specializing in various animal groups native to Brazil. This comprehensive database offers detailed and standardized ecological descriptions, nomenclatural information, geographic distribution, and identification keys for both native and non-native animals found in Brazil.

The faunabr package provides a suite of functions designed to retrieve, filter, and spatialize data from the Fauna do Brasil dataset.

## Package citation
Trindade, W. C. F. (2025). faunabr: An R package to explore taxonomic data and map species distributions using the Catálogo Taxonômico da Fauna do Brasil. Zoologia (Curitiba), 42, e25027. [https://doi.org/10.1590/s1984-4689.v42.e25027](https://doi.org/10.1590/s1984-4689.v42.e25027).

## Overview of functions

### Download and import Fauna do Brasil Dataset

- `fauna_version()`: Check if you have the latest version of Fauna do
  Brasil dataset available.
- `get_faunabr()`: Download the latest version of Fauna do Brasil
  database.
- `load_faunabr()`: Load Fauna do Brasil database.

### Check names

- `check_fauna_names()`: Check species names.
- `extract_binomial()`: Extract the binomial name (Genus + specific epithet)
  from a complete Scientific Name.
- `fauna_synonym()`: Retrieve synonyms for species.

### Select or subset species

- `fauna_attributes()`: Get available attributes to filter species.
- `fauna_by_vernacular()`: Search for taxa using vernacular names/common names.
- `select_fauna()`: Select species based on its characteristics and
  distribution.
- `subset_fauna()`: Extract a subset of species from Fauna do Brasil database.

### Spatialize distribution and use Fauna do Brasil to flag records

- `fauna_pam()`: Get a presence-absence matrix of species based on its 
distribution in brazilian states and/or countries
- `filter_faunabr()`: Identify records outside natural ranges according
  to Fauna do Brasil.
- `fauna_spat_occ()`: Get Spatial polygons (SpatVectors) of species based
  on its distribution (States and countries) according to Fauna do Brasil

## Installation

### Install stable version from CRAN

To install the stable version of `faunabr` use:

``` r
install.packages("faunabr")
```

<br>

### Install development version from GitHub

You can install the development version of faunabr from
[GitHub](https://github.com/wevertonbio/faunabr) with:

``` r
if(!require(remotes)){
    install.packages("remotes")
}

if(!require(faunabr)){
remotes::install_github('wevertonbio/faunabr')}

library(faunabr)
```

Before downloading the data available in the Fauna do Brasil, we need to
create a folder to save the data:

``` r
#Creating a folder in a temporary directory
#Replace 'file.path(tempdir(), "faunabr")' by a path folder to be create in your computer
my_dir <- file.path(file.path(tempdir(), "faunabr"))
dir.create(my_dir)
```

You can now utilize the `get_faunabr` function to retrieve the most recent 
version of the data:

``` r
get_faunabr(output_dir = my_dir, #directory to save the data
            data_version = "latest", #get the most recent version available
            overwrite = T) #Overwrite data, if it exists
```

You also have the option to download an older, specific version of the Fauna do 
Brasil dataset. To explore the available versions, please refer to
[this link](https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil).
For downloading a particular version, simply replace ‘latest’ with the
desired version number. For example:

``` r
get_faunabr(output_dir = my_dir, #directory to save the data
            data_version = "1.10", #Version 1.10, published on 2024-02-01
            overwrite = T) #Overwrite data, if it exists
```

## Loading data

In order to use the other functions of the package, you need to load the
data into your environment. To achieve this, utilize the `load_faunabr()` 
function. By default, the function will automatically search for the latest 
available version in your directory. However, you have the option to specify a 
particular version using the *data_version* parameter. Additionally, you can 
choose between two versions of the data: the ‘short’ version (containing the 19
columns required for run the other functions of the package) or the ‘complete’ 
version (with all original 31 columns). The function imports the ‘short’ version
by default.

``` r
#Load the short version
bf <- load_fauna(data_dir = my_dir,
                   data_version = "latest",
                   type = "short") #short
```

## Package website

See the package website (<https://wevertonbio.github.io/faunabr/>) for
further functions explanation and vignettes.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  warning = FALSE
)

## -----------------------------------------------------------------------------
# library(faunabr)
# #Carregar dados
# bf <- load_faunabr(data_dir = my_dir, #Pasta onde foi salvo o arquivo com get_faunabr()
#                    data_version = "latest", #Versão mais recente
#                    type = "short") #Versão resumida
# #> Loading version 1.3

## -----------------------------------------------------------------------------
# #Obter as opções disponíveis para filtrar por forma de vida, habitat e origem
# fauna_at <- fauna_attributes(data = bf,
#                              attribute = c("lifeForm", "habitat", "origin"))
# head(fauna_at$lifeForm)
# #>          lifeForm
# #> 1        colonial
# #> 2       comensal
# #> 3    ectoparasito
# #> 4 ectoparasitoide
# #> 5    endoparasito
# #> 6  endoparasitoide
# 
# head(fauna_at$habitat)
# #> habitat
# #> 1          agua_doce
# #> 2 aguas_subterraneas
# #> 3            arboreo
# #> 4        cavernicola
# #> 5          fossorial
# #> 6         hiporreico
# 
# head(fauna_at$origin)
# #>         origin
# #> 1 criptogenica
# #> 2      exotica
# #> 3       nativa

## -----------------------------------------------------------------------------
# insects_rj <- select_fauna(data = bf, include_subspecies = FALSE,
#                            phylum = "all",
#                            class = "Insecta", #Insetos
#                            order = "all",
#                            family = "all",
#                            genus = "all",
#                            lifeForm = "all", filter_lifeForm = "in",
#                            habitat = "arboreo", filter_habitat = "in",
#                            states = "RJ", #Rio de Janeiro
#                            filter_states = "in", #Que ocorrem NO RJ
#                            country = "all", filter_country = "in",
#                            origin = "nativa",
#                            taxonomicStatus = "valido")
# nrow(insects_rj)
# #> [1] 114

## -----------------------------------------------------------------------------
# #Primeiros 7 valores únicos de Estados onde as espécies ocorrem
# unique(insects_rj$states)[1:7]
# #> [1] "AC;AM;AP;BA;CE;DF;ES;GO;MA;MG;MS;MT;PA;PE;RJ;RN;RO;RR;SE;SP;TO"
# #> [2] "BA;MA;MG;MT;PR;RJ;RS;SC;SP"
# #> [3] "ES;MG;PE;PR;RJ;RS;SP"
# #> [4] "ES;PR;RJ;SP"
# #> [5] "RJ"
# #> [6] "RJ;SP"
# #> [7] "ES;MG;RJ;RS;SC;SP"

## -----------------------------------------------------------------------------
# insects_rj_only <- select_fauna(data = bf, include_subspecies = FALSE,
#                            phylum = "all",
#                            class = "Insecta", #Insetos
#                            order = "all",
#                            family = "all",
#                            genus = "all",
#                            lifeForm = "all", filter_lifeForm = "in",
#                            habitat = "arboreo", filter_habitat = "in",
#                            states = "RJ", #Rio de Janeiro
#                            filter_states = "only", #Com ocorrência APENAS no RJ
#                            country = "all", filter_country = "in",
#                            origin = "nativa",
#                            taxonomicStatus = "valido")
# nrow(insects_rj_only)
# #> [1] 22
# unique(insects_rj_only$states)
# #> [1] "RJ"

## -----------------------------------------------------------------------------
# insects_south <- select_fauna(data = bf, include_subspecies = FALSE,
#                            phylum = "all",
#                            class = "Insecta", #Insetos
#                            order = "all",
#                            family = "all",
#                            genus = "all",
#                            lifeForm = "all", filter_lifeForm = "in",
#                            habitat = "all", filter_habitat = "in",
#                            states = c("PR", "SC", "RS"), #Estados do Sul do Brasil
#                            filter_states = "in", #EM qualquer um dos estados
#                            country = "all", filter_country = "in",
#                            origin = "nativa",
#                            taxonomicStatus = "valido")
# nrow(insects_south)
# #> [1] 11461
# 
# #Primeiros 10 valores únicos de Estados onde as espécies ocorrem
# unique(insects_south$states)[1:10]
# #> [1] "PR;SP"
# #> [2] "BA;MT;RJ;RS"
# #> [3] "RS"
# #> [4] "PR"
# #> [5] "PR;RS"
# #> [6] "GO;PA;RO;RR;SC"
# #> [7] "MG;PR;SP"
# #> [8] "GO;MG;PR;RS;SC"
# #> [9] "GO;RJ;SC"
# #> [10] "AC;AL;AM;AP;BA;CE;DF;ES;GO;MA;MG;MS;MT;PA;PB;PE;PI;PR;RJ;RN;RO;RR;RS;SC;SE;SP;TO"

## -----------------------------------------------------------------------------
# insects_south_and <- select_fauna(data = bf, include_subspecies = FALSE,
#                            phylum = "all",
#                            class = "Insecta", #Insetos
#                            order = "all",
#                            family = "all",
#                            genus = "all",
#                            lifeForm = "all", filter_lifeForm = "in",
#                            habitat = "all", filter_habitat = "in",
#                            states = c("PR", "SC", "RS"),
#                            filter_states = "and", #PR & SC & RS
#                            country = "all", filter_country = "in",
#                            origin = "nativa",
#                            taxonomicStatus = "valido")
# nrow(insects_south_and)
# #> [1] 1924
# 
# #Primeiros 10 valores únicos de Estados onde as espécies ocorrem
# unique(insects_south_and$states)[1:10]
# #> [1] "GO;MG;PR;RS;SC"
# #> [2] "AC;AL;AM;AP;BA;CE;DF;ES;GO;MA;MG;MS;MT;PA;PB;PE;PI;PR;RJ;RN;RO;RR;RS;SC;SE;SP;TO"
# #> [3] "AC;AM;MG;MS;MT;PA;PE;PR;RO;RS;SC;SP"
# #> [4] "AM;BA;CE;DF;ES;GO;MG;MS;MT;PA;PE;PR;RJ;RS;SC;SP"
# #> [5] "AC;AM;AP;BA;CE;DF;ES;GO;MG;MS;MT;PA;PE;PR;RJ;RO;RR;RS;SC;SE;SP;TO"
# #> [6] "BA;ES;MG;PR;RJ;RS;SC;SP"
# #> [7] "AM;DF;ES;GO;PR;RJ;RR;RS;SC;SP"
# #> [8] "ES;GO;MG;PR;RJ;RS;SC;SP"
# #> [9] "AC;AM;AP;BA;CE;ES;GO;MA;MG;MT;PA;PR;RJ;RO;RS;SC;SP;TO"
# #> [10] "GO;MG;MS;PR;RO;RS;SC;SP"

## -----------------------------------------------------------------------------
# insects_south_only <- select_fauna(data = bf, include_subspecies = FALSE,
#                            phylum = "all",
#                            class = "Insecta", #Insetos
#                            order = "all",
#                            family = "all",
#                            genus = "all",
#                            lifeForm = "all", filter_lifeForm = "in",
#                            habitat = "all", filter_habitat = "in",
#                            states = c("PR", "SC", "RS"),
#                            filter_states = "only", #Apenas no PR & SC & RS
#                            country = "all", filter_country = "in",
#                            origin = "nativa",
#                            taxonomicStatus = "valido")
# nrow(insects_south_only)
# #> [1] 134
# 
# #Valor único de Estados onde as espécies ocorrem
# unique(insects_south_only$states)
# #> [1] "PR;RS;SC"

## -----------------------------------------------------------------------------
# complete_names <- c("Pantera onca (Linnaeus, 1758)",
#                     "Zonotrichia capensis subtorquata Swainson, 1837",
#                     "Mazama bororo Duarte, 1996",
#                     "Mazama jucunda Thomas, 1913",
#                     "Arrenurus tumulosus intercursor",
#                     "Araucaria angustifolia")
# #Panthera onca com erros de digitação para ilustrar como a próxima função os corrige
# #Araucaria angustifolia (uma planta) foi usada apenas como um exemplo que será utilizado para ilustrar a próxima função
# binomial_names <- extract_binomial(species_names = complete_names)
# binomial_names
# #> [1] "Pantera onca"                         "Zonotrichia capensis"
# #> [3] "Mazama bororo"                        "Mazama jucunda"
# #> [5] "Arrenurus tumulosus"                  "Araucaria angustifolia"

## -----------------------------------------------------------------------------
# #Checar nomes
# checked_names <- check_fauna_names(data = bf,
#                              species = binomial_names,
#                              include_subspecies = FALSE,
#                              max_distance = 0.1)
# tibble::tibble(checked_names) #Exibir tabela como tibble
# #> # A tibble: 6 x 9
# #>   input_name               Spelling           Suggested_name       Distance taxonomicStatus nomenclaturalStatus validName   family        matches
# #>   <chr>                    <chr>              <chr>                   <dbl> <chr>           <chr>               <chr>          <chr>         <chr>
# #> 1 "Zonotrichia capensis"   Correct            Zonotrichia capensis        0 valido    NA                  NA             Passerellidae single
# #> 2 "Mazama bororo"          Correct            Mazama bororo               0 sinonimo        NA                  Mazama jucunda Cervidae      single
# #> 3 "Mazama jucunda"         Correct            Mazama jucunda              0 valido     NA                  NA             Cervidae      single
# #> 4 "Arrenurus tumulosus"    Correct            Arrenurus tumulosus         0 valido     NA                  NA             Arrenuridae   single
# #> 5 "Pantera onca"           Probably_incorrect Panthera onca               2 valido     NA                  NA             Felidae       single
# #> 6 "Araucaria angustifolia" Not_found          NA                         NA NA              NA                  NA             NA            none

## -----------------------------------------------------------------------------
# #Obter apenas nomes validos
# valids <- unique(checked_names$validName)
# valids <- na.omit(valids) #Remove NAs
# 
# #Subselecionar especies
# my_sp <- subset_fauna(data = bf, species = valids,
#                       include_subspecies = FALSE)
# tibble::tibble(my_sp)
# # A tibble: 4 x 21
# #>   species           subspecies scientificName validName kingdom phylum class order family genus lifeForm habitat states countryCode origin
# #>   <chr>             <chr>      <chr>          <chr>        <chr>   <chr>  <chr> <chr> <chr>  <chr> <chr>    <chr>   <chr>  <chr>       <chr>
# #> 1 Panthera onca     NA         Panthera onca~ NA           Animal~ Chord~ Mamm~ Carn~ Felid~ Pant~ "vida_l~ "terre~ AC;AM~ AR;BO;BR;B~ nativa
# #> 2 Arrenurus tumulo~ NA         Arrenurus (In~ NA           Animal~ Arthr~ Arac~ Trom~ Arren~ Arre~ ""       ""      NA     NA          NA
# #> 3 Mazama jucunda    NA         Mazama jucund~ NA           Animal~ Chord~ Mamm~ Arti~ Cervi~ Maza~ "herbiv~ "terre~ BA;ES~ BR          nativa
# #> 4 Zonotrichia cape~ NA         Zonotrichia c~ NA           Animal~ Chord~ Aves  Pass~ Passe~ Zono~ ""       ""      NA     NA          nativa
# # i 5 more variables: nomenclaturalStatus <chr>, vernacularName <chr>, taxonRank <chr>, id <int>, language <chr>

## -----------------------------------------------------------------------------
# my_sp2 <- subset_fauna(data = bf, species = valids,
#                       include_subspecies = TRUE)
# tibble::tibble(my_sp2) #print data.frame as tibble
# #> # A tibble: 14 × 19
# #>    species      subspecies scientificName validName kingdom phylum class order family genus lifeForm habitat states countryCode #> origin
# #>    <chr>        <chr>      <chr>          <chr>        <chr>   <chr>  <chr> <chr> <chr>  <chr> <chr>    <chr>   <chr>  <chr>       #> <chr>
# #>  1 Panthera on… ""         Panthera onca… ""           Animal… Chord… Mamm… Carn… Felid… Pant… "free_l… "terre… AC;AM… argentina;… #> "nati…
# #>  2 Arrenurus (… ""         Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      "" #>
# #>  3 Arrenurus (… "Arrenuru… Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      "" #>
# #>  4 Arrenurus (… "Arrenuru… Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      "" #>
# #>  5 Mazama jucu… ""         Mazama jucund… ""           Animal… Chord… Mamm… Arti… Cervi… Maza… "herbiv… "terre… BA;ES… brazil      #> "nati…
# #>  6 Zonotrichia… ""         Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AL;AP… brazil      #> "nati…
# #>  7 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AP     brazil      #> "nati…
# #>  8 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      RR     brazil      #> "nati…
# #>  9 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      RR     brazil      #> "nati…
# #> 10 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      BA;MA… brazil      #> "nati…
# #> 11 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      PA     brazil      #> "nati…
# #> 12 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AP;PA… brazil      #> "nati…
# #> 13 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      ES;MG… brazil      #> "nati…
# #> 14 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      TO     brazil      #> "nati…
# #> # ℹ 4 more variables: taxonomicStatus <chr>, nomenclaturalStatus <chr>, vernacularName <chr>, taxonRank <chr>
# #>

## -----------------------------------------------------------------------------
# spp <- c("Panthera onca", "Mazama jucunda", "Subulo gouzoubira")
# spp_syn <- fauna_synonym(data = bf, species = spp)
# spp_syn
# #>               validName               synonym   taxonomicStatus
# #> 49343     Panthera onca                  <NA>    valido
# #> 60523    Mazama jucunda         Mazama bororo    sinonimo
# #> 61168 Subulo gouzoubira    Mazama gouazoubira    sinonimo

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message  = FALSE,
  eval = FALSE,
  fig.width = 8,
  fig.height = 6
)

## ----Get data, results='hide', message=FALSE, warning=FALSE, echo=FALSE-------
# my_dir <- file.path(file.path(tempdir(), "faunabr"))
# dir.create(my_dir)
# get_faunabr(output_dir = my_dir, #directory to save the data
#             data_version = "latest", #get the most recent version available
#             overwrite = T) #Overwrite data, if it exists

## ----message=FALSE, warning=F-------------------------------------------------
# library(faunabr)
# library(terra)
# #Folder where you stored the data with the function get_faunabr()
# #Load data
# bf <- load_faunabr(data_dir = my_dir,
#                    data_version = "latest",
#                    type = "short") #short version
# #> Loading version 1.3

## -----------------------------------------------------------------------------
# #Select native species of mammals with confirmed occurrence in Brazil
# br_mammals <- select_fauna(data = bf,
#                            include_subspecies = FALSE, phylum = "all",
#                            class = "Mammalia",
#                            order = "all", family = "all",
#                            genus = "all",
#                            lifeForm = "all", filter_lifeForm = "in",
#                            habitat = "all", filter_habitat = "in",
#                            states = "all", filter_states = "in",
#                            country = "BR", filter_country = "in",
#                            origin = "all", taxonomicStatus = "valid")
# 
# #Get presence-absence matrix in states and countries
# pam_mammals <- fauna_pam(data = br_mammals, by_state = TRUE,
#                          by_country = FALSE,
#                          remove_empty_sites = TRUE,
#                          return_richness_summary = TRUE,
#                          return_spatial_richness = TRUE,
#                          return_plot = TRUE)
# 
# #Visualize (as tibble) the PAM for the first 5 species and 7 sites
# tibble::tibble(pam_mammals$PAM[1:7, 1:5])
# #> # A tibble: 7 × 5
# #>   states `Platyrrhinus aurarius` `Kannabateomys amblyonyx` `Callicebus lucifer` `Cerradomys maracajuensis`
# #>   <fct>                    <dbl>                     <dbl>                <dbl>                      <dbl>
# #> 1 AM                           1                         0                    1                          0
# #> 2 ES                           0                         1                    0                          0
# #> 3 MG                           0                         1                    0                          1
# #> 4 PR                           0                         1                    0                          0
# #> 5 RJ                           0                         1                    0                          0
# #> 6 RS                           0                         1                    0                          0
# #> 7 SC                           0                         1                    0                          0

## -----------------------------------------------------------------------------
# #Visualize (as tibble) the richness summary table
# tibble::tibble(pam_mammals$Richness_summary[1:7,])
# #> # A tibble: 7 × 3
# #> states   richness
# #>   <fct>     <dbl>
# #> 1 AM          225
# #> 2 ES          120
# #> 3 MG          188
# #> 4 PR          116
# #> 5 RJ          133
# #> 6 RS          105
# #> 7 SC          101

## ----IMG09, eval=TRUE, echo = FALSE-------------------------------------------
knitr::include_graphics("vignettes_img/IMG05.png")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message  = FALSE,
  eval = FALSE,
  fig.width = 8,
  fig.height = 6
)

## -----------------------------------------------------------------------------
# library(faunabr)
# library(terra)
# #Carregar dados
# bf <- load_faunabr(data_dir = my_dir, #Pasta onde foi salvo o arquivo com get_faunabr()
#                    data_version = "latest", #Versão mais recente
#                    type = "short") #Versão resumida
# #> Loading version 1.3

## -----------------------------------------------------------------------------
# #Espécies de exemplo
# spp <- c("Panthera onca", "Mazama jucunda")
# #Obter polígonos espaciais
# spp_spt <- fauna_spat_occ(data = bf, species = spp, state = TRUE,
#                           country = TRUE, verbose = TRUE)
# #> Getting states of Panthera onca
# #> Getting countries of Panthera onca
# #> Getting states of Mazama jucunda
# #> Getting countries of Mazama jucunda

## -----------------------------------------------------------------------------
# par(mfrow = c(2, 2), mar = c(2, 0, 2, 0))
# plot(spp_spt$`Panthera onca`$states,
#      main = paste0(names(spp_spt)[[1]], " - States"), mar = NA)
# plot(spp_spt$`Panthera onca`$countries,
#      main = paste0(names(spp_spt)[[1]], " - Countries"), mar = NA)
# plot(spp_spt$`Mazama jucunda`$states,
#      main = paste0(names(spp_spt)[[2]], " - States"), mar = NA)
# plot(spp_spt$`Mazama jucunda`$countries,
#      main = paste0(names(spp_spt)[[2]], " - Countries"), mar = NA)

## ----IMG01, eval=TRUE, echo=FALSE---------------------------------------------
knitr::include_graphics("vignettes_img/IMG01.png")

## -----------------------------------------------------------------------------
# my_spp <- c("Panthera onca", "Chaetomys subspinosus")
# pol_spp <- fauna_spat_occ(data = bf, species = my_spp,
#                        state = TRUE, country = TRUE,
#                        verbose = TRUE)
# par(mfrow = c(2, 2), mar = c(2, 0, 2, 0))
# plot(pol_spp$`Panthera onca`$states,
#      main = paste0(names(pol_spp)[[1]], " - States"), mar = NA)
# plot(pol_spp$`Panthera onca`$countries,
#      main = paste0(names(pol_spp)[[1]], " - Countries"), mar = NA)
# plot(pol_spp$`Chaetomys subspinosus`$states,
#      main = paste0(names(pol_spp)[[2]], " - States"), mar = NA)
# plot(pol_spp$`Chaetomys subspinosus`$countries,
#      main = paste0(names(pol_spp)[[2]], " - Countries"), mar = NA)

## ----IMG02, eval=TRUE, echo = FALSE-------------------------------------------
knitr::include_graphics("vignettes_img/IMG02.png")

## -----------------------------------------------------------------------------
# data("occurrences")
# head(occurrences)
# #>         species         x          y  source
# #> 1 Panthera onca -90.38409  17.377023    gbif
# #> 2 Panthera onca -90.24368  17.240507    gbif
# #> 3 Panthera onca -77.36680   0.287624    gbif
# #> 4 Panthera onca -56.61023 -17.239688    gbif
# #> 5 Panthera onca -61.04386  -2.387029    gbif
# #> 6 Panthera onca -77.28850   0.288757    gbif

## -----------------------------------------------------------------------------
# occ_check <- filter_faunabr(data = bf, occ = occurrences,
#                     by_state = TRUE, buffer_state = 20,
#                     by_country = TRUE, buffer_country = 20,
#                     value = "flag&clean", keep_columns = TRUE,
#                     verbose = FALSE)
# #> Returning list with flagged and cleaned occurrences

## ----warning=FALSE, results='hide'--------------------------------------------
# #Instala pacote (se necessário)
# if(!require(mapview)){
#     install.packages("mapview")
# }
# #Carregar pacote
# library(mapview)
# library(terra)

## ----fig.width= 9-------------------------------------------------------------
# #Converter pontos para spatvector
# panthera_occ <- subset(occ_check$flagged,
#                        occ_check$flagged$species == "Panthera onca")
# panthera_occ <- vect(panthera_occ, geom = c("x", "y"),
#                      crs = crs(pol_spp$`Panthera onca`$states))
# 
# #Plot interativo
# mapview(pol_spp$`Panthera onca`$states,
#         layer.name = "States with confirmed occurrences") +
#   mapview(panthera_occ, zcol = "inside_state", layer.name = "Inside state?",
#           col.regions = c("red", "green"))

## ----IMG03, eval=TRUE, echo = FALSE, fig.pos="H", out.width="90%"-------------
knitr::include_graphics("vignettes_img/IMG03.png")

## ----fig.width= 9-------------------------------------------------------------
# #Plot interativo
# mapview(pol_spp$`Panthera onca`$countries,
#         layer.name = "Countries with confirmed occurrences", burst = TRUE, legend = F) +
#   mapview(panthera_occ, zcol = "inside_country",
#           col.regions = c("red", "green"),
#           layer.name = "Inside country?")

## ----IMG04, eval=TRUE, echo = FALSE, fig.pos="H", out.width="90%"-------------
knitr::include_graphics("vignettes_img/IMG04.png")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = F,
  warning = FALSE
)

## ----results='hide', message=FALSE--------------------------------------------
# if(!require(remotes)){
#     install.packages("remotes")
# }
# 
# if(!require(faunabr)){
# remotes::install_github('wevertonbio/faunabr')}
# 
# library(faunabr)

## -----------------------------------------------------------------------------
# #Criando uma pasta em um diretório temporário
# #Substitua 'file.path(tempdir(), "faunabr")' por um caminho de pasta a ser
# #criada em seu computador
# my_dir <- file.path(file.path(tempdir(), "faunabr"))
# dir.create(my_dir)

## ----results='hide', message=FALSE, warning=FALSE-----------------------------
# get_faunabr(output_dir = my_dir, #Pasta para salvar os dados
#             data_version = "latest", #Obter a versão mais recente
#             translate = FALSE, #Manter versão original em português
#             overwrite = T) #Sobrescrever dados, se eles já existirem

## ----results='hide', message=FALSE, warning=FALSE-----------------------------
# get_faunabr(output_dir = my_dir, #Pasta para salvar os dados
#             data_version = "1.10", #Versão 1.10, publicada em 01-02-2024
#             translate = FALSE, #Manter versão original em português
#             overwrite = T) #Sobrescrever dados, se eles já existirem

## -----------------------------------------------------------------------------
# fauna_version(data_dir = my_dir)
# #> You have the following versions of Fauna do Brasil:
# #>  1.17
# #>  1.10
# #>  It includes the latest version:  1.17

## -----------------------------------------------------------------------------
# #Short version
# bf <- load_faunabr(data_dir = my_dir,
#                    data_version = "latest",
#                    type = "short") #resumida
# #> Loading version 393.401
# colnames(bf) #Ver colunas da versão resumida
# #>  [1] "species"             "subspecies"          "scientificName"
# #>  [4] "validName"           "kingdom"             "phylum"
# #>  [7] "class"               "order"               "family"
# #> [10] "genus"               "lifeForm"            "habitat"
# #> [13] "states"              "countryCode"         "origin"
# #> [16] "taxonomicStatus"     "nomenclaturalStatus" "vernacularName"
# #> [19] "taxonRank"           "id"                  "language"

## -----------------------------------------------------------------------------
# #Complete version
# bf_complete <- load_faunabr(data_dir = my_dir,
#                    data_version = "latest",
#                    type = "complete") #completa
# 
# colnames(bf_complete) #Ver colunas da versão completa
# #>  [1] "id"                       "taxonID"
# #>  [3] "species"                  "subspecies"
# #>  [5] "scientificName"           "validName"
# #>  [7] "validNameUsage"           "parentNameUsage"
# #>  [9] "namePublishedInYear"      "higherClassification"
# #> [11] "kingdom"                  "phylum"
# #> [13] "class"                    "order"
# #> [15] "family"                   "genus"
# #> [17] "specificEpithet"          "infraspecificEpithet"
# #> [19] "taxonRank"                "scientificNameAuthorship"
# #> [21] "taxonomicStatus"          "nomenclaturalStatus"
# #> [23] "vernacularName"           "lifeForm"
# #> [25] "habitat"                  "origin"
# #> [27] "states"                   "countryCode"
# #> [29] "modified"                 "bibliographicCitation"
# #> [31] "relationshipOfResource"   "language"

## -----------------------------------------------------------------------------
# # Traduzir do português para o inglês
# bf_ingles <- translate_faunabr(data = bf, to = "en")
# # Ver formas de vida (em ingles)
# fauna_attributes(bf_ingles, "lifeForm")
# #>  $lifeForm
# #>                  lifeForm
# #> 1                colonial
# #> 2               commensal
# #> 3            ectoparasite
# #> 4          ectoparasitoid
# #> 5            endoparasite
# #> 6          endoparasitoid
# #> 7                epibiont
# #> 8                eusocial
# #> 9  free_living_individual
# #> 10              herbivore
# #> 11        hyperparasitoid
# #> 12              inquiline
# #> 13            mutualistic
# #> 14              polynizer
# #> 15               predator
# #> 16                sessile
# 
# # Traduzir do inglês para o português
# bf_portugues <- translate_faunabr(data = bf_ingles, to = "pt_br")
# #Ver formas de vida disponíveis em portugues novamente
# fauna_attributes(bf_portugues, "lifeForm")
# #>  $lifeForm
# #>                 lifeForm
# #> 1               colonial
# #> 2               comensal
# #> 3           ectoparasito
# #> 4        ectoparasitoide
# #> 5           endoparasito
# #> 6        endoparasitoide
# #> 7              epibionte
# #> 8              eussocial
# #> 9              herbivoro
# #> 10      hiperparasitoide
# #> 11             inquilino
# #> 12                mutual
# #> 13           polinizador
# #> 14              predador
# #> 15                sessil
# #> 16 vida_livre_individual

## ----eval=FALSE, warning=FALSE, message=FALSE---------------------------------
# write.csv(x = bf,
#           file = file.path(my_dir, "BrazilianFauna.csv"),
#           row.names = F)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message  = FALSE,
  eval = FALSE,
  fig.width = 8,
  fig.height = 6
)

## -----------------------------------------------------------------------------
# library(faunabr)
# #Carregar dados
# bf <- load_faunabr(data_dir = my_dir, #Pasta onde foi salvo o arquivo com get_faunabr()
#                    data_version = "latest", #Versão mais recente
#                    type = "short") #Versão resumida
# #> Loading version 1.3

## -----------------------------------------------------------------------------
# #Selecionar espécies de mamíferos nativos do Brasil:
# br_mammals <- select_fauna(data = bf,
#                            include_subspecies = FALSE, phylum = "all",
#                            class = "Mammalia",
#                            order = "all", family = "all",
#                            genus = "all",
#                            lifeForm = "all", filter_lifeForm = "in",
#                            habitat = "all", filter_habitat = "in",
#                            states = "all", filter_states = "in",
#                            country = "BR", filter_country = "in",
#                            origin = "all", taxonomicStatus = "valido")
# #PAM em Estados do Brasil
# pam_mammals <- fauna_pam(data = br_mammals, by_state = TRUE,
#                          by_country = FALSE,
#                          remove_empty_sites = TRUE,
#                          return_richness_summary = TRUE,
#                          return_spatial_richness = TRUE,
#                          return_plot = TRUE)
# #Visualizar (em tibble) PAM com as primeiras 5 espécies nos primeiros 5 sites
# tibble::tibble(pam_mammals$PAM[1:7, 1:5])
# #> # A tibble: 7 × 5
# #>   states `Platyrrhinus aurarius` `Kannabateomys amblyonyx` `Callicebus lucifer` `Cerradomys maracajuensis`
# #>   <fct>                    <dbl>                     <dbl>                <dbl>                      <dbl>
# #> 1 AM                           1                         0                    1                          0
# #> 2 ES                           0                         1                    0                          0
# #> 3 MG                           0                         1                    0                          1
# #> 4 PR                           0                         1                    0                          0
# #> 5 RJ                           0                         1                    0                          0
# #> 6 RS                           0                         1                    0                          0
# #> 7 SC                           0                         1                    0                          0

## -----------------------------------------------------------------------------
# #Visualise (em tibble) a tabela com a riqueza por site
# tibble::tibble(pam_mammals$Richness_summary[1:7,])
# #> # A tibble: 7 × 3
# #> states   richness
# #>   <fct>     <dbl>
# #> 1 AM          225
# #> 2 ES          120
# #> 3 MG          188
# #> 4 PR          116
# #> 5 RJ          133
# #> 6 RS          105
# #> 7 SC          101

## ----IMG09, eval=TRUE, echo = FALSE-------------------------------------------
knitr::include_graphics("vignettes_img/IMG05.png")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  warning = FALSE
)

## -----------------------------------------------------------------------------
# library(faunabr)
# #Folder where you stored the data with the function get_faunabr()
# #Load data
# bf <- load_faunabr(data_dir = my_dir,
#                    data_version = "latest",
#                    type = "short") #short version
# #> Loading version 1.3

## -----------------------------------------------------------------------------
# #Get available options to filter by lifeForm, habitat and origin
# fauna_at <- fauna_attributes(data = bf,
#                              attribute = c("lifeForm", "habitat", "origin"))
# head(fauna_at$lifeForm)
# #>          lifeForm
# #> 1        colonial
# #> 2       commensal
# #> 3    ectoparasite
# #> 4 ectoparasiteIDE
# #> 5    endoparasite
# #> 6  endoparasiteid
# 
# head(fauna_at$habitat)
# #>         habitat
# #> 1      arboreal
# #> 2 cavernicolous
# #> 3     fossorial
# #> 4    freshwater
# #> 5     hyporheic
# #> 6        marine
# 
# head(fauna_at$origin)
# #>         origin
# #> 1  cryptogenic
# #> 2 domesticated
# #> 3   introduced
# #> 4     invasive
# #> 5       native

## -----------------------------------------------------------------------------
# insects_rj <- select_fauna(data = bf, include_subspecies = FALSE,
#                            phylum = "all",
#                            class = "Insecta", #Select insecs
#                            order = "all",
#                            family = "all",
#                            genus = "all",
#                            lifeForm = "all", filter_lifeForm = "in",
#                            habitat = "arboreal", filter_habitat = "in",
#                            states = "RJ", #Rio de Janeiro
#                            filter_states = "in", #Select IN Rio de Janeiro
#                            country = "all", filter_country = "in",
#                            origin = "native",
#                            taxonomicStatus = "valid")
# nrow(insects_rj)
# #> [1] 114

## -----------------------------------------------------------------------------
# #First 7 unique values of states in the filtered dataset
# unique(insects_rj$states)[1:7]
# #> [1] "AC;AM;AP;BA;CE;DF;ES;GO;MA;MG;MS;MT;PA;PE;RJ;RN;RO;RR;SE;SP;TO"
# #> [2] "BA;MA;MG;MT;PR;RJ;RS;SC;SP"
# #> [3] "ES;MG;PE;PR;RJ;RS;SP"
# #> [4] "ES;PR;RJ;SP"
# #> [5] "RJ"
# #> [6] "RJ;SP"
# #> [7] "ES;MG;RJ;RS;SC;SP"

## -----------------------------------------------------------------------------
# insects_rj_only <- select_fauna(data = bf, include_subspecies = FALSE,
#                            phylum = "all",
#                            class = "Insecta", #Select insecs
#                            order = "all",
#                            family = "all",
#                            genus = "all",
#                            lifeForm = "all", filter_lifeForm = "in",
#                            habitat = "arboreal", filter_habitat = "in",
#                            states = "RJ", #Rio de Janeiro
#                            filter_states = "only", #Select ONLY in Rio de Janeiro
#                            country = "all", filter_country = "in",
#                            origin = "native",
#                            taxonomicStatus = "valid")
# nrow(insects_rj_only)
# #> [1] 22
# unique(insects_rj_only$states)
# #> [1] "RJ"

## -----------------------------------------------------------------------------
# insects_south <- select_fauna(data = bf, include_subspecies = FALSE,
#                            phylum = "all",
#                            class = "Insecta", #Select insecs
#                            order = "all",
#                            family = "all",
#                            genus = "all",
#                            lifeForm = "all", filter_lifeForm = "in",
#                            habitat = "all", filter_habitat = "in",
#                            states = c("PR", "SC", "RS"), #States from southern Brazil
#                            filter_states = "in", #IN any of these states
#                            country = "all", filter_country = "in",
#                            origin = "native",
#                            taxonomicStatus = "valid")
# nrow(insects_south)
# #> [1] 11461
# 
# #First 10 unique values of states in the filtered dataset
# unique(insects_south$states)[1:10]
# #> [1] "PR;SP"
# #> [2] "BA;MT;RJ;RS"
# #> [3] "RS"
# #> [4] "PR"
# #> [5] "PR;RS"
# #> [6] "GO;PA;RO;RR;SC"
# #> [7] "MG;PR;SP"
# #> [8] "GO;MG;PR;RS;SC"
# #> [9] "GO;RJ;SC"
# #> [10] "AC;AL;AM;AP;BA;CE;DF;ES;GO;MA;MG;MS;MT;PA;PB;PE;PI;PR;RJ;RN;RO;RR;RS;SC;SE;SP;TO"

## -----------------------------------------------------------------------------
# insects_south_and <- select_fauna(data = bf, include_subspecies = FALSE,
#                            phylum = "all",
#                            class = "Insecta", #Select insecs
#                            order = "all",
#                            family = "all",
#                            genus = "all",
#                            lifeForm = "all", filter_lifeForm = "in",
#                            habitat = "all", filter_habitat = "in",
#                            states = c("PR", "SC", "RS"), #States from southern Brazil
#                            filter_states = "and", #in PR AND SC AND RS
#                            country = "all", filter_country = "in",
#                            origin = "native",
#                            taxonomicStatus = "valid")
# nrow(insects_south_and)
# #> [1] 1924
# 
# #First 10 unique values of states in the filtered dataset
# unique(insects_south_and$states)[1:10]
# #> [1] "GO;MG;PR;RS;SC"
# #> [2] "AC;AL;AM;AP;BA;CE;DF;ES;GO;MA;MG;MS;MT;PA;PB;PE;PI;PR;RJ;RN;RO;RR;RS;SC;SE;SP;TO"
# #> [3] "AC;AM;MG;MS;MT;PA;PE;PR;RO;RS;SC;SP"
# #> [4] "AM;BA;CE;DF;ES;GO;MG;MS;MT;PA;PE;PR;RJ;RS;SC;SP"
# #> [5] "AC;AM;AP;BA;CE;DF;ES;GO;MG;MS;MT;PA;PE;PR;RJ;RO;RR;RS;SC;SE;SP;TO"
# #> [6] "BA;ES;MG;PR;RJ;RS;SC;SP"
# #> [7] "AM;DF;ES;GO;PR;RJ;RR;RS;SC;SP"
# #> [8] "ES;GO;MG;PR;RJ;RS;SC;SP"
# #> [9] "AC;AM;AP;BA;CE;ES;GO;MA;MG;MT;PA;PR;RJ;RO;RS;SC;SP;TO"
# #> [10] "GO;MG;MS;PR;RO;RS;SC;SP"

## -----------------------------------------------------------------------------
# insects_south_only <- select_fauna(data = bf, include_subspecies = FALSE,
#                            phylum = "all",
#                            class = "Insecta", #Select insecs
#                            order = "all",
#                            family = "all",
#                            genus = "all",
#                            lifeForm = "all", filter_lifeForm = "in",
#                            habitat = "all", filter_habitat = "in",
#                            states = c("PR", "SC", "RS"), #States from southern Brazil
#                            filter_states = "only", #ONLY in PR, SC and RS
#                            country = "all", filter_country = "in",
#                            origin = "native",
#                            taxonomicStatus = "valid")
# nrow(insects_south_only)
# #> [1] 134
# 
# #The unique state in the filtered dataset
# unique(insects_south_only$states)
# #> [1] "PR;RS;SC"

## -----------------------------------------------------------------------------
# complete_names <- c("Pantera onça (Linnaeus, 1758)",
#                     "Zonotrichia capensis subtorquata Swainson, 1837",
#                     "Mazama bororo Duarte, 1996",
#                     "Mazama jucunda Thomas, 1913",
#                     "Arrenurus tumulosus intercursor",
#                     "Araucaria angustifolia")
# #Panthera onca with typos to illustrate how the next function corrects it
# #Araucaria angustifolia (a Plant) was used just as an example that will be used to illustrate the
# #next function
# binomial_names <- extract_binomial(species_names = complete_names)
# binomial_names
# #> [1] "Pantera onça"                         "Zonotrichia capensis"
# #> [3] "Mazama bororo"                        "Mazama jucunda"
# #> [5] "Arrenurus tumulosus"                  "Araucaria angustifolia"

## -----------------------------------------------------------------------------
# #Create example
# checked_names <- check_fauna_names(data = bf,
#                              species = binomial_names,
#                              include_subspecies = FALSE,
#                              max_distance = 0.1)
# tibble::tibble(checked_names) #print data.frame as tibble
# #> # A tibble: 6 × 8
# #>   input_name           Spelling Suggested_name Distance taxonomicStatus validName #> family
# #>   <chr>                <chr>    <chr>             <dbl> <chr>           <chr>        #> <chr>
# #> 1 Zonotrichia capensis Correct  Zonotrichia c…        0 valid   Zonotrichia… #> Passe…
# #> 2 Mazama bororo        Correct  Mazama bororo         0 synonym         Mazama jucu… #> Cervi…
# #> 3 Mazama jucunda       Correct  Mazama jucunda        0 valid   Mazama jucu… #> Cervi…
# #> 4 Arrenurus (Incertae… Correct  Arrenurus tum…        0 valid   Arrenurus t… #> Arren…
# #> 5 Pantera onça         Probabl… Panthera onca         2 valid   Panthera on… #> Felid…
# #> 6 Araucaria angustifo… Not_fou… NA                   NA NA              NA           NA
# 

## -----------------------------------------------------------------------------
# #Get only valid names
# valids <- unique(checked_names$validName)
# valids <- na.omit(valids) #Remove NA
# 
# #Subset species
# my_sp <- subset_fauna(data = bf, species = valids,
#                       include_subspecies = FALSE)
# tibble::tibble(my_sp) #print data.frame as tibble
# #> # A tibble: 4 × 19
# #>   species       subspecies scientificName validName kingdom phylum class order family genus lifeForm habitat states countryCode #> origin
# #>   <chr>         <chr>      <chr>          <chr>        <chr>   <chr>  <chr> <chr> <chr>  <chr> <chr>    <chr>   <chr>  <chr>       #> <chr>
# #> 1 Panthera onca ""         Panthera onca… ""           Animal… Chord… Mamm… Carn… Felid… Pant… "free_l… "terre… AC;AM… argentina;… #> "nati…
# #> 2 Arrenurus (I… ""         Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      ""
# #> 3 Mazama jucun… ""         Mazama jucund… ""           Animal… Chord… Mamm… Arti… Cervi… Maza… "herbiv… "terre… BA;ES… brazil      #> "nati…
# #> 4 Zonotrichia … ""         Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AL;AP… brazil      #> "nati…
# #> # ℹ 4 more variables: taxonomicStatus <chr>, nomenclaturalStatus <chr>, vernacularName <chr>, taxonRank <chr>

## -----------------------------------------------------------------------------
# my_sp2 <- subset_fauna(data = bf, species = valids,
#                       include_subspecies = TRUE)
# tibble::tibble(my_sp2) #print data.frame as tibble
# #> # A tibble: 14 × 19
# #>    species      subspecies scientificName validName kingdom phylum class order family genus lifeForm habitat states countryCode #> origin
# #>    <chr>        <chr>      <chr>          <chr>        <chr>   <chr>  <chr> <chr> <chr>  <chr> <chr>    <chr>   <chr>  <chr>       #> <chr>
# #>  1 Panthera on… ""         Panthera onca… ""           Animal… Chord… Mamm… Carn… Felid… Pant… "free_l… "terre… AC;AM… argentina;… #> "nati…
# #>  2 Arrenurus (… ""         Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      "" #>
# #>  3 Arrenurus (… "Arrenuru… Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      "" #>
# #>  4 Arrenurus (… "Arrenuru… Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      "" #>
# #>  5 Mazama jucu… ""         Mazama jucund… ""           Animal… Chord… Mamm… Arti… Cervi… Maza… "herbiv… "terre… BA;ES… brazil      #> "nati…
# #>  6 Zonotrichia… ""         Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AL;AP… brazil      #> "nati…
# #>  7 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AP     brazil      #> "nati…
# #>  8 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      RR     brazil      #> "nati…
# #>  9 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      RR     brazil      #> "nati…
# #> 10 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      BA;MA… brazil      #> "nati…
# #> 11 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      PA     brazil      #> "nati…
# #> 12 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AP;PA… brazil      #> "nati…
# #> 13 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      ES;MG… brazil      #> "nati…
# #> 14 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      TO     brazil      #> "nati…
# #> # ℹ 4 more variables: taxonomicStatus <chr>, nomenclaturalStatus <chr>, vernacularName <chr>, taxonRank <chr>
# #>

## -----------------------------------------------------------------------------
# spp <- c("Panthera onca", "Mazama jucunda", "Subulo gouzoubira")
# spp_syn <- fauna_synonym(data = bf, species = spp)
# spp_syn
# #>           validName               synonym        taxonomicStatus
# #> 49343     Panthera onca                  <NA>             valid
# #> 60523    Mazama jucunda         Mazama bororo             synonym
# #> 61168 Subulo gouzoubira    Mazama gouazoubira             synonym

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = F,
  warning = FALSE
)

## ----results='hide', message=FALSE--------------------------------------------
# if(!require(remotes)){
#     install.packages("remotes")
# }
# 
# if(!require(faunabr)){
# remotes::install_github('wevertonbio/faunabr')}
# 
# library(faunabr)

## -----------------------------------------------------------------------------
# #Creating a folder in a temporary directory
# #Replace 'file.path(tempdir(), "faunabr")' by a path folder to be create in
# #your computer
# my_dir <- file.path(file.path(tempdir(), "faunabr"))
# dir.create(my_dir)

## ----results='hide', message=FALSE, warning=FALSE-----------------------------
# get_faunabr(output_dir = my_dir, #directory to save the data
#             data_version = "latest", #get the most recent version available
#             translate = TRUE, #translate data to English
#             overwrite = T) #Overwrite data, if it exists

## ----results='hide', message=FALSE, warning=FALSE-----------------------------
# get_faunabr(output_dir = my_dir, #directory to save the data
#             data_version = "1.10", #Version 1.10, published on 2024-02-01
#             overwrite = T) #Overwrite data, if it exists

## -----------------------------------------------------------------------------
# fauna_version(data_dir = my_dir)
# #> You have the following versions of Fauna do Brasil:
# #>  1.17
# #>  1.10
# #>  It includes the latest version:  1.17

## -----------------------------------------------------------------------------
# #Short version
# bf <- load_faunabr(data_dir = my_dir,
#                    data_version = "latest",
#                    type = "short") #short
# #> Loading version 393.401
# colnames(bf) #See variables from short version
# #>  [1] "species"             "subspecies"          "scientificName"
# #>  [4] "validName"           "kingdom"             "phylum"
# #>  [7] "class"               "order"               "family"
# #> [10] "genus"               "lifeForm"            "habitat"
# #> [13] "states"              "countryCode"         "origin"
# #> [16] "taxonomicStatus"     "nomenclaturalStatus" "vernacularName"
# #> [19] "taxonRank"           "id"                  "language"

## -----------------------------------------------------------------------------
# #Complete version
# bf_complete <- load_faunabr(data_dir = my_dir,
#                    data_version = "latest",
#                    type = "complete") #complete
# 
# colnames(bf_complete) #See variables from complete version
# #>  [1] "id"                       "taxonID"
# #>  [3] "species"                  "subspecies"
# #>  [5] "scientificName"           "validName"
# #>  [7] "validNameUsage"           "parentNameUsage"
# #>  [9] "namePublishedInYear"      "higherClassification"
# #> [11] "kingdom"                  "phylum"
# #> [13] "class"                    "order"
# #> [15] "family"                   "genus"
# #> [17] "specificEpithet"          "infraspecificEpithet"
# #> [19] "taxonRank"                "scientificNameAuthorship"
# #> [21] "taxonomicStatus"          "nomenclaturalStatus"
# #> [23] "vernacularName"           "lifeForm"
# #> [25] "habitat"                  "origin"
# #> [27] "states"                   "countryCode"
# #> [29] "modified"                 "bibliographicCitation"
# #> [31] "relationshipOfResource"   "language"

## -----------------------------------------------------------------------------
# #Translate from English to Portuguese
# bf_portuguese <- translate_faunabr(data = bf, to = "pt_br")
# # See lifeforms (in portuguese)
# fauna_attributes(bf_portuguese, "lifeForm")
# #>  $lifeForm
# #>                 lifeForm
# #> 1               colonial
# #> 2               comensal
# #> 3           ectoparasito
# #> 4        ectoparasitoide
# #> 5           endoparasito
# #> 6        endoparasitoide
# #> 7              epibionte
# #> 8              eussocial
# #> 9              herbivoro
# #> 10      hiperparasitoide
# #> 11             inquilino
# #> 12                mutual
# #> 13           polinizador
# #> 14              predador
# #> 15                sessil
# #> 16 vida_livre_individual
# 
# #Translate from Portuguese to English
# bf_english <- translate_faunabr(data = fauna_portuguese, to = "en")
# #See life forms(in English)
# fauna_attributes(bf_english, "lifeForm")
# #>  $lifeForm
# #>                  lifeForm
# #> 1                colonial
# #> 2               commensal
# #> 3            ectoparasite
# #> 4          ectoparasitoid
# #> 5            endoparasite
# #> 6          endoparasitoid
# #> 7                epibiont
# #> 8                eusocial
# #> 9  free_living_individual
# #> 10              herbivore
# #> 11        hyperparasitoid
# #> 12              inquiline
# #> 13            mutualistic
# #> 14              polynizer
# #> 15               predator
# #> 16                sessile
# 

## ----eval=FALSE, warning=FALSE, message=FALSE---------------------------------
# write.csv(x = bf,
#           file = file.path(my_dir, "BrazilianFauna.csv"),
#           row.names = F)

#' Extract a subset of species from Fauna do Brasil database
#' @description
#' Returns a data.frame with a subset of species from Fauna do Brasil database
#'
#' @param data (data.frame) the data.frame imported with the
#' \code{\link{load_faunabr}} function.
#' @param species (character) names of the species to be extracted from Fauna do
#' Brasil database.
#' @param include_subspecies (logical) include subspecies? Default = FALSE
#'
#' @return A data.frame with the selected species.
#' @usage subset_fauna(data, species, include_subspecies = FALSE)
#' @export
#'
#' @examples
#' data("fauna_data") #Load data example
#' #Species to extract from database
#' spp <- c("Panthera onca", "Mazama jucunda", "Subulo gouzoubira")
#' spp_subset <- subset_fauna(data = fauna_data, species = spp,
#'                       include_subspecies = FALSE)
#' spp_subset
subset_fauna <- function(data,
                         species, include_subspecies = FALSE){
  if (missing(data)) {
    stop("Argument data is not defined")
  }

  #Check classes
  if (!is.data.frame(data)) {
    stop(paste0("Argument data must be a data.frame, not ", class(data)))
  }

  if (!is.character(species)) {
    stop(paste0("Argument species must be a character, not ", class(species)))
  }

  if (!is.logical(include_subspecies)) {
    stop(paste0("Argument include_subspecies must be logical, not ",
                class(include_subspecies)))
  }


  #Get binomial names of species
  Species <- extract_binomial(species)

  #Start to filter...
  #Taxon Rank
  #Taxon Rank
  if(include_subspecies){
    ss <- c("subspecies", "sub_especie")
  } else {ss <- NULL}

  tr <- c("species", "especie", ss)

  data <- data[data$taxonRank %in% tr,]

  #Check if there is any species absent in d
  no_match <- setdiff(Species, unique(data$species))

  if(length(no_match) == length(Species)){
    stop("All species are absent of Fauna do Brasil database.
         Check the species names using the check_names() function")
  }

  if(length(no_match) > 0) {
    warning(paste("Some species are absent of Fauna do Brasil database\n",
                  "Check the species names using the check_names() function"))
  }

  return(subset(data, data$species %in% Species))

}

#' Get Spatial polygons (SpatVectors) of species based on its distribution
#' (states and countrys) according to Fauna do Brasil
#'
#' @param data (data.frame) the data.frame imported with the
#' \code{\link{load_faunabr}} function.
#' @param species (character) one or more species names (only genus and
#' specific epithet, eg. "Panthera onca")
#' @param state (logical) get SpatVector of states with occurrence of the
#' species? Default = TRUE
#' @param country (logical) get SpatVector of countrys with occurrence of the
#' species? Default = TRUE
#' @param spat_state (SpatVector) a SpatVector of the Brazilian states. By
#' default, it uses the SpatVector provided by geobr::read_state(). It can be
#' another Spatvector, but the structure must be identical to
#' 'faunabr::states', with a column called "abbrev_state" identifying the states
#' codes.
#' @param spat_country (SpatVector) a SpatVector of the world countries. By
#' default, it uses the SpatVector provided by rnaturalearth::ne_countries. It
#' can be another Spatvector, but the structure must be identical to
#' 'faunabr::world_fauna', with a column called "country_code" identifying the
#' country codes.
#' @param verbose (logical) Whether to display species being filtered during
#' function execution. Set to TRUE to enable display, or FALSE to run silently.
#' Default = TRUE.
#'
#' @return A list with SpatVectors of states and/or countrys for each
#' specie.
#' @importFrom terra subset unwrap intersect mask
#' @importFrom data.table rbindlist
#' @importFrom stats na.omit
#' @usage fauna_spat_occ(data, species, state = TRUE, country = TRUE,
#'                       spat_state = NULL, spat_country = NULL, verbose = TRUE)
#' @export
#' @examples
#' library(terra)
#' data("fauna_data")
#' spp <- c("Panthera onca", "Mazama jucunda")
#' #Get states, countrys and intersection states-countrys of species
#' spp_spt <- fauna_spat_occ(data = fauna_data, species = spp, state = TRUE,
#'                           country = TRUE, verbose = TRUE)
#' #Plot states with confirmed occurrence of Panthera onca and Mazama jucunda
#' plot(spp_spt$`Panthera onca`$states)
#' plot(spp_spt$`Mazama jucunda`$states)
#' #Plot countries with confirmed occurrence of Panthera onca and Mazama jucunda
#' plot(spp_spt$`Panthera onca`$countries)
#' plot(spp_spt$`Mazama jucunda`$countries)
#'
fauna_spat_occ <- function(data, species, state = TRUE,
                           country = TRUE,
                           spat_state = NULL,
                           spat_country = NULL,
                           verbose = TRUE) {
  if (missing(data)) {
    stop("Argument data is not defined")
  }

  if (missing(species)) {
    stop("Argument occ is not defined")
  }

  if (!inherits(data, "data.frame")) {
    stop(paste0("Argument data must be a data.frame, not ", class(data)))
  }

  if (!is.character(species)) {
    stop(paste0("Argument species must be a character, not ", class(species)))
  }

  if (!is.logical(state)) {
    stop(paste0("Argument state must be logical, not ", class(state)))
  }

  if (!is.logical(country)) {
    stop(paste0("Argument country must be logical, not ", class(country)))
  }

  if (!is.logical(verbose)) {
    stop(paste0("Argument verbose must be logical, not ", class(verbose)))
  }

  #Check colnames in data
  if(!all(c("species", "states", "countryCode") %in%
          colnames(data))) {
    stop("Important columns are missing in data. Check if data is an object
         created by 'load_faunabr()")
  }

  #Check if there is at least one TRUE in states or countrys
  if(!state & !country){
    stop("At least one of the parameters state or country must be TRUE")
  }

  #Load data
  d <- data[,c("species", "states", "countryCode")]

  #Check if all species are in fauna do Brasil data
  spp <- extract_binomial(species_names = species)
  #Get binomial names of species
  spp_out <- setdiff(spp, unique(data$species))
  if(length(spp_out) > 0) {
    stop(paste(length(spp_out), "species are not in the data. Check the species
               names using the check_fauna_names() function"))
  }
  #Subset info
  d_info <- subset(d, d$species %in% spp)
  d_info[d_info == ""] <- NA

  #Get only one line by species, merging information of same species
  sp_info <- lapply(seq_along(spp), function(i) {
    sp <- subset(d_info, d_info$species == spp[i])
    sp$states <- paste0(stats::na.omit(unique(sp$states)),
                        collapse = ";")
    sp$countryCode <- paste0(stats::na.omit(unique(sp$countryCode)),
                       collapse = ";")
    return(sp)
  })
  sp_info <- unique(data.table::rbindlist(sp_info))

  #Load data
  if(state) {
    if(is.null(spat_state)){
    states <- terra::unwrap(faunabr::states)} else {
      states <- spat_state
    }
  }
  if(country) {
    if(is.null(spat_country)){
    countrys <- terra::unwrap(faunabr::world_fauna) } else {
    countrys <- spat_country
    }
  }

  #Get state and countries
  l_occ <- lapply(seq_along(spp), function(i){
    occ_i <- subset(sp_info, sp_info$species == spp[i])

    if(!state) {states_v <- NULL}

    if(state) {
      if(verbose) {
        message("Getting states of ", spp[i], "\n") }

      sp_i_state <- unique(gsub(";", "|", occ_i$states[1]))

      if(sp_i_state == "" | is.na(sp_i_state)) {
        if(verbose) {
          message(spp[i], "lacks info about state - SpatialVector not
                  returned")}
        states_v <- "No_info"
      } else {
        states_v <- terra::subset(states, grepl(sp_i_state,
                                                states$abbrev_state)) }
    }

    if(!country) {countrys_v <- NULL}

    if(country) {
      if(verbose) {
        message("Getting countries of ", spp[i], "\n") }
      sp_i_country <- unique(gsub(";", "|", occ_i$country[1]))

      if(sp_i_country == "" | is.na(sp_i_country)) {
        if(verbose){
          message(spp[i], "lacks info about country - SpatialVector not
                  returned")}
        countrys_v <- "No_info"
      } else {
        countrys_v <- terra::subset(countrys, grepl(sp_i_country,
                                                countrys$country_code)) }
    }

    #Save objects in a list
    final_list <- list(states_v, countrys_v)
    names(final_list) <- c("states", "countries")
    return(final_list)
  })
  names(l_occ) <- spp
  #Drop off null elements
  return(lapply(l_occ, function(x) x[lengths(x) > 0]))
} #End if function

# #Test function
# library(terra)
# data("fauna_data")
# spp <- c("Panthera onca", "Mazama jucunda")
# #Get states, countrys and intersection states-countrys of species
# spp_spt <- fauna_spat_occ(data = fauna_data, species = spp, state = TRUE,
#                           country = TRUE, state_and_country = TRUE,
#                           verbose = TRUE)
# #Plot states with confirmed occurrence of Panthera onca and Mazama jucunda
# plot(spp_spt$`Panthera onca`$states)
# plot(spp_spt$`Mazama jucunda`$states)
# #Plot countries with confirmed occurrence of Panthera onca and Mazama jucunda
# plot(spp_spt$`Panthera onca`$countrys)
# plot(spp_spt$`Mazama jucunda`$countrys)

#' Search for taxa using vernacular names
#'
#' @param data (data.frame) the data.frame imported with the
#' \code{\link{load_faunabr}} function or generated with the function
#' \code{\link{select_fauna}}.
#' @param names (character) vernacular name ("Nome comum") of the species to be
#' searched
#' @param exact (logic) if TRUE, the function will search only for exact
#' matches. For example, if names = "veado-mateiro" and exact = TRUE, the function
#' will return only the species popularly known as "veado-mateiro". On the other
#' hand, if names = "veado-mateiro" and exact = FALSE, the function will return
#' other results as "Veado-mateiro-pequeno". Default = FALSE
#'
#' @return a data.frame with the species with vernacular names that match the
#' input names
#' @export
#' @examples
#' data("fauna_data") #Load Fauna do Brasil data
#' #Search for species whose vernacular name is 'veado-mateiro'
#' veado_exact <- fauna_by_vernacular(data = fauna_data,
#'                                    names = "veado-mateiro",
#'                                    exact = TRUE)
#' veado_exact
#' #Search for species whose vernacular name is 'veado_mateiro', allowing non-exact
#' #matches
#' veado_not_exact <- fauna_by_vernacular(data = fauna_data,
#'                                        names = "veado-mateiro",
#'                                       exact = FALSE)
#'
fauna_by_vernacular <- function(data, names,
                                exact = FALSE){
  if (missing(data)) {
    stop("Argument data is not defined")
  }

  if (missing(names)) {
    stop("Argument names is not defined")
  }
  #Check classes
  if (!inherits(data, "data.frame")) {
    stop(paste0("Argument data must be a data.frame, not ", class(data)))
  }

  if (!is.character(names)) {
    stop(paste0("Argument names must be a character, not ", class(names)))
  }

  if (!is.logical(exact)) {
    stop(paste0("Argument exact must be logical, not ", class(exact)))
  }


  #trim and lower case
  names <- trimws(tolower(names))
  #trim and lower case
  new_names <- trimws(tolower(data$vernacularName))

  #If there is - in the pattern, use - in all names
  if(any(grepl("-", names))){
    new_names <- gsub("(?<=\\p{L}) (?=\\p{L})", "-", new_names, perl = TRUE)
  }

  #If there is space in the pattern, use space in all names
  if(any(grepl(" ", names))){
    new_names <- gsub("(?<=\\p{L})-(?=\\p{L})", " ", new_names, perl = TRUE)
  }

  #Subset
  if(!exact){
    vn_index <- grep(names, new_names, fixed = TRUE)
  } else {
    all_names <- strsplit(new_names, ", ")
    vn_index <- which(sapply(all_names, function(x) any(x == names)))
  }

  # has_word <- function(n) {
  #     palavras <- unlist(strsplit(n, ", "))
  #     any(grepl(paste0("^", names, "$"), palavras))
  #   }
  # dv <- dv[apply(dv, 1, function(x) has_word(x["vernacularName"])), ]


  #Subset original data
  dv <- data[vn_index,]

  if(nrow(dv) == 0) {
    stop(paste("There isn't any species in the Fauna do Brasil with this",
    "\nvernacular name"))
  }

  return(dv)
}

#fauna_by_vernacular(data = d, names = "pimenta", exact = T)

# #Test function
# data("fauna_data") #Load Fauna do Brasil data
# #Search for species whose vernacular name is 'veado-mateiro'
# veado_exact <- fauna_by_vernacular(data = fauna_data,
#                                     names = "veado-mateiro",
#                                     exact = TRUE)
# veado_exact
# #Search for species whose vernacular name is 'veado_mateiro', allowing non-exact
# #matches
# veado_not_exact <- fauna_by_vernacular(data = fauna_data,
#                                        names = "veado-mateiro",
#                                        exact = FALSE)

translate_column_terms <- function(text, map_df, to = "pt_br", sep = ";") {

  if (!is.character(text)) {
    warning("'text' must be a character, not ", class(text))
    text <- as.character(text)
  }

  # Define columns from and to translate
  if (to == "en") {
    source_col_name <- "pt_br"
    target_col_name <- "en"
  } else if (to == "pt_br") {
    source_col_name <- "en"
    target_col_name <- "pt_br"
  }

  # Function to translate
  process_single_string <- function(single_text) {
    if (is.na(single_text) || single_text == "") {
      return(single_text)
    }

    # Split string
    terms <- unlist(strsplit(single_text, split = sep, fixed = TRUE))

    # Set to lower case
    terms_lower <- tolower(terms)

    # Translate
    translated_terms <- map_df[[target_col_name]][match(terms_lower, map_df[[source_col_name]])]

    # Keep NA when is not possible to translate
    translated_terms[is.na(translated_terms)] <- terms[is.na(translated_terms)]

    # Paste terms and collapse with ;
    return(paste(translated_terms, collapse = sep))
  }

  # Apply function to each vector
  translated_vector <- sapply(text, process_single_string, USE.NAMES = FALSE)

  return(translated_vector)
}

# #### Create dataframes with translations ####
# mapa_traducao_lifeform <- data.frame(
#   pt_br = c(
#     "vida_livre_individual", "colonial", "sessil", "herbivoro",
#     "eussocial", "predador", "endoparasitoide", "ectoparasitoide",
#     "hiperparasitoide", "ectoparasito", "endoparasito", "comensal",
#     "epibionte", "polinizador", "inquilino", "mutual",
#     "endoparasiteid", "ectoparasiteid" # Erros comuns ou variações
#   ),
#   en = c(
#     "free_living_individual", "colonial", "sessile", "herbivore",
#     "eusocial", "predator", "endoparasitoid", "ectoparasitoid",
#     "hyperparasitoid", "ectoparasite", "endoparasite", "commensal",
#     "epibiont", "polynizer", "inquiline", "mutualistic",
#     "endoparasitoid", "ectoparasitoid" # Corrigindo para os termos corretos em inglês
#   ),
#   stringsAsFactors = FALSE # Importante para evitar que strings virem fatores
# )
#
# mapa_traducao_origins <- data.frame(
#   pt_br = c(
#     "nativa",
#     "introduzida",
#     "criptogenica",
#     "domesticada",
#     "invasora"
#   ),
#   en = c(
#     "native",
#     "introduced",
#     "cryptogenic",
#     "domesticated",
#     "invasive"
#   ),
#   stringsAsFactors = FALSE
# )
#
# # Mapa de Tradução para Habitat
# mapa_traducao_habitat <- data.frame(
#   pt_br = c(
#     "agua_doce",
#     "marinho",
#     "terrestre",
#     "arboreo",
#     "fossorial",
#     "cavernicola",
#     "aguas_subterraneas",
#     "nidicola",
#     "hiporreico"
#   ),
#   en = c(
#     "freshwater",
#     "marine",
#     "terrestrial",
#     "arboreal",
#     "fossorial",
#     "cavernicolous",
#     "subterranean waters",
#     "nidicolous",
#     "hyporheic"
#   ),
#   stringsAsFactors = FALSE
# )
#
# # Mapa de Tradução para Taxon Rank
# mapa_traducao_taxonrank <- data.frame(
#   pt_br = c(
#     "filo",
#     "classe",
#     "ordem",
#     "sub_ordem",
#     "familia",
#     "sub_familia",
#     "tribo",
#     "genero",
#     "sub_genero",
#     "especie",
#     "sub_especie",
#     "sub_classe",
#     "infra_ordem",
#     "super_familia",
#     "super_classe",
#     "sub_tribo",
#     "super_ordem",
#     "infra_classe",
#     NA # Para o caso de NA -> "unknown"
#   ),
#   en = c(
#     "phylum",
#     "class",
#     "order",
#     "suborder",
#     "family",
#     "subfamily",
#     "tribe",
#     "genus",
#     "subgenus",
#     "species",
#     "subspecies",
#     "subclass",
#     "infraorder",
#     "superfamily",
#     "superclass",
#     "subtribe",
#     "superorder",
#     "infraclass",
#     "unknown" # Mapeia NA para "unknown"
#   ),
#   stringsAsFactors = FALSE
# )
#
# mapa_traducao_taxonomicStatus <- data.frame(pt_br = c("valido", "sinonimo"),
#                                             en = c("valid", "synonym")
#                                             )
#
# #Save in list
# l <- list(lifeForm = mapa_traducao_lifeform,
#           origin = mapa_traducao_origins,
#           habitat = mapa_traducao_habitat,
#           taxonRank = mapa_traducao_taxonrank,
#           taxonomicStatus = mapa_traducao_taxonomicStatus)
# sapply(l, colnames)
# map_translation <- l
#
# #Save as data
# usethis::use_data(map_translation, overwrite = TRUE)

#' Resolve discrepancies between species and subspecies information
#'
#' @param data (data.frame) the data.frame imported with the
#' \code{\link{load_faunabr}} function.
#'
#' @return a data.frame with the discrepancies solved
#' @usage fauna_discrepancies(data)
#' @details
#' In the original dataset, discrepancies may exist between species and
#' subspecies information. An example of a discrepancy is when
#' species occurs only in one state (e.g., SP), but a subspecies or variety
#' of the same species occurs in another states (e.g., SP and RJ). This function
#' rectifies such discrepancies by considering distribution (states and countries)
#' life form, and habitat. For instance, if a subspecies is recorded in a specific
#' state, it implies that the species also occurs in that state
#'
#' @export
#'
#' @examples
#' data("fauna_data") #Load fauna e Funga do Brasil data
#' #Check if discrepancies were solved in the dataset
#' attr(fauna_data, "solved_discrepancies")
#' #Solve discrepancies
#' fauna_solved <- fauna_discrepancies(fauna_data)
#' #Check if discrepancies were solved in the dataset
#' attr(fauna_solved, "solved_discrepancies")
fauna_discrepancies <- function(data) {
  if (missing(data)) {
    stop("Argument data is not defined")
  }
  if (!inherits(data, "data.frame")) {
    stop(paste0("Argument data must be a data.frame, not ",
                class(data)))
  }

  if(isTRUE(attr(data, "fauna_discrepancies"))) {
    stop("Any discrepancies have already been resolved in this dataset")
  }

  #Get subspecies with valid names
  ind_spp <- which(data$taxonRank %in% c("subspecies", "sub_especie") &
                     data$taxonomicStatus %in% c("valid", "valido"))
  spp_var <- data[ind_spp, "species"]
  #Create temporarily column with binomial species name
  spp_var_bin <- extract_binomial(spp_var)
  data$species_bin <-  extract_binomial(data$species)
  #Get only species that exists as Species in dataframe
  spp_var_yes <- intersect(data$species_bin[which(
    data$taxonRank %in% c("species", "especie") &
      data$taxonomicStatus %in% c("valid", "valido"))],
    spp_var_bin)
  #Other species
  spp_var_no <- na.omit(setdiff(na.omit(spp_var_bin),
                                data$species_bin[which(
                                  data$taxonRank %in% c("species", "especie"))]))

  #Get dataframe to update
  d_upt <- subset(data, data$species_bin %in% spp_var_yes)

  #Update columns
  dd_updated_list <- lapply(split(d_upt, d_upt$species_bin),
                            update_columns)

  # Merge dataframes
  d_upt2 <- do.call(rbind, dd_updated_list)
  row.names(d_upt) <- NULL

  #Update final dataframe
  data_solved <- rbind(subset(data, !(data$id %in% d_upt2$id)), d_upt2)

  #Fix varieties and subspecies that does not appear as species
  if(length(spp_var_no) > 0) {
    df_no_species <- data[data$species_bin %in% na.omit(spp_var_no),]
    #Change taxonrank
    if(data$language == "en"){
    df_no_species$taxonRank <- "species"} else {
      df_no_species$taxonRank <- "especie"
    }
    #Create new id
    df_no_species$id <- sample(setdiff(1:50000, data$id), nrow(df_no_species))
    #Subset columns
    df_no_species <- df_no_species[, colnames(data)]
    #Merge data
    data_solved <- rbind(data_solved, df_no_species)
  }

  #Add attribute
  attr(data_solved, "solved_discrepancies") <- TRUE

  #Remove Unknown when there are info
  columns <- c("lifeForm", "habitat", "states", "origin")
  for(i in columns) {
    data_solved[[i]][which(
      grepl(";Unknown|Unknown;", data_solved[[i]]))] <- gsub(
        ";Unknown|Unknown;", "", data_solved[[i]][which(grepl(";Unknown|Unknown;",
                                                              data_solved[[i]]))])
  }

  #Remove column with binomial name without var or subsp
  data_solved$species_bin <- NULL

  #Return
  return(data_solved)
}

#' Download the latest version of Catálogo Taxonômico da Fauna do Brasil
#'
#' @description
#' This function downloads the latest or an older version of Catálogo Taxonômico
#' da Fauna do Brasil database, merges the information into a single data.frame,
#' and saves this data.frame in the specified directory.
#'
#' @param output_dir (character) a directory to save the data downloaded from
#' Fauna do Brasil
#' @param data_version (character) Version of the Fauna do Brasil database to
#' download. Use "latest" to get the most recent version, which is updated
#' frequently. Alternatively, specify an older version (e.g.,
#' data_version = "1.2").Default value is "latest".
#' @param solve_discrepancies Resolve inconsistencies between species and
#' subspecies  information. When set to TRUE (default), species
#' information is updated based on unique data from subspecies.
#' For example, if a subspecies occurs in a certain state, it implies that the
#' species also occurs in that state.
#' @param translate (logical) whether to translate the original dataset
#' ("lifeForm", "origin", "habitat", and "taxonRank") from Portuguese to English.
#' Default is TRUE.
#' @param overwrite (logical) If TRUE, data is overwritten. Default = TRUE.
#' @param verbose (logical) Whether to display messages during function
#' execution. Set to TRUE to enable display, or FALSE to run silently.
#' Default = TRUE.
#'
#' @returns
#' The function downloads the latest version of the Catálogo Taxonômico da Fauna
#' do Brasil database from the official source. It then merges the information
#' into a single data.frame, containing details on species, taxonomy, occurrence,
#' and other relevant data.
#' The merged data.frame is then saved as a file in the specified output
#' directory. The data is saved in a format that allows easy loading using the
#' \code{\link{load_faunabr}} function for further analysis in R.
#'
#' @usage get_faunabr(output_dir, data_version = "latest",
#'                  solve_discrepancies = TRUE, translate = TRUE,
#'                  overwrite = TRUE, verbose = TRUE)
#' @export
#'
#' @importFrom httr GET write_disk
#' @importFrom XML htmlParse xpathSApply xmlGetAttr
#' @importFrom utils unzip
#' @importFrom utils read.csv
#' @importFrom data.table fwrite
#'
#' @references
#' Brazilian Zoology Group. Catálogo Taxonômico da Fauna do Brasil. Available at:
#' https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil
#'
#' @examples
#' \dontrun{
#' #Creating a folder in a temporary directory
#' #Replace 'file.path(tempdir(), "faunaabr")' by a path folder to be create in
#' #your computer
#' my_dir <- file.path(file.path(tempdir(), "faunabr"))
#' dir.create(my_dir)
#' #Download, merge and save data
#' get_faunabr(output_dir = my_dir)
#' }
get_faunabr <- function(output_dir, data_version = "latest",
                        solve_discrepancies = TRUE,
                        translate = TRUE,
                        overwrite = TRUE,
                        verbose = TRUE) {
  #Set folder
  if(is.null(output_dir)) {
    stop(paste("Argument output_dir is not defined, this is necessary for",
         "\n downloading and saving data"))
  }
  if (!is.character(output_dir)) {
    stop(paste0("Argument output_dir must be a character, not ",
                class(output_dir)))
  } else {
    path_data <- output_dir
  }

  if(!file.exists(output_dir)){
    dir.create(output_dir)
    warning("'output_dir' does not exists. Creating folder:\n" , output_dir)
  }

  if (!is.character(data_version)) {
    stop(paste0("Argument data_version must be a character, not ",
                class(data_version)))
  }

  if (!is.logical(solve_discrepancies)) {
    stop(paste0("Argument solve_discrepancies must be logical, not ",
                class(overwrite)))
  }

  if (!is.logical(overwrite)) {
    stop(paste0("Argument overwrite must be logical, not ",
                class(overwrite)))
  }


  #Print message
  if(verbose) {
  message("Data will be saved in ", path_data, "\n") }


  if(data_version != "latest") {
  link_download <- paste0(
    "https://ipt.jbrj.gov.br/jbrj/archive.do?r=catalogo_taxonomico_da_fauna_do_brasil&v=",
                          data_version)
  version_data <- data_version
  }


  if(data_version == "latest") {
  #Get link of latest version
  response <- httr::GET(
    "https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil")
  parse <- XML::htmlParse(response)
  links <- unlist(XML::xpathSApply(parse, path = "//a", XML::xmlGetAttr,
                                   "href"))
  download_pattern <- "https://ipt.jbrj.gov.br/jbrj/archive.do?r=catalogo_taxonomico_da_fauna_do_brasil&v="
  version_data <- subset(links, grepl(download_pattern, links, fixed = TRUE))
  version_data <- gsub(download_pattern, "", version_data, fixed = TRUE)
  link_download <- paste0(
    "https://ipt.jbrj.gov.br/jbrj/archive.do?r=catalogo_taxonomico_da_fauna_do_brasil&v=",
    version_data)
  }

  #Print message
  if(!is.null(version_data) & verbose) {
      message("Downloading version: ", version_data, "\n")


  #Download data
  httr::GET(link_download, httr::write_disk(file.path(
    path_data,
    paste0(version_data, ".zip")),
                                      overwrite = overwrite))
  }

  #Unzip folder
  version_data_numeric <- as.numeric(version_data)
  utils::unzip(zipfile = paste0(file.path(path_data, version_data), ".zip"),
        exdir = file.path(path_data, version_data_numeric))

  #Print message
  if(verbose){
  message("Merging data. Please wait a moment...\n") }

  #Merge data
  merge_data(path_data = path_data, version_data = version_data_numeric,
             translate = translate,
             solve_discrepancies = solve_discrepancies, verbose = verbose)

  #Print final message
  if(verbose){
  message("Data downloaded and merged successfully. Final data saved in",
              file.path(path_data, version_data, "CompleteBrazilianFauna.gz"))
  }

}

#' Get available attributes to filter species
#'
#' @description
#' This function displays all the options available to filter species by its
#' characteristics
#'
#'
#' @param data (data.frame) a data.frame imported with the
#' \code{\link{load_faunabr}} function or a data.frame generated with the
#'  \code{\link{select_fauna}} function.
#' @param attribute (character) the type of characteristic. Accept more than one
#' option. See detail to see the options.
#'
#' @details
#' The attribute argument accepts the following options: phylum, class, family,
#' genus, lifeForm, habitat, states, country, origin, and taxonomicstatus. These options
#' represent different characteristics of species that can be used for filtering.
#'
#' @return a list of data.frames with the available options to use in the
#' \code{\link{select_fauna}} function.
#'
#' @usage fauna_attributes(data, attribute)
#'
#' @export
#'
#' @examples
#' data("fauna_data") #Load data example
#' # Get available states, countries and lifeForms to filter species
#' d <- fauna_attributes(data = fauna_data,
#'                     attribute = c("country", "lifeform", "states"))

fauna_attributes <- function(data, attribute) {

  if (missing(data)) {
    stop("Argument data is not defined")
  }

  #Change specified attribute to lowercase
  attrib <- tolower(attribute)

  #Correct attribute
  attrib[which(attrib == "lifeform")] <- "lifeForm"
  attrib[which(attrib == "taxonomicstatus")] <- "taxonomicStatus"
  attrib[which(attrib == "country")] <- "countryCode"

  if (missing(attribute)) {
    stop("Argument attribute is not defined. Valid attributes:
    phylum, class, family, genus, lifeForm, states, country, origin, and
    taxonomicstatus")
  }

  #Check classes
  if (!inherits(data, "data.frame")) {
    stop(paste0("Argument data must be a data.frame, not ", class(data)))
  }

  if (!is.character(attribute)) {
    stop(paste0("Argument attribute must be a character, not ",
                class(attribute)))
  }


  #Check attributes
  no_valid <- setdiff(attrib, c("phylum", "class", "family", "genus", "lifeForm",
                                "habitat",  "states", "countryCode", "origin",
                                "taxonomicStatus"))

  if(length(no_valid) > 0) {
    stop(paste0("The informed attributes is/are not valid:\n",
                paste(no_valid, collapse = "\n")),
         "\nValid attributes:
    phylum, class, family, genus, lifeForm, habitat, states, country, origin, and
    taxonomicstatus")
  }

  #Get unique attributes
  d_l <- lapply(attrib, function(i){
    d_at <- data[,i]
    at <- unique(unlist(strsplit(d_at, ";")))
    #Save in dataframe
    att_f <- data.frame(var = sort(at))
    colnames(att_f) <- i

    #If states, return name of the states in another column
    if(i == "states") {
      s_df <- data.frame(
        states = c("AC", "AL", "AP", "AM", "BA", "CE", "DF", "ES", "GO", "MA", "MT",
                   "MS", "MG", "PA", "PB", "PR", "PE", "PI", "RJ", "RN", "RS", "RO",
                   "RR", "SC", "SP", "SE", "TO"),
        states_name = c("Acre", "Alagoas", "Amapa", "Amazonas", "Bahia", "Ceara",
                        "Distrito Federal", "Espirito Santo", "Goias", "Maranhao",
                        "Mato Grosso", "Mato Grosso do Sul", "Minas Gerais", "Para",
                        "Paraiba", "Parana", "Pernambuco", "Piaui", "Rio de Janeiro",
                        "Rio Grande do Norte", "Rio Grande do Sul", "Rondonia",
                        "Roraima", "Santa Catarina", "Sao Paulo", "Sergipe",
                        "Tocantins"))
      att_f <- s_df[which(s_df$states %in% att_f$states),] }

    return(att_f)
  })
  names(d_l) <- attribute

  return(d_l)
}

#####Helper functions####

#First letter to upper case
firstup <- function(x) {
  x <- tolower(x)
  substr(x, 1, 1) <- toupper(substr(x, 1, 1))
  x
}

# #Translate origin, habitat, lifeform, taxonomicStatus and nomenclaturalStatus
# translate_lifeform <- function(text) {
#   # Aplicar gsub para traduzir e converter para letras minúsculas cada forma de vida em português
#   new_lifeform <- gsub("VIDA_LIVRE_INDIVIDUAL", "free_living_individual", text, ignore.case = TRUE)
#   new_lifeform <- gsub("COLONIAL", "colonial", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("SESSIL", "sessile", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("HERBIVORO", "herbivore", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("EUSSOCIAL", "eusocial", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("PREDADOR", "predator", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("ENDOPARASITOIDE", "endoparasitoid", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("ECTOPARASITOIDE", "ectoparasitoid", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("HIPERPARASITOIDE", "hyperparasitoid", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("ECTOPARASITO", "ectoparasite", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("ENDOPARASITO", "endoparasite", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("COMENSAL", "commensal", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("EPIBIONTE", "epibiont", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("POLINIZADOR", "polynizer", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("INQUILINO", "inquiline", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("MUTUAL", "mutualistic", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("endoparasiteid", "endoparasitoid", new_lifeform, ignore.case = TRUE)
#   new_lifeform <- gsub("ectoparasiteid", "ectoparasitoid", new_lifeform, ignore.case = TRUE)
#   return(new_lifeform)
# }
#
# translate_origins <- function(text) {
#   # Aplicar gsub para traduzir e converter para letras minúsculas cada origem
#   text[text == ""] <- NA
#   new_origins <- gsub("NATIVA", "native", text, ignore.case = TRUE)
#   new_origins <- gsub("INTRODUZIDA", "introduced", new_origins, ignore.case = TRUE)
#   new_origins <- gsub("CRIPTOGENICA", "cryptogenic", new_origins, ignore.case = TRUE)
#   new_origins <- gsub("DOMESTICADA", "domesticated", new_origins, ignore.case = TRUE)
#   new_origins <- gsub("INVASORA", "invasive", new_origins, ignore.case = TRUE)
#   return(new_origins)
# }
#
# translate_habitat <- function(text) {
#   # Aplicar gsub para traduzir e converter para letras minúsculas cada categoria de habitat
#   text[text == ""] <- NA
#   new_habitat <- gsub("AGUA_DOCE", "freshwater", text, ignore.case = TRUE)
#   new_habitat <- gsub("MARINHO", "marine", new_habitat, ignore.case = TRUE)
#   new_habitat <- gsub("TERRESTRE", "terrestrial", new_habitat, ignore.case = TRUE)
#   new_habitat <- gsub("ARBOREO", "arboreal", new_habitat, ignore.case = TRUE)
#   new_habitat <- gsub("FOSSORIAL", "fossorial", new_habitat, ignore.case = TRUE)
#   new_habitat <- gsub("CAVERNICOLA", "cavernicolous", new_habitat, ignore.case = TRUE)
#   new_habitat <- gsub("AGUAS_SUBTERRANEAS", "subterranean waters", new_habitat, ignore.case = TRUE)
#   new_habitat <- gsub("NIDICOLA", "nidicolous", new_habitat, ignore.case = TRUE)
#   new_habitat <- gsub("HIPORREICO", "hyporheic", new_habitat, ignore.case = TRUE)
#   return(new_habitat)}
#
# translate_taxonrank <- function(rank_vector) {
#   # Converter o vetor para caracteres (strings) e tratar NA como um valor especial
#   rank_vector <- as.character(rank_vector)
#
#   # Realizar as substituições diretamente
#   rank_vector[rank_vector == "FILO"] <- "phylum"
#   rank_vector[rank_vector == "CLASSE"] <- "class"
#   rank_vector[rank_vector == "ORDEM"] <- "order"
#   rank_vector[rank_vector == "SUB_ORDEM"] <- "suborder"
#   rank_vector[rank_vector == "FAMILIA"] <- "family"
#   rank_vector[rank_vector == "SUB_FAMILIA"] <- "subfamily"
#   rank_vector[rank_vector == "TRIBO"] <- "tribe"
#   rank_vector[rank_vector == "GENERO"] <- "genus"
#   rank_vector[rank_vector == "SUB_GENERO"] <- "subgenus"
#   rank_vector[rank_vector == "ESPECIE"] <- "species"
#   rank_vector[rank_vector == "SUB_ESPECIE"] <- "subspecies"
#   rank_vector[rank_vector == "SUB_CLASSE"] <- "subclass"
#   rank_vector[rank_vector == "INFRA_ORDEM"] <- "infraorder"
#   rank_vector[rank_vector == "SUPER_FAMILIA"] <- "superfamily"
#   rank_vector[rank_vector == "SUPER_CLASSE"] <- "superclass"
#   rank_vector[rank_vector == "SUB_TRIBO"] <- "subtribe"
#   rank_vector[rank_vector == "SUPER_ORDEM"] <- "superorder"
#   rank_vector[rank_vector == "INFRA_CLASSE"] <- "infraclass"
#
#   # Substituir NA por "unknown"
#   rank_vector[is.na(rank_vector)] <- "unknown"
#
#   return(rank_vector)
# }

#Extract string between patterns
extract_between <- function(str, left, right) {
  inicio <- regexpr(left, str) + attr(regexpr(left, str), "match.length")
  str_inicio <- substring(str, first = inicio)
  fim <- regexpr(right, str_inicio)
  final <- substring(str_inicio, first = 0, last = fim - 1)
  ifelse(inicio > 0 & fim > 0, final, NA)
}


#Solve incongruences between varieties/subspecies and species
update_columns <- function(df) {
  # Get unique values of lifeForm, habitat, vegetationType, Biome e States
  unique_lifeForm <- sort(unique(unlist(strsplit(df$lifeForm, ";"))))
  unique_habitat <- sort(unique(unlist(strsplit(df$habitat, ";"))))
  unique_states <- sort(unique(unlist(strsplit(df$states, ";"))))
  unique_countries <- sort(unique(unlist(strsplit(df$countryCode, ";"))))

  # Update columns where taxonRank == "ESPECIE"
  df$lifeForm[df$taxonRank == "species"] <- paste(unique_lifeForm, collapse = ";")
  df$habitat[df$taxonRank == "species"] <- paste(unique_habitat, collapse = ";")
  df$states[df$taxonRank == "species"] <- paste(unique_states, collapse = ";")
  df$states[df$taxonRank == "species"] <- paste(unique_states, collapse = ";")
  df$countryCode[df$taxonRank == "species"] <- paste(unique_countries,
                                                     collapse = ";")

  #Return only taxonRank == "ESPECIE"
  df <- subset(df, df$taxonRank == "species")

  return(df)
}

#Extract binomial name
extract_species <- function(texto) {

  # Remove curly braces {}
  texto2 <- gsub("\\{|\\}", "", texto)

  # Remove excess of whitespace between words
  texto2 <- gsub("\\s+", " ", texto2)

  # Remove leading and/or trailing whitespace
  texto2 <- trimws(texto2)

  # Extrair two first words
  binomial <- gsub("^([[:alnum:]]+[-[:alnum:]]*(?:[[:space:]]+[[:alnum:]]+[-[:alnum:]]*)?)\\b.*",
                   "\\1", texto2)

  if (grepl("-", binomial)) {
    return(binomial)
  }

  if (!grepl("-", binomial)) {
    # Verificar se há um padrão de palavra seguida por espaço e outra palavra
    match <- regexpr("\\b\\w+(-\\w+)?\\s+\\(\\w+\\)\\s+\\w+(-\\w+)?\\b|\\b\\w+(-\\w+)?\\s+\\w+\\b", texto2, perl = TRUE)
    if (match > 0) {
      # Extrair o trecho correspondente ao padrão encontrado
      palavras <- regmatches(texto2, match)
      return(palavras)
    } else {
      # Se não encontrar nenhum padrão específico, retornar a string original
      return(texto2)
    }
  }
}

#Extract year from scientificname
extract_year <- function(sn) {
  match <- gregexpr("\\b\\d{4}\\b", sn)
  if (any(match[[1]] > 0)) {
    return(regmatches(sn, match))
  } else {
    return(NA)
  }
}


merge_data <- function(path_data, version_data, solve_discrepancies = TRUE,
                       translate = TRUE, encoding = "UTF-8", verbose = TRUE) {

  #Set folder
  if(is.null(path_data)) {
    stop(paste("Argument path_data is not defined, this is necessary for",
               "\n saving data"))
  }

  #Print message
  if(verbose) {
    message("Data will be saved in ", path_data, "\n") }

  #Get latest available version if version was not set
  if(version_data == "latest") {
    all_dirs <- list.dirs(path = path_data, recursive = FALSE,
                          full.names = FALSE)
    dir_versions <- stats::na.omit(as.numeric(all_dirs)) #Actual version
    #Get highest version
    if(length(dir_versions) > 0) {
      high_version <- max(dir_versions)
      version_data <- high_version } else {
        version_data <- 0
      } }

  #Taxon
  taxon <- utils::read.csv(file.path(path_data, version_data, "taxon.txt"),
                           header=TRUE, sep = "\t",
                           encoding = encoding, na.strings = "")
  #Remove accents
  taxon$higherClassification <- iconv(taxon$higherClassification,
                                      to="ASCII//TRANSLIT")

  #Vernacular name
  vernacular <- utils::read.csv(file.path(path_data, version_data,
                                          "vernacularname.txt"),
                                header=TRUE, sep = "\t",
                                encoding = encoding, na.strings = "")
  #Remove accents
  vernacular$vernacularName <- iconv(vernacular$vernacularName,
                                     to="ASCII//TRANSLIT")

  #Group vernacular names from same species
  grouped <- split(vernacular, vernacular$id)
  summarized <- lapply(grouped, function(group) {
    paste(group$vernacularName, collapse = ", ")
  })
  vernacular_final <- data.frame(
    id = as.numeric(names(summarized)),
    vernacularName = unlist(summarized)
  )

  ###Species Profile
  spProfile <- utils::read.csv(file.path(path_data, version_data,
                                         "speciesprofile.txt"),
                               header=TRUE, sep = "\t",
                               encoding = encoding, na.strings = "")
  #Remove accents
  spProfile$lifeForm <- iconv(spProfile$lifeForm, to="ASCII//TRANSLIT")
  spProfile$habitat <- iconv(spProfile$habitat, to="ASCII//TRANSLIT")


  ###Distribution and Location
  dist <- utils::read.csv(file.path(path_data, version_data,
                                    "distribution.txt"),
                          header=TRUE, sep = "\t",
                          encoding = encoding, na.strings = "")
  # #Remove accents
  # dist$occurrenceRemarks <- iconv(dist$occurrenceRemarks,
  #                                 to="ASCII//TRANSLIT")
  #Extrair informações para novas coluna

    #origin
  dist$origin <- dist$establishmentMeans


  #Organize information
  #Local
  Local <- dist[,c("id","locality","countryCode")]
  #Group location of same species
  grouped <- split(Local, Local$id)
  summarized <- lapply(grouped, function(group) {
    if(!is.na(group$locality)){
    paste(group$locality, collapse = ";")} else {group$locality}
    })
  Local_final <- data.frame(
    id = as.numeric(names(summarized)),
    locality = unlist(summarized)
  )
  #Merge distribution data again
  dist_final <- dist[, c("id", "countryCode", "origin")]
  dist_final <- merge(dist_final, Local_final, by = "id")
  dist_final <- unique(dist_final[,colnames(dist_final)])

  #Merge all information
  df_final1 <- merge(taxon, vernacular_final, by = "id", all = TRUE)
  df_final2 <- merge(df_final1, spProfile, by = "id", all = TRUE)
  df_final3 <- merge(df_final2, dist_final, by = "id", all = TRUE)

  #Create columns with name of the specie and accepted name
  which_is_species <- which(df_final3$taxonRank %in%
                              c("ESPECIE", "SUB_ESPECIE",
                                "species", "subspecies"))
  which_is_subspecies <- which(df_final3$taxonRank %in%
                                 c("SUB_ESPECIE", "subspecies"))
  # Create column
  df_final3$species <- NA
  df_final3$species[which_is_species] <- paste(df_final3$genus[which_is_species],
                                                df_final3$specificEpithet[which_is_species])
  df_final3$subspecies <- NA
  df_final3$subspecies[df_final3$taxonRank %in%
                         c("SUB_ESPECIE", "subspecies")] <- paste(df_final3$species[which_is_subspecies],
                                                                      df_final3$infraspecificEpithet[which_is_subspecies])

  #Accepted name when is synonymn
  df_final3$acceptedName <- NA
  sp_syn <- which(df_final3$taxonRank %in% c("ESPECIE" , "SUB_ESPECIE",
                                             "species", "subspecies") &
                  df_final3$taxonomicStatus %in% c("SINONIMO", "synonym") &
                    !is.na(df_final3$acceptedNameUsage))
  df_final3[sp_syn, "acceptedName"] <- sapply(df_final3[sp_syn, "acceptedNameUsage"],
                                              extract_species)

  #resourcerelationship
  rr <- utils::read.csv(file.path(path_data, version_data,
                                 "resourcerelationship.txt"),
                       header=TRUE, sep = "\t",
                       encoding = encoding, na.strings = "")

  #Group resourcerelationship of same species
  grouped <- split(rr, rr$id)
  summarized_relatedResourceID <- lapply(grouped, function(group) {
    paste(group$relatedResourceID, collapse = ";")
    })
  summarized_relationshipOfResource <- lapply(grouped, function(group) {
    paste(group$relationshipOfResource, collapse = ";")
  })

  rr_final <- data.frame(
    id = as.numeric(names(summarized_relatedResourceID)),
    relatedResourceID = unlist(summarized_relatedResourceID),
    relationshipOfResource = unlist(summarized_relationshipOfResource)
  )
  df_final4 <- merge(df_final3, rr_final, by = "id", all = TRUE)


  #Extract year
  df_final4$namePublishedInYear <- sapply(df_final4$scientificName,
                                          extract_year)

  #Remove {} from acceptednameUsage usage
  df_final4$acceptedNameUsage <- gsub("\\{|\\}", "", df_final4$acceptedNameUsage)
  df_final4$acceptedNameUsage[which(df_final4$acceptedNameUsage == "")] <- NA
  df_final4$acceptedName <- gsub("\\{|\\}", "", df_final4$acceptedName)
  df_final4$acceptedName[which(df_final4$acceptedName == "")] <- NA

  #Order columns
  df_final <- df_final4[,c(c("id", "taxonID","species", "subspecies",
                             "scientificName",
                             "acceptedName",
                             "acceptedNameUsage",
                             "parentNameUsage",
                             "namePublishedInYear",
                             "higherClassification", "kingdom",
                             "phylum", "class", "order", "family", "genus",
                             "specificEpithet",
                             "infraspecificEpithet", "taxonRank",
                             "scientificNameAuthorship",
                             "taxonomicStatus", "nomenclaturalStatus",
                             "vernacularName", "lifeForm",
                             "habitat", "origin",
                             "locality",
                             "countryCode", "modified", "bibliographicCitation",
                             "relationshipOfResource"))]

  #Sort information and separe using ;
  df_final$lifeForm <- vapply(df_final$lifeForm, FUN.VALUE = character(1),
                             function(x){
                               paste(sort(unlist(strsplit(x, split = " \\| "))),collapse = ";")
                             }, USE.NAMES = FALSE)
  df_final$habitat <- vapply(df_final$habitat, FUN.VALUE = character(1),
                            function(x){
                              paste(sort(unlist(strsplit(x, split = " \\| "))),collapse = ";")
                            }, USE.NAMES = FALSE)


  #Rename columns
  colnames(df_final)[colnames(df_final) == "locality"] <- "states"

  #Fix countries
  df_final$countryCode[is.na(df_final$countryCode) & !is.na(df_final$states)] <- "BR"
  #Get index for more complex fixing
  index_fix <- !grepl("BR", df_final$countryCode) & !is.na(df_final$states)
  countries_fix <- df_final$countryCode[index_fix]
  countries_fixed <- sapply(countries_fix, function(x) {
    paste(sort(c("BR", unlist(strsplit(x, ";", fixed = TRUE)))), collapse = ";")
    }, USE.NAMES = FALSE)
  df_final$countryCode[index_fix] <- countries_fixed
  #df_final[index_fix,] %>% View()

  #Fix states
  df_final$states <- gsub("BR-", "", df_final$states)

  # #Translate and put in lower case
  # df_final$lifeForm <- translate_lifeform(df_final$lifeForm)
  # df_final$lifeForm[df_final$lifeForm == ""] <- NA
  # df_final$origin <- translate_origins(df_final$origin)
  # df_final$habitat <- translate_habitat(df_final$habitat)
  # df_final$taxonRank <- translate_taxonrank(df_final$taxonRank)
  # df_final$taxonomicStatus[df_final$taxonomicStatus=="NOME_ACEITO"] <- "accepted"
  # df_final$taxonomicStatus[df_final$taxonomicStatus=="SINONIMO"] <- "synonym"
  # df_final$nomenclaturalStatus <- tolower(df_final$nomenclaturalStatus)
  # #df_final$countryCode <- tolower(df_final$countryCode)
  # df_final$origin[df_final$origin=="EXOTICA"] <- "exotic"

  #Replace accepted_name with valid in taxonomicStatus
  df_final$taxonomicStatus[df_final$taxonomicStatus %in% c("NOME_ACEITO", "accepted")] <- "valido"

  #Change name of acceptedName to validName
  colnames(df_final)[colnames(df_final) == "acceptedName"] <- "validName"
  colnames(df_final)[colnames(df_final) == "acceptedNameUsage"] <- "validNameUsage"

  #Add language
  df_final$language <- "pt_br"

  if(translate){ #Translate
    df_final <- translate_faunabr(df_final, to = "en")} else {
      #Or put columns to lower
      df_final$lifeForm <- tolower(df_final$lifeForm)
      df_final$origin <- tolower(df_final$origin)
      df_final$habitat <- tolower(df_final$habitat)
      df_final$taxonRank <- tolower(df_final$taxonRank)
      df_final$taxonomicStatus <- tolower(df_final$taxonomicStatus)
    }


  #Solve incongruencies?
  if(solve_discrepancies){
    df_final <- fauna_discrepancies(df_final)
  }

  #Save as gzip format
  data.table::fwrite(df_final,
          file = file.path(path_data, version_data,
                           "CompleteBrazilianFauna.gz"),
          row.names = FALSE,
          compress = "gzip")

}

# ####Generate data to examples #####
# library(dplyr)
# library(data.table)
# library(terra)
# library(geobr)
#
# ####Get Fauna do Brazil dataset####
# my_dir <- "../faunabrdata/"
# dir.create(my_dir)
# get_faunabr(output_dir = my_dir, solve_discrepancies = FALSE)
#
# #Fauna do Brazil data
# df <- load_faunabr(data_dir = my_dir, type = "short")
#
# #Get only native species of chordata
# p <- df %>% filter(taxonRank == "species", origin == "native", phylum == "Chordata")
# #Get only accepted names
# pac <- p %>% filter(taxonomicStatus == "accepted")
# #Include some synonyms
# syn <- df %>% filter(acceptedName %in% c("Mazama nemorivaga", "Mazama jucunda",
#                                     "Subulo gouzoubira"))
#
# #Final data
# fauna_data <- bind_rows(pac, syn)
# usethis::use_data(fauna_data, overwrite = TRUE)

#
# ####Get species occurrences####
# library(plantR)
# library(CoordinateCleaner)
# library(pbapply)
# library(dplyr)
#
# spp <- c("Panthera onca", "Chaetomys subspinosus")
#
# oc.gbif <- pblapply(spp, function(i) {
#   rgbif2(species = i, force = TRUE, remove_na = TRUE) })
# oc.gbif <- bind_rows(oc.gbif)
#
#
# #Clean data
# library(CoordinateCleaner)
# oc_n <- oc.gbif %>% mutate(decimalLatitude = as.numeric(decimalLatitude),
#                            decimalLongitude = as.numeric(decimalLongitude))
#
# occ_f <- clean_coordinates(x = oc_n, lon = "decimalLongitude",
#                            lat = "decimalLatitude",
#                            species = "species", countries = "countryCode",
#                            tests = c("capitals", "centroids", "equal", "gbif",
#                                      "institutions","seas", "zeros"))
# #Select only valid records
# occ <- occ_f %>% filter(.summary == TRUE) %>%
#   dplyr::select(species, x = "decimalLongitude", y = "decimalLatitude",
#                 datasetKey) #To get DOI
# #Remove duplicates
# occ_dup <- cc_dupl(occ, species = "species", lon = "x", lat = "y")
# occurrences <- data.frame(occ_dup)
#
# #Generate fake data to flag
# #States in Brazil without occurrence of panthera onca
# br <- vect(geobr::read_state())
# #Pernambuco and alagoas centroid
# pe <- as.data.frame(terra::centroids(br[br$abbrev_state %in% c("PE", "AL")],
#                                      inside = TRUE),
#                     geom = "XY")
# fake_occ1 <- data.frame("species" = "Panthera onca", x = pe$x, y = pe$y)
# #States in Brazil without occurrence of "Chaetomys subspinosus"
# #DF and SP
# dfsp <- as.data.frame(terra::centroids(br[br$abbrev_state %in% c("DF", "SP")],
#                                        inside = TRUE),
#                       geom = "XY")
# fake_occ2 <- data.frame("species" = "Chaetomys subspinosus", x = dfsp$x, y = dfsp$y)
#
# #Countries without occurrence of panthera onca
# #Chile and Cuba for panthera onca
# w <- terra::unwrap(faunabr::world_fauna)
# chile_cuba <- as.data.frame(terra::centroids(w[w$name %in% c("chile", "cuba")],
#                                              inside = TRUE),
#                             geom = "XY")
# fake_occ3 <- data.frame("species" = "Panthera onca", x = chile_cuba$x, y = chile_cuba$y)
#
# #Countries without occurrence of Chaetomys subspinosus
# #Paraguai and bolivia for Chaetomys subspinosus
# pb <- as.data.frame(terra::centroids(w[w$name %in% c("paraguay", "bolivia")],
#                                              inside = TRUE),
#                             geom = "XY")
# fake_occ4 <- data.frame("species" = "Chaetomys subspinosus",
#                         x = pb$x, y = pb$y)
#
# #Join data and identify as fake data
# fake <- bind_rows(fake_occ1, fake_occ2, fake_occ3, fake_occ4)
# fake$source <- "fake_data"
# occurrences$source <- "gbif"
# occurrences <- bind_rows(occurrences, fake)

# #Plot records to see
# pts <- vect(occurrences, geom = c(x = "x",y = "y"), crs = "EPSG:4326")
# mapview::mapview(pts, zcol = "species", burst = T)
#
# # #Data set key
# # ds_key <- occurrences %>% count(datasetKey)
# #
# # derived_dataset(
# #   citation_data = ds_key,
# #   title = "florabr R package: Records of plant species",
# #   description="This data was downloaded using plantR::rgbif2, filtered using
# #   CoordinateCleaner::clean_coordinates and later incorported as data example in
# #   florabr R Package",
# #   source_url="https://github.com/wevertonbio/florabr/raw/main/data/occurrences.rda",
# #   gbif_download_doi = NULL,
# #   user = user, #User in GBIF
# #   pwd = pwd) #Password in GBIF
#
# #Remove datasetKey column
# occurrences <- occurrences %>% dplyr::select(-datasetKey)
# usethis::use_data(occurrences, overwrite = TRUE)

# # #### Get shape of countries ####
# w <- vect(rnaturalearth::ne_countries(scale = "large",
#                                       returnclass = "sf"))
# as.data.frame(w) %>% View()
# #Match with countrycodes in data
# cc <- bf$countryCode %>% strsplit(";") %>% unlist() %>% unique()
# country_names <- countrycode::countrycode(cc, origin = "iso2c",
#                                           destination = "country.name")
# country_names <- gsub("&", "and", country_names)
# # Create a dataframe
# country_df <- data.frame(
#   country_code = cc,
#   country_name = country_names,
#   stringsAsFactors = FALSE
# )
# #Fix some names manually
# country_df$country_name[country_df$country_code == "KOS"] <- "Kosovo"
# country_df$country_name[country_df$country_code == "PS-GZA"] <- "Palestine"
# #Saves as data names country_codes
# country_codes <- country_df %>% na.omit()
# #Match with names in w
# w$name %>% as.data.frame %>% View()
# c_matches <- florabr::match_names(species = country_df$country_name,
#                                   species_to_match = w$name_long) %>%
#   filter(!is.na(input_name))
# #Merge data
# c_matches <- c_matches %>% rename(bf_name = input_name,
#                                   map_name = Suggested_name) %>%
#   select(-Distance) %>%
#   left_join(country_codes, by = join_by("bf_name" =="country_name"))
# #Fix manually
# c_matches$map_name[c_matches$country_code == "CV"] <- "Republic of Cabo Verde"
# c_matches$map_name[c_matches$country_code == "CD"] <- "Democratic Republic of the Congo"
# c_matches$map_name[c_matches$country_code == "CG"] <- "Republic of the Congo"
# c_matches$map_name[c_matches$country_code == "VC"] <- "Saint Vincent and the Grenadines"
# c_matches$map_name[c_matches$country_code == "KR"] <- "Republic of Korea"
# c_matches$map_name[c_matches$country_code == "CZ"] <- "Czech Republic"
# c_matches$map_name[c_matches$country_code == "SH"] <- "Saint Helena"
# c_matches$map_name[c_matches$country_code == "LC"] <- "Saint Lucia"
# c_matches$map_name[c_matches$country_code == "FM"] <- 'Federated States of Micronesia'
# c_matches$map_name[c_matches$country_code == "KP"] <- 'Dem. Rep. Korea'
# c_matches$map_name[c_matches$country_code == "MM"] <- 'Myanmar'
# c_matches$map_name[c_matches$country_code == "VI"] <- 'United States Virgin Islands'
# c_matches$map_name[c_matches$country_code == "HK"] <- 'Hong Kong'
# c_matches$map_name[c_matches$country_code == "MO"] <- 'Macao'
# c_matches$map_name[c_matches$country_code == "UM"] <- 'United States Minor Outlying Islands'
# c_matches$map_name[c_matches$country_code == "TF"] <- 'French Southern and Antarctic Lands'
# c_matches$map_name[c_matches$country_code == "GS"] <- 'South Georgia and the Islands'
# c_matches$map_name[c_matches$country_code == "SZ"] <- 'Kingdom of eSwatini'
# #Save as data
# country_codes <- c_matches %>% select(-bf_names)
# usethis::use_data(country_codes, overwrite = TRUE)
#
# #Get only column with country
# w <- w[,"name_long"]
#
# #Simplify
# w2 <- terra::simplifyGeom(w)
#
# #Merge with countryCode
# w3 <- merge(w2, country_codes, by.x = "name_long", by.y = "map_name")
# world_fauna <- terra::wrap(w3)
# usethis::use_data(world_fauna, overwrite = TRUE)
# writeVector(w, "data/teste.gpkg")
#
#
# #To help fix names
# #Get all names
# n <- unique(w$name)
# #Get all countries in faunadata
# df <- load_faunabr("../faunabrdata/")
# co <- unique(unlist(strsplit(df$countryCode, ";")))
# #Get difference
# no_co <- setdiff(co, n)
# sort(no_co)
# data.frame(n) %>% View()

# # States #
# states <- terra::unwrap(florabr::states)
# crs(states) <- crs(countrys)
# states <- terra::wrap(states)
# usethis::use_data(states, overwrite = TRUE)

# states <- geobr::read_state() %>% vect()
# crs(states) <- "EPSG:4326"
# states <- terra::wrap(states)
# usethis::use_data(states, overwrite = TRUE)


# #### Make sticker ####
# library(hexSticker)
# library(showtext)
# ## Loading Google fonts (http://www.google.com/fonts)
# font_families_google()
# font_add_google(name = "Markazi Text")
# ## Automatically use showtext to render text for future devices
# showtext_auto()
#
# sticker(subplot = "../faunabrdata/Mapa.png", package="faunabr", p_size=22.5,
#         p_fontface = "bold", p_color = "black",
#         p_family = "Markazi Text",
#         p_y = 1.59,
#         s_x=1, s_y=.76, s_width=.67,
#         h_fill = "#008080",
#         filename="../faunabrdata/HexSticker_markazi_blue4.png",
#         spotlight = TRUE, l_alpha = 0.3)

#' Identify records outside natural ranges according to Fauna do Brasil
#'
#' @description This function removes or flags records outside of the species'
#' natural ranges according to information provided by the Fauna do Brasil database
#'
#' @param data (data.frame) the data.frame imported with the
#' \code{\link{load_faunabr}} function.
#' @param occ (data.frame) a data.frame with the records of the species.
#' @param species (character) column name in occ with species names.
#' Default = "species"
#' @param long (character) column name in occ with longitude data. Default = "x"
#' @param lat (character) column name in occ with latitude data. Default = "y"
#' @param by_state (logical) filter records by state? Default = TRUE
#' @param buffer_state (numeric) buffer (in km) around the polygons of the
#' states of occurrence of the specie. Default = 20.
#' @param by_country (logical) filter records by country? Default = TRUE
#' @param buffer_country (numeric) buffer (in km) around the polygons of the
#' countries of occurrence of the specie. Default = 20.
#' @param value (character) Defines output values. See Value section.
#' Default = "flag&clean".
#' @param keep_columns (logical) if TRUE, keep all the original columns of the
#' input occ. If False, keep only the columns species, long and lat.
#' Default = TRUE
#' @param spat_state (SpatVector) a SpatVector of the Brazilian states. By
#' default, it uses the SpatVector provided by geobr::read_state(). It can be
#' another Spatvector, but the structure must be identical to
#' 'faunabr::states', with a column called "abbrev_state" identifying the states
#' codes.
#' @param spat_country (SpatVector) a SpatVector of the world countries. By
#' default, it uses the SpatVector provided by rnaturalearth::ne_countries. It
#' can be another Spatvector, but the structure must be identical to
#' 'faunabr::world_fauna', with a column called "country_code" identifying the
#' country codes.
#' @param verbose (logical) Whether to display species being filtered during
#' function execution. Set to TRUE to enable display, or FALSE to run silently.
#' Default = TRUE.
#' @details
#' If by_state = TRUE and/or by_country = TRUE, the function takes polygons
#' representing the states and/or countrys with confirmed occurrences of the
#' specie, draws a buffer around the polygons, and tests if the records of the
#' species fall inside it.
#'
#'
#' @return Depending on the 'value' argument. If value = "flag", it returns the
#' same data.frame provided in data with additional columns indicating if the
#' record falls inside the natural range of the specie (TRUE) or outside
#' (FALSE).
#' If value = "clean", it returns a data.frame with only the records that passes
#' all the tests (TRUE for all the filters). If value = "flag&clean" (Default),
#' it returns a list with two data.frames: one with the flagged records and one
#' with the cleaned records.
#' @usage filter_faunabr(data, occ, species = "species", long = "x", lat = "y",
#'                       by_state = TRUE, buffer_state = 20, by_country = TRUE,
#'                       buffer_country = 20, value = "flag&clean",
#'                       keep_columns = TRUE, spat_state = NULL,
#'                       spat_country = NULL, verbose = TRUE)
#' @export
#'
#' @importFrom terra aggregate subset buffer unwrap mask as.data.frame vect
#' @importFrom data.table rbindlist
#' @importFrom stats na.omit
#' @examples
#' data("fauna_data") #Load fauna e Funga do Brasil data
#' data("occurrences") #Load occurrences
#' pts <- subset(occurrences, species == "Panthera onca")
#' fd <- filter_faunabr(data = fauna_data,
#'                      occ = pts, long = "x", lat = "y", species = "species",
#'                      by_state = TRUE, buffer_state = 20,
#'                      by_country = TRUE, buffer_country = 20,
#'                      value = "flag&clean", keep_columns = TRUE,
#'                      verbose = FALSE)
filter_faunabr <- function(data,
                           occ,
                           species = "species", long = "x", lat = "y",
                           by_state = TRUE, buffer_state = 20,
                           by_country = TRUE, buffer_country = 20,
                           value = "flag&clean", keep_columns = TRUE,
                           spat_state = NULL,
                           spat_country = NULL,
                           verbose = TRUE) {
  if (missing(data)) {
    stop("Argument data is not defined")
  }
  if (missing(occ)) {
    stop("Argument occ is not defined")
  }
  if (!inherits(data, "data.frame")) {
    stop(paste0("Argument data must be a data.frame, not ",
                class(data)))
  }
  if (!inherits(occ, "data.frame")) {
    stop(paste0("Argument occ must be a data.frame, not ",
                class(occ)))
  }
  if (!is.character(species)) {
    stop(paste0("Argument species must be a character, not ",
                class(species)))
  }
  if (!is.character(long)) {
    stop(paste0("Argument long must be a character, not ",
                class(long)))
  }
  if (!is.character(lat)) {
    stop(paste0("Argument lat must be a character, not ",
                class(lat)))
  }
  if (!is.logical(by_state)) {
    stop(paste0("Argument by_state must be logical, not ",
                class(by_state)))
  }
  if (!is.logical(by_country)) {
    stop(paste0("Argument by_country must be logical, not ",
                class(by_country)))
  }

  if (!is.numeric(buffer_state)) {
    stop(paste0("Argument buffer_state must be numeric, not ",
                class(buffer_state)))
  }
  if (!is.numeric(buffer_country)) {
    stop(paste0("Argument buffer_country must be numeric, not ",
                class(buffer_country)))
  }
  allowed_values <- c("flag&clean", "flag", "clean")
  if (!(value %in% allowed_values)) {
    stop("Argument value must be 'flag', 'clean' or 'flag&clean'")
  }
  if (!is.logical(keep_columns)) {
    stop(paste0("Argument keep_columns must be logical, not ",
                class(keep_columns)))
  }
  if (!is.logical(verbose)) {
    stop(paste0("Argument verbose must be logical, not ",
                class(verbose)))
  }

  #Convert colnames to lower case
  original_colnames <- colnames(data)
  colnames(data) <- tolower(colnames(data))

  if (!all(c("species", "states", "countrycode") %in%
           colnames(data))) {
    stop("Important columns are missing in data. Check if data is an object\n created by 'load_faunabr()")
  }
  if (!all(c(species, long, lat) %in% colnames(occ))) {
    stop("Important columns are missing in occurrence data. Check if correct\n         column names were set in species, long and lat")
  }
  d <- data[, c("species", "states", "countrycode")]
  occ$id_f <- seq_len(nrow(occ))
  occ_info <- occ[, c(species, long, lat, "id_f")]
  colnames(occ_info) <- c("species", "x", "y", "id_f")
  spp <- unique(occ_info$species)
  spp_out <- setdiff(spp, unique(data$species))
  if (length(spp_out) > 0) {
    stop(paste(length(spp_out), "species are not in the data. Check the species
names using the check_names() function or remove the species from
data.frame"))
  }
  d_info <- subset(d, d$species %in% unique(occ_info$species))
  d_info[d_info == ""] <- NA

   sp_info <- lapply(seq_along(spp), function(i) {
    sp <- subset(d_info, d_info$species == spp[i])
    sp$states <- paste0(na.omit(unique(sp$states)), collapse = ";")
    sp$countrycode <- paste0(na.omit(unique(sp$countrycode)), collapse = ";")
    return(sp)
  })
  sp_info <- unique(data.table::rbindlist(sp_info))
  occ_info <- merge(occ_info, sp_info, by = "species")
  occ_info <- terra::vect(occ_info, geom = c("x", "y"),
                          crs = "EPSG:4326")


  #Load data
  if(is.null(spat_state)){
    states <- terra::unwrap(faunabr::states)} else {
      states <- spat_state
      }

  if(is.null(spat_country)){
    countrys <- terra::unwrap(faunabr::world_fauna) } else {
      countrys <- spat_country
      }


  #Buffer around Brazil (to select records inside Brazil)
  if(by_state){
  br_v <- terra::buffer(countrys[countrys$bf_name == "Brazil"],
                        width = buffer_state * 1000)

  #Check records inside Brazil
  occ_info$inside_br <- terra::is.related(occ_info, states, "intersects")
  } else {
    occ_info$inside_br <- NA
  }

  if (by_state) {
    l_state <- lapply(seq_along(spp), function(i) {
      if (verbose) {
        message("Filtering ", spp[i], " by state\n")
      }
      occ_i <- occ_info[occ_info$species == spp[i]]
      sp_i_state <- unique(gsub(";", "|", occ_i$states[1]))
      if (sp_i_state == "" | is.na(sp_i_state)) {
        if (verbose) {
          message(spp[i], "lacks info about state - Filter not applicable\n")
        }
        states_final <- occ_i
        states_final$inside_state <- "No info"
      } else {
        states_v <- terra::aggregate(terra::subset(states, grepl(sp_i_state,states$abbrev_state)))
        #Get distance
        distance_i <- terra::distance(occ_i[occ_i$inside_br], states_v,
                                      unit = "km")[,1]
        distance_i <- distance_i <= buffer_state
        #Create columns
        occ_i$inside_state <- NA
        #Fill inside_state of records inside br
        occ_i$inside_state[occ_i$inside_br] <- distance_i
      }
      return(occ_i)
    })
    occ_info <- do.call("rbind", l_state)
  }

  if (!by_country) {
    occ_info$inside_country <- NA
  } else {
    l_country <- lapply(seq_along(spp), function(i) {
      if (verbose) {
        message("Filtering ", spp[i], " by country\n")
      }
      occ_i <- terra::subset(occ_info, occ_info$species ==
                               spp[i])
      sp_i_country <- unique(gsub(";", "|", occ_i$countrycode[1]))
      if (sp_i_country == "" | is.na(sp_i_country)) {
        if (verbose) {
          message(spp[i], "lacks info about country - Filter not applicable\n")
        }
        countrys_final <- occ_i
        countrys_final$inside_country <- "No info"
      } else {
        countrys_v <- terra::aggregate(terra::subset(countrys,
                                       grepl(sp_i_country, countrys$country_code)))
        #Get distance
        distance_i <- terra::distance(occ_i, countrys_v,
                                      unit = "km")[,1]
        distance_i <- distance_i <= buffer_country

        #Create columns
        occ_i$inside_country <- NA
        #Fill inside country
        occ_i$inside_country <- distance_i
      }
      return(occ_i)
    })
    occ_info <- do.call("rbind", l_country)
  }

  occ_flag <- as.data.frame(occ_info)
  colnames(occ_flag)[which(colnames(occ_flag) == "species")] <- species

  if (keep_columns) {
    occ_flag <- merge(occ_flag, occ, by = c(species, "id_f"))
    occ_flag$id_f <- NULL
    occ_flag <- occ_flag[, c(species, long, lat, colnames(occ_flag)[!(colnames(occ_flag) %in%
                                                                        c(species, long, lat))])]
    colnames(occ_flag)[colnames(occ_flag) %in% c(species,
                                                 long, lat)] <- c(species, long, lat)
  }
  if (!keep_columns) {
    occ_flag <- merge(occ_flag, occ[, c(species, lat, long,
                                        "id_f")], by = c(species, "id_f"))
    occ_flag$id_f <- NULL
    occ_flag <- occ_flag[, c(species, long, lat, names(occ_flag)[!(names(occ_flag) %in%
                                                                     c(species, long, lat))])]
    colnames(occ_flag)[colnames(occ_flag) %in% c(species,
                                                 long, lat)] <- c(species, long, lat)
  }

  #Remove columns
  occ_flag$inside_br <- NULL

  if (!by_state) {
    occ_flag$inside_state <- NULL
  }
  if (!by_country) {
    occ_flag$inside_country <- NULL
  }
  col_check <- intersect(c("inside_state", "inside_country",
                           "inside_br"), names(occ_flag))
  if (length(col_check) == 1) {
    occ_flag$filters_ok <- ifelse(occ_flag[, col_check] ==
                                    TRUE | is.na(occ_flag[, col_check]), TRUE, FALSE)
  }  else {
    occ_flag$filters_ok <- ifelse(rowSums(!is.na(occ_flag[,
                                                          col_check]) & occ_flag[, col_check] == FALSE) >
                                    0, FALSE, TRUE)
  }
  occ_clean <- subset(occ_flag, occ_flag$filters_ok == TRUE)
  occ_clean <- subset(occ_clean, select = setdiff(colnames(occ_clean),
                                                  c(col_check, "filters_ok")))
  if (value == "flag&clean") {
    res_final <- list(occ_flag, occ_clean)
    names(res_final) <- c("flagged", "cleaned")
    if (verbose) {
      message("Returning list with flagged and cleaned occurrences\n")
    }
  }
  if (value == "flag") {
    res_final <- occ_flag
    if (verbose) {
      message("Returning dataframe with flagged occurrences\n")
    }
  }
  if (value == "clean") {
    res_final <- occ_clean
    if (verbose) {
      message("Returning dataframe with cleaned occurrences")
    }
  }
  return(res_final)
}

# #Test function
# data("fauna_data") #Load fauna e Funga do Brasil data
# data("occurrences") #Load occurrences
# pts <- subset(occurrences, species == "Panthera onca")
# fd <- filter_faunabr(data = fauna_data,
#                      occ = pts, long = "x", lat = "y", species = "species",
#                      by_state = TRUE, buffer_state = 20,
#                      by_country = TRUE, buffer_country = 20,
#                      value = "flag&clean", keep_columns = TRUE,
#                      verbose = FALSE)

#' Selection of species based on its characteristics and distribution
#'
#' @description select_fauna allows filter species based on its
#' characteristics and distribution available in Brazilian Fauna
#'
#' @param data (data.frame) the data.frame imported with the
#' \code{\link{load_faunabr}} function.
#' @param include_subspecies (logical) include subspecies?
#' Default = FALSE
#' @param phylum (character) The phyla for filtering the dataset. It can
#' be included more than one phylum. Default = "all".
#' @param class (character) The classes for filtering the dataset. It can
#' be included more than one class. Default = "all".
#' @param order (character) The orders for filtering the dataset. It can
#' be included more than one order. Default = "all".
#' @param family (character) The families for filtering the dataset. It can
#' be included more than one family. Default = "all".
#' @param genus (character) The genus for filtering the dataset. It can
#' be included more than one genus. Default = "all".
#' @param lifeForm (character) The life forms for filtering the dataset. It can
#' be included more than one lifeForm. Default = "all"
#' @param filter_lifeForm (character) The type of filtering for life forms. It
#' can be "in", "only", "not_in" and "and". See details for more about this
#' argument.
#' @param habitat (character) The life habitat for filtering the dataset. It can
#' be included more than one habitat. Default = "all"
#' @param filter_habitat (character) The type of filtering for habitat. It
#' can be "in", "only", "not_in" and "and". See details for more about this
#' argument.
#' @param states (character) The states for filtering the dataset. It can
#' be included more than one state. Default = "all".
#' @param filter_states (character) The type of filtering for states. It
#' can be "in", "only", "not_in" and "and". See Details for more about this
#' argument.
#' @param country (character) The country or countries with confirmed
#' occurrences for filtering the dataset. It can be included more than one
#' country. Default = "all".
#' @param filter_country (character) The type of filtering for country. It can
#' be "in", "only", "not_in" and "and". See details for more about this argument.
#' @param origin (character) The origin for filtering the dataset. It can
#' be "native", "introduced", "cryptogenic", "domesticaded" and "invasora".
#' Default = "all".
#' @param taxonomicStatus (character) The taxonomic status for filtering the
#' dataset. It can be "valid", "synonym" or "all".
#' Default = "valid".
#'
#' @details It's possible to choose 4 ways to filter by lifeform, by habitat,
#' by states and by country:
#' "in": selects species that have any occurrence of the determined values. It
#' allows multiple matches. For example, if country = c("brazil", argentina") and
#' filter_country = "in", it will select all species that occur in Brazil and/or
#' Argentina, some of which may also occur in other countries.
#'
#' "only": selects species that have only occurrence of the determined values.
#' It allows only single matches. For example, if
#' country = c("brazil", argentina") and filter_country = "in", it will select
#' all species that occur exclusively in both countries, without any occurrences
#' in other countries.
#'
#' "not_in": selects species that don't have occurrence of the determined
#' values. It allows single and multiple matches. For example,
#' if country = c("brazil", argentina") and filter_country = "not_in", it will select
#' all species without occurrences in Brazil and Argentina.
#'
#' "and": selects species that have occurrence in all determined values. It
#' allows single and multiple matches. For example,
#' if country = c("brazil", argentina") and filter_country = "and", it will select
#' all species that occurs only in both countries,including species that occurs
#' in other countries too.
#'
#' To get the complete list of arguments available for phylum, class, order,
#' family, genus, lifeForm, habitat, states, country and origins, use
#' the function \code{\link{fauna_attributes}}
#'
#' @return A new dataframe with the filtered species.
#' @usage select_fauna(data, include_subspecies = FALSE, phylum = "all",
#'                       class = "all", order = "all", family = "all",
#'                       genus = "all",
#'                       lifeForm = "all", filter_lifeForm = "in",
#'                       habitat = "all", filter_habitat = "in",
#'                       states = "all", filter_states = "in",
#'                       country = "all", filter_country = "in",
#'                       origin = "all", taxonomicStatus = "valid")
#' @export
#' @references
#' Brazilian Zoology Group. Catálogo Taxonômico da Fauna do Brasil. Available at:
#' https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil
#'
#' @examples
#' data("fauna_data") #Load data example
#' #Select endemic and native species of birds (Aves) with confirmed occurrence
#' #in Brazil or Argentina
#' aves_br_ar <- select_fauna(data = fauna_data, include_subspecies = FALSE,
#'                            phylum = "all", class = "Aves",
#'                            order = "all",
#'                            family = "all",
#'                            genus = "all",
#'                            lifeForm = "all", filter_lifeForm = "in",
#'                            habitat = "all", filter_habitat = "in",
#'                            states = "all", filter_states = "in",
#'                            country = c("BR", "AR"),
#'                            filter_country = "in",
#'                            origin = "native",
#'                            taxonomicStatus = "valid")
select_fauna <- function(data, include_subspecies = FALSE,
                         phylum = "all", class = "all", order = "all",
                         family = "all",
                         genus = "all",
                         lifeForm = "all", filter_lifeForm = "in",
                         habitat = "all", filter_habitat = "in",
                         states = "all", filter_states = "in",
                         country = "all", filter_country = "in",
                         origin = "all",
                         taxonomicStatus = "valid") {
  if (missing(data)) {
    stop("Argument data is not defined")
  }

  #Check classes
  if (!inherits(data, "data.frame")) {
    stop(paste0("Argument data must be a data.frame, not ", class(data)))
  }


  if (!is.logical(include_subspecies)) {
    stop(paste0("Argument include_subspecies must be logical, not ",
                class(include_subspecies)))
  }

  #Correct characters, if necessary
    phylum <- firstup(trimws(phylum))
    phylum[phylum == "All"] <- "all"
    class <- firstup(trimws(class))
    class[class == "All"] <- "all"
    order <- firstup(trimws(order))
    order[order == "All"] <- "all"
    family <- firstup(trimws(family))
    family[family == "All"] <- "all"
    genus <- firstup(trimws(genus))
    genus[genus == "All"] <- "all"
    lifeForm <- tolower(trimws(lifeForm))
    habitat <- tolower(trimws(habitat))
    states <- toupper(trimws(states))
    states[states == "ALL"] <- "all"
    country <- toupper(trimws(country))
    country[country == "ALL"] <- "all"
    origin <- tolower(trimws(origin))
    taxonomicStatus <- tolower(trimws(taxonomicStatus))

  #Check available arguments#
  if(!(filter_lifeForm %in% c("in", "only", "not_in", "and"))) {
    stop(paste0("Argument filter_lifeForm must be:\n",
                "'in', 'only', 'not_in' or 'and'"))
  }

  if(!(filter_habitat %in% c("in", "only", "not_in", "and"))) {
    stop(paste0("Argument filter_habitat must be:\n",
                "'in', 'only', 'not_in' or 'and'"))
  }

  if(!(filter_country %in% c("in", "only", "not_in", "and"))) {
      stop(paste0("Argument filter_country must be:\n",
                  "'in', 'only', 'not_in' or 'and'"))
  }

  if(!(filter_states %in% c("in", "only", "not_in", "and"))) {
      stop(paste0("Argument filter_country must be:\n",
                  "'in', 'only', 'not_in' or 'and'"))
  }

  if(all(phylum != "all") & !(phylum %in% unique(data$phylum))) {
    stop(paste("phylum not valid. The phylums availables are:\n",
               paste(unique(data$phylum), collapse = ", ")))  }

  # if(all(class != "all") & !(class %in% unique(data$class))) {
  #   stop(paste("class not valid. Check the available classes with the function
  #              get_fauna_att()"))  }

  if(all(order != "all") & !(order %in% unique(data$order))) {
    stop(paste("order not valid. Check the available orders with the function
               get_fauna_att()"))  }

  if(all(family != "all") & !all(family %in% unique(data$family))) {
    stop(paste("Family not valid.\n",
               "Check the available families with the function
               get_fauna_att()")
         ) }

  if(genus != "all" & !(genus %in% unique(data$genus))) {
    stop(paste("Genus not valid.\n")) }


  if(all(origin != "all") & !any(origin %in% unique(data$origin))) {
    stop(paste("origin not valid. The options availables are:\n",
               paste(unique(data$origin), collapse = ", ")))
    }

  if(taxonomicStatus != "all" & !any(taxonomicStatus %in% unique(data$taxonomicStatus))) {
    stop(paste("taxonomicStatus not valid. The options availables are:\n",
               paste(unique(data$taxonomicStatus), collapse = ", ")))
    }

  #Start to filter...
    #Taxon Rank
    if(!include_subspecies) {
      d <- subset(data, data$taxonRank %in% c("species", "especie")) }
    if(include_subspecies) {
      d <- subset(data, data$taxonRank %in% c("species", "especie",
                                              "subspecies", "sub_especie")) }

    #phylum
    if(all(phylum != "all")) {
    d <- d[which(d$phylum %in% phylum),]}

    #class
    if(all(class != "all")) {
    d <- d[which(d$class %in% class),]}

    #order
    if(all(order != "all")) {
      d <- d[which(d$order %in% order),]}

    #family
    if(all(family != "all")) {
      d <- d[which(d$family %in% family),]}

    #genus
    if(all(genus != "all")) {
      d <- d[which(d$genus %in% genus),]}


  #lifeForm ####
  if(all(lifeForm  == "all")) {
    d <- d }

  #Check if it is a valid lifeForm
  if(all(lifeForm != "all")) {
    all_lf <- unique(unlist(strsplit(d$lifeForm, split = ";")))
    newlifeForm <- gsub(" ", "", lifeForm)
    newlifeForm <- vapply(lifeForm, FUN.VALUE = character(1), function(x){
      paste(sort(gsub(" ", "", unlist(strsplit(x, split = ",")))),
            collapse = ";")
    }, USE.NAMES = FALSE)
    newlifeForm <- sort(newlifeForm)
    #Check if all lifeform exists
    newlifeForm2 <- unique(unlist(strsplit(newlifeForm, split = ";")))
    any_diff <- setdiff(newlifeForm2 , all_lf)
    if(length(any_diff) > 0) {
      warning(paste("The following life forms are not valid:\n",
                    paste(any_diff, collapse = ", ")))
    }
  }

  #Filter by lifeform
  if(all(lifeForm != "all") & filter_lifeForm == "in") {
    d <- subset(d, grepl(paste(newlifeForm, collapse = "|"),
                         d$lifeForm)) }

  if(all(lifeForm != "all") & filter_lifeForm == "only") {
    d <- subset(d, d$lifeForm == paste(newlifeForm, collapse = ";"))
  }

  if(all(lifeForm != "all") & filter_lifeForm == "not_in") {
    d <- subset(d, !grepl(paste(newlifeForm, collapse = "|"),
                          d$lifeForm))
  }

  if(all(lifeForm != "all") & filter_lifeForm == "and") {
    d <- d[rowSums(sapply(newlifeForm, function(x)
      grepl(x, d$lifeForm))) == length(newlifeForm),]
    }


  #habitat ####
  if(all(habitat  == "all")) {
    d <- d }

  #Check if it is a valid habitat
  if(all(habitat != "all")) {
    all_hab <- unique(unlist(strsplit(d$habitat, split = ";")))
    newhabitat <- gsub(" ", "", habitat)
    newhabitat <- vapply(habitat, FUN.VALUE = character(1), function(x){
      paste(sort(gsub(" ", "", unlist(strsplit(x, split = ",")))),
            collapse = ";")
    }, USE.NAMES = FALSE)
    newhabitat <- sort(newhabitat)
    #Check if all habitat exists
    newhabitat2 <- unique(unlist(strsplit(newhabitat, split = ";")))
    any_diff <- setdiff(newhabitat2 , all_hab)
    if(length(any_diff) > 0) {
      warning(paste("The following habitats are not valid:\n",
                    paste(any_diff, collapse = ", ")))
    }
  }

  #Filter by habitat
  if(all(habitat != "all") & filter_habitat == "in") {
    d <- subset(d, grepl(paste(newhabitat, collapse = "|"),
                         d$habitat)) }

  if(all(habitat != "all") & filter_habitat == "only") {
    d <- subset(d, d$habitat == paste(newhabitat, collapse = ";"))
  }

  if(all(habitat != "all") & filter_habitat == "not_in") {
    d <- subset(d, !grepl(paste(newhabitat, collapse = "|"),
                          d$habitat))
  }

  if(all(habitat != "all") & filter_habitat == "and") {
    d <- d[rowSums(sapply(newhabitat, function(x)
      grepl(x, d$habitat))) == length(newhabitat),]
    }

  #states ####
  if(all(states  == "all")) {
    d <- d}

  #Check if it is a valid state
  if(all(states != "all")) {
    all_state <- unique(unlist(strsplit(d$states, split = ";")))
    newstate <- gsub(" ", "", states)
    newstate <- vapply(states, FUN.VALUE = character(1), function(x){
      paste(sort(gsub(" ", "", unlist(strsplit(x, split = ",")))),
            collapse = ";")
    }, USE.NAMES = FALSE)
    newstate <- sort(newstate)
    #Check if all state exists
    newstate2 <- unique(unlist(strsplit(newstate, split = ";")))
    any_diff <- setdiff(newstate2 , all_state)
    if(length(any_diff) > 0) {
      warning(paste("The following states are not valid:\n",
                    paste(any_diff, collapse = ", ")))
    }
  }


  #Filter by state
  if(all(states != "all") & filter_states == "in") {
    d <- subset(d, grepl(paste(newstate, collapse = "|"),
                         d$states)) }

  if(all(states != "all") & filter_states == "only") {
    d <- subset(d, d$states == paste(newstate, collapse = ";"))
  }

  if(all(states != "all") & filter_states == "not_in") {
    d <- subset(d, !grepl(paste(newstate, collapse = "|"),
                          d$states))

  }

  if(all(states != "all") & filter_states == "and") {
    d <- d[rowSums(sapply(newstate, function(x)
      grepl(x, d$states))) == length(newstate),]
    }

    #Check if it is a valid country
    if(all(country != "all")) {
      all_country <- unique(unlist(strsplit(d$countryCode, split = ";")))
      newcountry <- gsub(" ", "", country)
      newcountry <- vapply(country, FUN.VALUE = character(1), function(x){
        paste(sort(gsub(" ", "", unlist(strsplit(x, split = ",")))),
              collapse = ";")
      }, USE.NAMES = FALSE)
      newcountry <- sort(newcountry)
      #Check if all country exists
      newcountry2 <- unique(unlist(strsplit(newcountry, split = ";")))
      any_diff <- setdiff(newcountry2 , all_country)
      if(length(any_diff) > 0) {
        warning(paste("The following country/countries is/are not valid:\n",
                      paste(any_diff, collapse = ", ")))
      }
    }


    #Filter by country
    if(all(country != "all") & filter_country == "in") {
      d <- subset(d, grepl(paste(newcountry, collapse = "|"),
                           d$countryCode)) }

    if(all(country != "all") & filter_country == "only") {
      d <- subset(d, d$countryCode == paste(newcountry, collapse = ";"))
    }

    if(all(country != "all") & filter_country == "not_in") {
      d <- subset(d, !grepl(paste(newcountry, collapse = "|"),
                            d$countryCode))

    }

    if(all(country != "all") & filter_country == "and") {
      d <- d[rowSums(sapply(newcountry, function(x)
        grepl(x, d$countryCode))) == length(newcountry),]
    }

  #origin ####
  if(all(origin == "all")) {
    d <- d }

  #Filter by origin
  if(all(origin != "all")) {
    all_origin <- unique(d$origin)
    any_diff <- setdiff(origin, all_origin)
    if(length(any_diff) > 0) {
      warning(paste("The following origins are not available:\n",
                    paste(any_diff, collapse = ", ")))
    }
    d <- d[d$origin %in% origin,] }

  #taxonomicStatus ####
  if(all(taxonomicStatus == "all")) {
    d <- d }

  #Filter by taxonomicStatus
  if(all(taxonomicStatus != "all")) {
    taxonomicStatus2 <- taxonomicStatus
    all_taxonomicStatus <- unique(d$taxonomicStatus)
    any_diff <- setdiff(taxonomicStatus, all_taxonomicStatus)
    if(length(any_diff) > 0) {
      warning(paste("The following taxonomicStatus are not valid:\n",
                    paste(any_diff, collapse = ", ")))
    }
    d <- subset(d, d$taxonomicStatus %in% taxonomicStatus2) }
  if(nrow(d) == 0) {
    warning("Combination of characteristics return 0 species")
  }

  return(d)
} #End of function

#' Get a presence-absence matrix
#' @description
#'  Get a presence-absence matrix of species based on its distribution
#' (brazilian states and/or countries) according to Fauna do Brasil.
#'
#' @param data (data.frame) a data.frame imported with the
#' \code{\link{load_faunabr}} function or generated by either
#' \code{\link{select_fauna}} or \code{\link{subset_fauna}} functions
#' @param by_state (logical) get occurrences by State. Default = TRUE
#' @param by_country (logical) get occurrences by countries. Default = FALSE
#' @param remove_empty_sites (logical) remove empty sites (sites without any
#' species) from final presence-absence matrix. Default = TRUE
#' @param return_richness_summary (logical) return a data.frame with the number
#' of species in each site. Default = TRUE
#' @param return_spatial_richness (logical) return a SpatVector with the number
#' of species in each site. Default = TRUE
#' @param return_plot (logical) plot map with the number of species in each
#' site.
#' Only works if return_spatial_richness = TRUE. Default = TRUE
#'
#' @return If return_richness_summary and/or return_spatial_richness is set to
#' TRUE, return a list with:
#' - PAM: the presence-absence matrix (PAM)
#' - Richness_summary: a data.frame with the number of species in each site
#' - Spatial_richness: a SpatVector with the number of species in each site
#' (by State and/or country)
#'
#' If return_richness_summary and return_spatial_richness is set to FALSE,
#' return a presence-absence matrix
#' @usage fauna_pam(data, by_state = TRUE, by_country= FALSE,
#'                remove_empty_sites = TRUE,
#'                return_richness_summary = TRUE,
#'                return_spatial_richness = TRUE,
#'                return_plot = TRUE)
#' @importFrom terra merge plot intersect unwrap
#' @importFrom stats quantile
#' @importFrom grDevices terrain.colors
#' @export
#' @examples
#' #Test function
#' data("fauna_data") #Load fauna e Funga do Brasil data
#' #Select native species of mammals with occurrence only in Brazil
#' br_mammals <- select_fauna(data = fauna_data,
#'                            include_subspecies = FALSE, phylum = "all",
#'                            class = "Mammalia",
#'                            order = "all", family = "all",
#'                            genus = "all",
#'                            lifeForm = "all", filter_lifeForm = "in",
#'                            habitat = "all", filter_habitat = "in",
#'                            states = "all", filter_states = "in",
#'                            country = "BR", filter_country = "only",
#'                            origin = "all", taxonomicStatus = "valid")
#' #Get presence-absence matrix in states
#' pam_mammals <- fauna_pam(data = br_mammals, by_state = TRUE,
#'                          by_country = FALSE,
#'                          remove_empty_sites = TRUE,
#'                          return_richness_summary = TRUE,
#'                          return_spatial_richness = TRUE,
#'                          return_plot = TRUE)
fauna_pam <- function(data, by_state = TRUE, by_country = FALSE,
                      remove_empty_sites = TRUE,
                      return_richness_summary = TRUE,
                      return_spatial_richness = TRUE,
                      return_plot = TRUE) {
  if (missing(data)) {
    stop("Argument data is not defined")
  }

  if (!inherits(data, "data.frame")) {
    stop(paste0("Argument data must be a data.frame, not ", class(data)))
  }

  if (!is.logical(by_state)) {
    stop(paste0("Argument by_state must be logical, not ", class(by_state)))
  }

  if (!is.logical(by_country)) {
    stop(paste0("Argument by_country must be logical, not ",
                class(by_country)))
  }

  if (!is.logical(remove_empty_sites)) {
    stop(paste0("Argument remove_empty_sites must be logical, not ",
                class(remove_empty_sites)))
  }

  if (!is.logical(return_spatial_richness)) {
    stop(paste0("Argument return_spatial_richness must be logical, not ",
                class(return_spatial_richness)))
  }

  #Check colnames in data
  if(!all(c("species", "states", "countryCode") %in%
          colnames(data))) {
    stop("Important columns are missing in data. Check if data is an object
         created by 'load_faunabr()', 'subset_fauna()' or 'select_fauna()'")
  }

  #Check if there is at least one TRUE in states or countries
  if(!by_state & !by_country){
    stop("At least one of the parameters by_state or by_country must be TRUE")
  }

  # Return_spatial_richnessnly works if by_state or by_country is set to TRUE
  if(return_spatial_richness & !by_state & !by_country){
    stop("return_spatial_richness=TRUE only works if by_state or/and by_country is set to TRUE")
  }

  #Get columns
  columns <- c("species")
  if (by_country) {
    columns <- c(columns, "countryCode")
  }
  if (by_state) {
    columns <- c(columns, "states")
  }
  d <- data[, columns, drop = FALSE]

  #Create list of unique values
  v <- colnames(d)[colnames(d)!= "species"]
  l <- lapply(v, function(i){
    unique(unlist(strsplit(d[,i], ";")))
  })
  names(l) <- v

  # All combinations of state/countries
  sites <- expand.grid(l)
  #Remove NA from states
  if(by_state){
    sites <- subset(sites, sites$states != "NA") }


  if(by_state & by_country){
    sites$states[sites$countryCode != "BR"] <- NA
    sites <- unique(sites)
  }

  # Create an empty presence-absence matrix
  presence_matrix <- matrix(0, nrow = nrow(sites), ncol = nrow(d))


  # Fill matrix with values of presence (1) and absence (0)
  for (i in 1:nrow(d)) {
    species_i <- d[i, "species"]

    #Get index
    site_index <- lapply(v, function(z){
      which(sites[,z] %in% unlist(strsplit(
        d[,z][which(d$species == species_i)], ";")))
    })
    site_index <- unique(unlist(site_index))
    presence_matrix[site_index, i] <- 1
  }

  # Name species in the column names
  colnames(presence_matrix) <- d$species

  #Append sites
  pam <- cbind(sites, presence_matrix)

  #Remove empty sites
  if(remove_empty_sites){
    remove_sites <- which(rowSums(pam[,-match(colnames(sites),
                                              colnames(pam))]) == 0)
    if(length(remove_sites) > 0){
      pam <- pam[-remove_sites,]
    }
  }


  ####Richness summary####
  if(return_richness_summary){
    r_sum <- cbind(pam[, intersect(names(pam), v), drop = FALSE],
                   richness = rowSums(pam[, setdiff(names(pam), v), drop = FALSE]))}

  ####Spatialize richness####
  if(return_spatial_richness) {
    #Load data
    if(by_state & !by_country) {
      m <- terra::unwrap(faunabr::states)
      names(m)[2] <- "states"
    }
    if(by_country & !by_state) {
      m <- terra::unwrap(faunabr::world_fauna)
      names(m)[3] <- "countryCode"
    }

    #If by_country and by_country, merge polygons
    if(by_country & by_state){
      m_world <- terra::simplifyGeom(terra::unwrap(faunabr::world_fauna))
      names(m_world)[3] <- "countryCode"
      m_states <- terra::simplifyGeom(terra::unwrap(faunabr::states))
      names(m_states)[2] <- "states"
      m <- terra::union(m_world, m_states)
      m$states[m$country_code != "BR"] <- NA
    }

    #Get columns
    id_pam <- unique(pam[, v, drop = FALSE])

    #Calculate richness
    r <- rowSums(pam[,d$species])

    #Create and transfer ID
    pam_m <- pam
    pam_m$site_id <- 1:nrow(pam_m)

    #Get new dataframe with richness by site
    pam_m <- cbind(pam_m[, v, drop = FALSE], richness = r)

    #####Create complete combination of states and countries####
    pam_m <- merge(m[[v]], pam_m, by = v, all.x = TRUE, all.y = FALSE)
    pam_m$richness[is.na(pam_m$richness)] <- 0 #Remove NA

    #Merge with spatial data
    m <- terra::merge(m, pam_m, by = v, na.rm = FALSE)

    if(return_plot){
      #Get unique values
      unique_values <- unique(pam_m$richness)

      n_breaks <- ifelse(length(unique_values) <= 10, length(unique_values), 10)
      set_breaks <- round(stats::quantile(1:max(pam_m$richness, na.rm = TRUE),
                                    probs = seq(0, 1, length.out = n_breaks + 1)),0)
      plot_title <- ifelse(by_state & !by_country, "Richness by State", ifelse(
        !by_state & by_country, "Richness by country", ifelse(
          by_state & by_country, "Richness by State and country", NA)
      ))

      if(n_breaks >= 10){
        terra::plot(m, "richness", breaks = set_breaks,
                    col = rev(grDevices::terrain.colors(length(set_breaks))),
                    main = plot_title) }    else{
        terra::plot(m, "richness", col = rev(grDevices::terrain.colors(n_breaks)),
                    main = plot_title)
      }
    }
  } #End of return_spatial_richness

  #Return final data
  if(!return_richness_summary) {
    r_sum <- NA
  }
  if(!return_spatial_richness){
    m <- NA
  }


  #Crate final list
  res <- list(PAM = pam,
              Richness_summary = r_sum,
              Spatial_richness = m)

  res <- res[!is.na(res)]

  if(length(res) > 1){
    return(res)
  } else {
    return(pam)
  }
}#End of function

# #Test function
# data("fauna_data") #Load fauna e Funga do Brasil data
# #Select native species of mammals with occurrence only in Brazil
# br_mammals <- select_fauna(data = fauna_data,
#                          include_subspecies = FALSE, phylum = "all",
#                          class = "Mammalia",
#                          order = "all", family = "all",
#                          genus = "all",
#                          lifeForm = "all", filter_lifeForm = "in",
#                          habitat = "all", filter_habitat = "in",
#                          states = "all", filter_states = "in",
#                          country = "brazil", filter_country = "only",
#                          origin = "all", taxonomicStatus = "valid")
# #Get presence-absence matrix in states
# pam_mammals <- fauna_pam(data = br_mammals, by_state = TRUE, by_country = FALSE,
#                          remove_empty_sites = TRUE,
#                          return_richness_summary = TRUE,
#                          return_spatial_richness = TRUE,
#                          return_plot = TRUE)

#' Extract the binomial name (Genus + specific epithet) from a Scientific Name
#'
#' @param species_names (character) Scientific names to be converted to
#' binomial names
#'
#' @return A vector with the binomial names (Genus + specific epithet).
#' @usage extract_binomial(species_names)
#' @export
#'
#' @examples
#' spp <- c("Panthera onca (Linnaeus, 1758)",
#' "Zonotrichia capensis subtorquata Swainson, 1837",
#' "Paraganaspis egeria Díaz & Gallardo, 1996",
#' "Arrenurus tumulosus intercursor")
#' spp_new <- extract_binomial(species_names = spp)
#' spp_new
#'
extract_binomial <- function(species_names) {
  if (!is.character(species_names)) {
    stop(paste0("Argument species_names must be a character, not ",
                class(species_names)))
  }

  # Remove excess of whitespace between words
  species_names <- gsub("\\s+", " ", species_names)

  # Remove leading and/or trailing whitespace
  species_names <- trimws(species_names)

  selected_species_names <- vapply(species_names, FUN.VALUE = character(1), function(text) {
    # Split words
    words <- strsplit(text, " ")[[1]]
    word_count <- length(words)

    # Check if there are parentesis and if they are after Genus
    if (word_count > 2 && grepl("\\(", words[2])) {
      # Find position
      close_paren_index <- which(grepl("\\)", words))

      # If there are parentesis, keep them
        if (length(close_paren_index) > 0) {
        selected_words <- paste(words[1:close_paren_index], collapse = " ")
        # Adicionar mais uma palavra após os parênteses fechados se houver
        if (word_count >= close_paren_index + 1) {
          selected_words <- paste(selected_words, words[close_paren_index + 1])
        }
        return(selected_words)
      }
    }

    # If not, keep only first two words
    if (word_count > 2) {
      selected_words <- paste(words[1:2], collapse = " ")
      return(selected_words)
    } else {
      return(text)
    }
  })

  names(selected_species_names) <- NULL
  return(selected_species_names)
}

#' Catálogo Taxonômico da Fauna do Brasil database - Version 1.17
#'
#' @description
#'  A dataset containing a subset of the Catálogo Taxonômico da Fauna do Brasil
#'  database (version 1.17)
#'
#' @usage data(fauna_data)
#'
#' @format A \code{data.frame} with 9558 rows and 19 variables:
#' \describe{
#'   \item{species}{Species names}
#'   \item{subspecies}{Subspecies names}
#'   \item{scientificName}{Complete scientific name of the species}
#'   \item{validName}{Valid name of the species (NA when the name in
#'   species is already a valid name)}
#'   \item{kingdom}{Kingdom to which species belongs (Animalia)}
#'   \item{phylum}{Phylum to which species belongs}
#'   \item{class}{Class to which species belongs}
#'   \item{order}{Order to which species belongs}
#'   \item{family}{Family to which species belongs}
#'   \item{genus}{Genus to which species belongs}
#'   \item{lifeForm}{Life form of the species (e.g: free_living_individual,
#'   colonial, sessile, etc.)}
#'   \item{habitat}{Habitat type of the species (e.g., terrestrial, arboreal,
#'   freshwater, etc.)}
#'   \item{states}{Federal states with confirmed occurrences of the species}
#'   \item{countryCode}{Countries with confirmed occurrences of the species}
#'   \item{origin}{Indicates whether the species is native, introduced,
#'   domesticated, cryptogenic or invasive}
#'   \item{taxonomicStatus}{Indicates the level of recognition and acceptance
#'   of the species (valid or synonym)}
#'   \item{nomenclaturalStatus}{Indicates the legitimacy and validity of the
#'   species name (original_combination, changed_combination, etc.)}
#'   \item{vernacularName}{Locally or culturally used name for the species}
#'   \item{taxonRank}{Taxonomic rank (Species, Genus, Family, Order, etc). This
#'   data contains only Species}
#' }
#' @references
#' Brazilian Zoology Group. Catálogo Taxonômico da Fauna do Brasil. Available at:
#' https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil
"fauna_data"

#' SpatVector of the federal states of Brazil
#'
#' @description
#'  A simplified and packed SpatVector of the polygons of the federal states of
#'  Brazil. The spatial data was originally obtained from
#'  \code{geobr::read_state}. Borders have been simplified by removing vertices
#'  of borders using \code{terra::simplifyGeom}. It's necessary unpack the
#'  Spatvectos using \code{terra::unwrap}
#'
#'  @usage data(states)
#'  states <- terra::unwrap(states)
#'
#' @format A \code{SpatVector} with 27 geometries and 3 attributes:
#' \describe{
#'   \item{abbrev_state}{State acronym}
#'   \item{name_state}{State's full name}
#'   \item{name_region}{The region to which the state belongs}
#'   }
"states"

#' SpatVector of the world countries
#'
#' @description
#'  A simplified and packed SpatVector of the world country polygons. The
#'  spatial data was originally obtained from \code{rnaturalearth::ne_countries}.
#'  Borders have been simplified by removing vertices of borders using
#'  \code{terra::simplifyGeom}.It's necessary unpack the Spatvectos using
#'  \code{terra::unwrap}
#'
#'  @usage data(world_fauna)
#'  biomes <- terra::unwrap(world_fauna)
#'
#' @format A \code{SpatVector} with 258 geometries and 1 attribute:
#' \describe{
#'   \item{name}{The name of the country (argentina, brazil, colombia, etc.)}
#'   }
"world_fauna"

#' Records of animal species
#'
#' @description
#'  A dataset containing records of 2 species downloaded from GBIF, with
#'  additional fake data. The records were obtained with \code{plantR::rgbif2}
#'
#' @usage data(occurrences)
#'
#' @format A \code{data.frame} with 2798 rows and 3 variables:
#' \describe{
#'   \item{species}{Species names (Panthera onca and Chaetomys subspinosus)}
#'   \item{x}{Longitude}
#'   \item{y}{Latitude}
#'   \item{source}{record downloaded from GBIF or fake data}
#' }
#' @references GBIF, 2024. florabr R package: Records of plant species. https://doi.org/10.15468/DD.QPGEB7
"occurrences"

#' Country Codes and Names
#'
#' @description
#' A dataset containing country codes used in the Catálogo Taxonômico da Fauna
#' do Brasil along with their corresponding country names, as defined in
#' \code{faunabr::world_fauna}.
#'
#' @usage data(country_codes)
#'
#' @format A \code{data.frame} with 244 rows and 2 variables:
#' \describe{
#'   \item{map_name}{Country names as defined in \code{faunabr::world_fauna}.}
#'   \item{country_code}{Country codes used in the Catálogo Taxonômico da Fauna
#'   do Brasil.}
#' }
"country_codes"

#' Helpers for translating data
#'
#' @description
#' A list of data.frames used by \code{faunabr::translate_faunabr()} function.
#' \code{faunabr::map_translation}.
#'
#' @usage data(map_translation)
#'
#' @format A \code{list} with 5 data.frames ("lifeForm", "origin", "habitat",
#' "taxonRank", and "taxonomicStatus"). Each data.frame has 2 columns:
#' \describe{
#'   \item{pt_br}{The attribute in Brazilian Portuguese.}
#'   \item{en}{The attribute in English.}
#' }
"map_translation"

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message  = FALSE,
  eval = FALSE,
  fig.width = 8,
  fig.height = 6
)

## ----Get data, results='hide', message=FALSE, warning=FALSE, echo=FALSE-------
# library(faunabr) #Load package
# my_dir <- file.path(file.path(tempdir(), "faunabr"))
# dir.create(my_dir)
# get_faunabr(output_dir = my_dir, #directory to save the data
#             data_version = "latest", #get the most recent version available
#             overwrite = T) #Overwrite data, if it exists

## ----message=FALSE, warning=F-------------------------------------------------
# library(terra)
# #Load data
# bf <- load_faunabr(data_dir = my_dir, #Folder where you stored the data with the function get_faunabr()
#                    data_version = "latest",
#                    type = "short") #short version
# #> Loading version 1.3

## -----------------------------------------------------------------------------
# #Example species
# spp <- c("Panthera onca", "Mazama jucunda")
# #Get spatial polygons
# spp_spt <- fauna_spat_occ(data = bf, species = spp, state = TRUE,
#                           country = TRUE, verbose = TRUE)
# #> Getting states of Panthera onca
# #> Getting countries of Panthera onca
# #> Getting states of Mazama jucunda
# #> Getting countries of Mazama jucunda

## -----------------------------------------------------------------------------
# par(mfrow = c(3, 2), mar = c(2, 0, 2, 0))
# plot(spp_spt$`Panthera onca`$states,
#      main = paste0(names(spp_spt)[[1]], " - States"), mar = NA)
# plot(spp_spt$`Panthera onca`$countries,
#      main = paste0(names(spp_spt)[[1]], " - Countries"), mar = NA)
# plot(spp_spt$`Mazama jucunda`$states,
#      main = paste0(names(spp_spt)[[2]], " - States"), mar = NA)
# plot(spp_spt$`Mazama jucunda`$countries,
#      main = paste0(names(spp_spt)[[2]], " - Countries"), mar = NA)

## ----IMG01, eval=TRUE, echo=FALSE---------------------------------------------
knitr::include_graphics("vignettes_img/IMG01.png")

## -----------------------------------------------------------------------------
# my_spp <- c("Panthera onca", "Chaetomys subspinosus")
# pol_spp <- fauna_spat_occ(data = bf, species = my_spp,
#                        state = TRUE, country = TRUE,
#                        verbose = TRUE)
# par(mfrow = c(2, 2), mar = c(2, 0, 2, 0))
# plot(pol_spp$`Panthera onca`$states,
#      main = paste0(names(pol_spp)[[1]], " - States"), mar = NA)
# plot(pol_spp$`Panthera onca`$countries,
#      main = paste0(names(pol_spp)[[1]], " - Countries"), mar = NA)
# plot(pol_spp$`Chaetomys subspinosus`$states,
#      main = paste0(names(pol_spp)[[2]], " - States"), mar = NA)
# plot(pol_spp$`Chaetomys subspinosus`$countries,
#      main = paste0(names(pol_spp)[[2]], " - Countries"), mar = NA)

## ----IMG02, eval=TRUE, echo = FALSE-------------------------------------------
knitr::include_graphics("vignettes_img/IMG02.png")

## -----------------------------------------------------------------------------
# data("occurrences")
# head(occurrences)
# #>         species         x          y  source
# #> 1 Panthera onca -90.38409  17.377023    gbif
# #> 2 Panthera onca -90.24368  17.240507    gbif
# #> 3 Panthera onca -77.36680   0.287624    gbif
# #> 4 Panthera onca -56.61023 -17.239688    gbif
# #> 5 Panthera onca -61.04386  -2.387029    gbif
# #> 6 Panthera onca -77.28850   0.288757    gbif

## -----------------------------------------------------------------------------
# occ_check <- filter_faunabr(data = bf, occ = occurrences,
#                     by_state = TRUE, buffer_state = 20,
#                     by_country = TRUE, buffer_country = 20,
#                     value = "flag&clean", keep_columns = TRUE,
#                     verbose = FALSE)
# #> Returning list with flagged and cleaned occurrences

## ----warning=FALSE, results='hide'--------------------------------------------
# #Install mapview if necessary and load package
# if(!require(mapview)){
#     install.packages("mapview")
# }
# #Load mapview
# library(mapview)

## ----fig.width= 9-------------------------------------------------------------
# #Convert points to spatvector
# panthera_occ <- subset(occ_check$flagged,
#                        occ_check$flagged$species == "Panthera onca")
# panthera_occ <- vect(panthera_occ, geom = c("x", "y"),
#                      crs = crs(pol_spp$`Panthera onca`$states))
# 
# #Iteractive plot
# mapview(pol_spp$`Panthera onca`$states,
#         layer.name = "States with confirmed occurrences") +
#   mapview(panthera_occ, zcol = "inside_state", layer.name = "Inside state?",
#           col.regions = c("red", "green"))

## ----IMG03, eval=TRUE, echo = FALSE, fig.pos="H", out.width="90%"-------------
knitr::include_graphics("vignettes_img/IMG03.png")

## ----fig.width= 9-------------------------------------------------------------
# #Iteractive plot
# mapview(pol_spp$`Panthera onca`$countries,
#         layer.name = "Countries with confirmed occurrences", burst = TRUE, legend = F) +
#   mapview(panthera_occ, zcol = "inside_country",
#           col.regions = c("red", "green"),
#           layer.name = "Inside country?")

## ----IMG04, eval=TRUE, echo = FALSE, fig.pos="H", out.width="90%"-------------
knitr::include_graphics("vignettes_img/IMG04.png")

#' Translate information in Brazilian Fauna database
#'
#' @description
#' This function translates information in the "lifeForm", "origin", "habitat",
#' "taxonRank", and "taxonomicStatus" columns between Portuguese and English.
#'
#' @param data (data.frame) the data.frame imported with the
#' \code{\link{load_faunabr}} function.
#' @param map_list (list) A list of data.frames used for translation. The default
#'   is NULL, which means it uses `faunabr::map_translation`. If not NULL, its
#'   structure (list names and data.frame column names) must be identical
#'   to `faunabr::map_translation`.
#' @param to (character) The target language for translation. Available options
#'   are "en" to translate from Portuguese to English, and "pt_br" to translate
#'   from English to Portuguese. The default is "en".
#'
#' @return A data.frame with the values in the "lifeForm", "origin", "habitat",
#'   "taxonRank", and "taxonomicStatus" columns translated.
#' @export
#'
#' @examples
#' data("fauna_data") #Load data example (in English)
#' #Translate to Portuguese
#' fauna_portugues <- translate_faunabr(data = fauna_data, to = "pt_br")
#' # See attributes of lifeForm in Portuguese
#' fauna_attributes(fauna_portugues, attribute = "lifeForm")
#'
translate_faunabr <- function(data, map_list = NULL, to = "en"){
  # Test data
  if (!is.data.frame(data)) {
    stop("data must be a 'data.frame', not ", class(data))
  }

  if(!(to %in% c("en", "pt_br"))){
    stop("'to' must be 'en' or 'pt_br'")
  }

  if("language" %in% colnames(data)){
    if(to == unique(data$language)){
      stop("The data is already in '", to, "'")
  }}

  if(!is.null(map_list)){
    if(!inherits(map_list, "list")){
      stop("map_list must be a 'list' with data.frames, not ", class(map_list))
    }
    names_out <- setdiff(c("lifeForm", "origin", "habitat", "taxonRank"),
                         names(map_list))
    if(length(names_out) > 0){
      stop("At least one of the data.frames within 'map_list' must be named as:
      'lifeForm', 'origin', 'habitat', or 'taxonRank'")
    }
  }

  if(is.null(map_list))
    map_list <- faunabr::map_translation

  # Iterate on 'map_list' names
  for (col_name in names(map_list)) {
    # Check if columns exists
    if (col_name %in% names(data) && !is.null(map_list[[col_name]])) {

      current_map <- map_list[[col_name]]

      # Apply function to translate
      data[[col_name]] <- translate_column_terms(
        text = data[[col_name]],
        map_df = current_map,
        to = to,
        sep = ";"
      )

      # Replace empty strings with NA
      data[[col_name]][data[[col_name]] == ""] <- NA

      #Replace NA with unknown in taxonRank
      if (col_name == "taxonRank" && to == "en") { # Assume que "unknown" é a tradução para NA em inglês
        data[[col_name]][is.na(data[[col_name]])] <- "unknown"
      } else if (col_name == "taxonRank" && to == "pt_br") { # Se traduzindo de EN para PT e tem "unknown"
        data[[col_name]][is.na(data[[col_name]])] <- "desconhecido"
      }
             }
  }

  #Update language
  data$language <- to

  return(data)
}

#' Check if you have the latest version of Fauna do Brasil data available
#'
#' @description
#' This function checks if you have the latest version of the Fauna do Brasil
#' data available in a specified directory.
#'
#' @param data_dir the directory where the data should be located.
#'
#' @return A message informing whether you have the latest version of Fauna do
#' Brasil available in the data_dir
#' @usage fauna_version(data_dir)
#' @export
#'
#' @importFrom httr GET
#' @importFrom XML htmlParse xpathSApply xmlGetAttr
#'
#' @examples
#' #Check if there is a version of Fauna do Brasil data available in the
#' #current directory
#' fauna_version(data_dir = getwd())

fauna_version <- function(data_dir) {
  #Set folder
  if (missing(data_dir)) {
    stop("Argument data_dir is not defined")
  }

  if (!is.character(data_dir)) {
    stop(paste0("Argument data_dir must be a character, not ", class(data_dir)))
  } else {
    path_data <- data_dir
  }

  #Search for directories with data
  all_dirs <- list.files(path = path_data, recursive = TRUE,
                         pattern = "CompleteBrazilianFauna.gz", full.names = FALSE)
  dir_versions <- dirname(all_dirs)


  #Get highest version
  if(length(dir_versions) > 0) {
    high_version <- max(as.numeric(dir_versions))
    } else {
      high_version <- 0
    }

  #Get link of latest version
  response <- httr::GET(
    "https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil")
  parse <- XML::htmlParse(response)
  links <- unlist(XML::xpathSApply(parse, path = "//a", XML::xmlGetAttr,
                                   "href"))
  download_pattern <- "https://ipt.jbrj.gov.br/jbrj/archive.do?r=catalogo_taxonomico_da_fauna_do_brasil&v="
  link_download <- subset(links, grepl(download_pattern, links,
                                       fixed = TRUE))

  #Get version
  latest_version <- as.numeric(gsub(".*catalogo_taxonomico_da_fauna_do_brasil&v=([0-9.]+).*", "\\1",
                         link_download))

  #Check if you have the latest version
  is_latest <- high_version == latest_version

  #Check how many versions exists
  many_versions <- length(dir_versions) > 1

  #Print messages

  if(length(dir_versions) == 0) {
    message(
      "You do not have any version of Fauna do Brasil in this directory.
    The latest version is ", latest_version, ". Please, change the directory or
    run the function get_faunabr() to download the latest version of fauna do
    Brasil.", "\n")
  }

  if(isTRUE(is_latest) & isFALSE(many_versions)) {
    message(paste("You have the latest version of Fauna do Brasil: Version", latest_version, "\n"))
  }

  if(isTRUE(is_latest) & isTRUE(many_versions)) {
    message(paste("You have the following versions of Fauna do Brasil:\n",
paste(dir_versions, collapse = "\n"),
"\n It includes the latest version: ",
latest_version, "\n"))
  }

  if(isFALSE(is_latest) & isTRUE(many_versions)) {
    message(paste0("You have the following versions of Fauna do Brasil:\n",
                   paste0(dir_versions, collapse = "\n"),
                   "\nHowever, it does not include the latest version: ",
                   latest_version,
                   "\nIf you want to download the latest version, run the function
    get_faunabr() again"))
  }
}

#' Check species names
#'
#' @description check_fauna_names checks if the species names are correct and searches
#'  for suggestions if the name is misspelled or not found in the Fauna do
#'  Brasil database
#'
#' @param data (data.frame) the data.frame imported with the
#' \code{\link{load_faunabr}} function.
#' @param species (character) names of the species to be checked.
#' @param max_distance (numeric) Maximum distance (as a fraction) allowed for
#' searching suggestions when the name is misspelled. It can be any value
#' between 0 and 1. The higher the value, the more suggestions are returned.
#' For more details, see \code{\link[base:agrep]{agrep}}. Default = 0.1.
#' @param include_subspecies whether include subspecies when checking names.
#' Default = TRUE.
#' @return a data.frame with the following columns:
#' - input_name: the species names informed in species argument
#' - Spelling: indicates if the species name is Correct (a perfect match with a
#' species name in the Flora e Funga do Brasil), Probably_incorrect
#' (partial match), or Not_found (no match with any species).
#' - Suggested name: If Spelling is Correct, it is the same as the input_name.
#' If Spelling is Probably_correct, one or more suggested names are listed,
#' found according to the maximum distance. If Spelling is "Not_found", the value
#' is NA.
#' - Distance: The integer Levenshtein edit distance. It represents the number
#' of single-character edits (insertions, deletions, or substitutions) required
#' to transform the input_name into the Suggested_name.
#' - taxonomicStatus: the taxonomic status of the species name ("valid" or
#' "synonym").
#' - nomenclaturalStatus: the nomenclatural status of the species name. This
#' information is not available for all species.
#' - validName: If the species name is not valid or incorrect, the
#' valid name of the specie. If the species name is valid and correct,
#' the same as input_name and Suggested_name.
#' - family: the family of the specie.
#' @usage check_fauna_names(data, species, max_distance = 0.1,
#'                          include_subspecies = TRUE)
#' @export
#' @importFrom utils adist
#' @importFrom stats ave
#' @references
#' Brazilian Zoology Group. Catálogo Taxonômico da Fauna do Brasil. Available at:
#' https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil
#' @examples
#' data("fauna_data")
#' spp <- c("Pantera onça", "Mazama bororo", "Mazama jucunda",
#'           "Araucaria angustifolia")
#' check_fauna_names(data = fauna_data, species = spp)

check_fauna_names <- function(data, species, max_distance = 0.1,
                              include_subspecies = TRUE){
  if (missing(data)) {
    stop("Argument data is not defined")
  }

  if (missing(species)) {
    stop("Argument species is not defined")
  }

  #Check classes
  if (!inherits(data, "data.frame")) {
    stop(paste0("Argument data must be a data.frame, not ", class(data)))
  }

  if (!is.character(species)) {
    stop(paste0("Argument species must be a character, not ", class(species)))
  }

  if (!is.numeric(max_distance)) {
    stop(paste0("Argument max_distance must be numeric, not ",
                class(max_distance)))
  }
  if(max_distance < 0 | max_distance > 1) {
    stop(paste0("Argument max_distance must be a numeric value between 0 and 1"
    ))
  }

  #Remove subspecies
  if(!include_subspecies){
    data <- data[data$taxonRank %in% c("species", "especie"),]
  }

  #Get all species available
  all_sp <- unique(data$species)


  #Get species with correct and incorrect spell
  species <- extract_binomial(species) #Get binomial names
  sp_in <- subset(species, species %in% all_sp)
  sp_in_df <- data.frame(input_name = sp_in, Suggested_name = sp_in)
  sp_out <- subset(species, !(species %in% all_sp))

  sp_l <- lapply(seq_along(sp_out), function(i){
    sp_agrep <- agrep(sp_out[i], all_sp, value = TRUE,
                      max.distance = max_distance)
    if(length(sp_agrep) == 0) {sp_agrep <- NA}
    sp_df <- data.frame(input_name = sp_out[i], Suggested_name = sp_agrep)
    return(sp_df)
  })
  spp <- do.call("rbind", sp_l)
  spp <- rbind(sp_in_df, spp)

  #Calculate distance
  spp$Distance <- vapply(seq_len(nrow(spp)), FUN.VALUE = double(1),
                         function(i){
                           Distance <- adist(spp[i,1], spp[i,2])
                         })

  #Create columns
  spp$Spelling <- ifelse(is.na(spp$Distance), "Not_found",
                         ifelse(spp$Distance > 0, "Probably_incorrect",
                                ifelse(spp$Distance == 0, "Correct", NA)))
  #Get information about Family, taxonomic and nomenclatural status
  d_info <- subset(data, species %in% spp$Suggested_name)
  d_info <- d_info[c("species", "taxonomicStatus", "nomenclaturalStatus",
                     "validName", "family")]
  #Merge info
  spp_info <- merge(spp, d_info, by.x ="Suggested_name", by.y = "species",
                    all = TRUE, sort = FALSE)
  #Organize columns
  spp_info <- unique(spp_info[, c("input_name", "Spelling", "Suggested_name",
                           "Distance", "taxonomicStatus",
                           "nomenclaturalStatus", "validName", "family")])
  spp_info$validName[which(spp_info$taxonomicStatus %in% c("valid", "valido") &
                                is.na(spp_info$validName))] <-
    spp_info$Suggested_name[which(spp_info$taxonomicStatus %in% c("valid", "valido") &
                                    is.na(spp_info$validName))]
  #Identify if there is single or multiple or none matches
  spp_info$matches <- ifelse(stats::ave(spp_info$input_name,
                                 spp_info$input_name, FUN = length) > 1,
                             "multiple", "single")
  spp_info$matches[is.na(spp_info$Suggested_name)] <- "none"
  return(spp_info)
}

#' Load Brazilian Fauna database
#'
#' @param data_dir (character) the same directory used to save the data
#' downloaded from Brazilian Fauna using the \link{get_faunabr} function.
#' @param data_version (character) the version of Brazilian Fauna database to
#' be loaded. It can be "latest", which will load the latest version
#' available; or another specified version, for example "1.2".
#' Default = "latest".
#' @param type (character) it determines the number of columns that will be
#' loaded. It can be "short" or "complete". Default = "short". See details.
#' @param verbose (logical) Whether to display messages during function
#' execution. Set to TRUE to enable display, or FALSE to run silently.
#' Default = TRUE.
#' @param encoding (character) the declared encodings for special characters.
#' Character strings in R can be declared to be encoded in "latin1" or "UTF-8".
#' Default: "UTF-8".
#' @details
#' The parameter type accepts two arguments. If type = short, it will load a
#' data.frame with the 20 columns needed to run the other functions of the
#' package: species, subspecies, scientificName, validName, kingdom, phylum,
#' class, order, family, genus, lifeForm, habitat, states, countryCode, origin,
#' taxonomicStatus, nomenclaturalStatus, vernacularName, and taxonRank.
#' If type = complete, it will load a data.frame with all 31 variables available
#' in Brazilian Fauna database.
#'
#' @return A data.frame with the specified version (Default is the latest
#' available) of the Brazilian Fauna database. This data.frame is necessary to
#' run most of the functions of the package.
#'
#' @usage load_faunabr(data_dir, data_version = "latest",
#'                     type = "short", verbose = TRUE, encoding = "UTF-8")
#' @importFrom stats na.omit
#' @importFrom data.table fread
#' @export
#' @references
#' Brazilian Zoology Group. Catálogo Taxonômico da Fauna do Brasil. Available at:
#' https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil
#'
#' @examples
#' \dontrun{
#' #Creating a folder in a temporary directory
#' #Replace 'file.path(tempdir(), "faunabr")' by a path folder to be create in
#' #your computer
#' my_dir <- file.path(file.path(tempdir(), "faunabr"))
#' dir.create(my_dir)
#' #Download, merge and save data
#' get_fauna(output_dir = my_dir, data_version = "latest", overwrite = TRUE,
#'             verbose = TRUE)
#' #Load data
#' df <- load_faunabr(data_dir = my_dir, data_version = "latest",
#'                    type = "short")
#' }
load_faunabr <- function(data_dir, data_version = "latest",
                         type = "short", verbose = TRUE, encoding = "UTF-8"){
  #Set folder
  if (missing(data_dir)) {
    stop("Argument data_dir is not defined")
  }

  #Check classes
  if (!is.character(data_dir)) {
    stop(paste0("Argument data_dir must be a character, not ", class(data_dir)))
  }

  if (!is.character(data_version)) {
    stop(paste0("Argument data_version must be a character, not ",
                class(data_version)))
  }

  if (!(type %in% c("short", "complete"))) {
    stop("Argument type must be 'short' or 'complete'")
  }

  #Check classes
  if (!is.character(encoding)) {
    stop(paste0("Argument encoding must be a character, not ", class(encoding)))
  }

#Set directory
  path_data <-  data_dir

  #Get latest available version if version was not set
  if(data_version == "latest") {
    #Search for directories
    all_dirs <- list.dirs(path = path_data, recursive = FALSE)
    dir_versions <- na.omit(suppressWarnings(
      as.numeric(sub(".*/([0-9.]+)$", "\\1", all_dirs))))

    #Get highest version
    if(length(dir_versions) > 0) {
      version_data <- max(dir_versions) } else {
        version_data <- 0
      }
  } else {version_data <-  data_version}

  #Check if version_data is avaliable
  check_folder <- version_data %in% list.dirs(path_data, full.names = F)

  if (!check_folder) {
    stop("The version specified - ", version_data, " - is not available in the
    specified directory. Please check the directory or run the 'get_faunabr' function
    to download the latest version of the data.")
  }

  #Load data
  if(verbose){
  message("Loading version ", version_data) }

  if(type == "complete") {
    return(data.table::fread(file.path(path_data, version_data,
                            "CompleteBrazilianFauna.gz"),
                            data.table = FALSE,
                            encoding = encoding,
                            na.strings = "")) }

  if(type == "short") {
    return(data.table::fread(file.path(path_data, version_data,
                                       "CompleteBrazilianFauna.gz"),
                             data.table = FALSE,
                             encoding = encoding,
                             select = c("species", "subspecies",
                                        "scientificName", "validName",
                                        "kingdom", "phylum", "class", "order",
                                        "family", "genus", "lifeForm",
                                        "habitat", "states", "countryCode",
                                        "origin", "taxonomicStatus",
                                        "nomenclaturalStatus",
                                        "vernacularName", "taxonRank", "id",
                                        "language"),
                             na.strings = ""))
  }
}

#' Retrieve synonyms for species
#'
#' @param data (data.frame) the data.frame imported with the
#' \code{\link{load_faunabr}} function
#' @param species (character) names of the species
#' @param include_subspecies (logical) include subspecies that are synonyms of
#' the species? Default = TRUE
#'
#' @return A data.frame containing unique synonyms of the specified species
#' along with relevant information on taxonomic status.
#' @usage fauna_synonym(data, species,
#'                    include_subspecies = TRUE)
#' @export
#' @examples
#' data("fauna_data") #Load Flora e Funga do Brasil data
#' #Species to extract synonyms
#' spp <- c("Panthera onca", "Mazama jucunda", "Subulo gouzoubira")
#' spp_synonyms <- fauna_synonym(data = fauna_data, species = spp,
#'                               include_subspecies = FALSE)
#' spp_synonyms
#'
fauna_synonym <- function(data, species,
                        include_subspecies = TRUE){

  if (missing(data)) {
    stop("Argument data is not defined")
  }

  #Check classes
  if (!is.data.frame(data)) {
    stop(paste0("Argument data must be a data.frame, not ", class(data)))
  }

  if (!is.character(species)) {
    stop(paste0("Argument species must be a character, not ", class(species)))
  }

  if (!is.logical(include_subspecies)) {
    stop(paste0("Argument include_subspecies must be logical, not ",
                class(include_subspecies)))
  }

  #Taxon Rank
  if(include_subspecies){
    ss <- c("subspecies", "sub_especie")
  } else {ss <- NULL}

  tr <- c("species", "especie", ss)

  data <- data[data$taxonRank %in% tr,]


  #Check if there is any species absent in d
  no_match <- setdiff(species, unique(data$species))
  if(length(no_match) > 0 & length(no_match) < length(species)) {
    warning(paste("Some species are absent of Fauna do Brasil database\n",
                  "Check the species names using the check_names() function"))
  }
  #Get match
  order <- setdiff(species, no_match)

  res <- unique(data[which(extract_binomial(data$validName) %in% species),
                     c("validName", "species", "taxonomicStatus")])

  #Get species withou synonyms
  no_syn <- setdiff(species, res$validName)
  if(length(no_syn) > 0){
    res_no_syn <- data[data$species %in% no_syn,
                       c("validName", "species", "taxonomicStatus")]
    res_no_syn$validName <- res_no_syn$species
    res_no_syn$species <- NA
    res <- rbind(res, res_no_syn)
  }


  if(nrow(res) > 0) {
    #Reorder
    res <- res[order(match(res$validName, order)), ]

    #Change name of the column
    colnames(res)[2] <- "synonym"

    #Remove valid names
    res <- subset(res, !(res$synonym %in% species))

    return(res) } else {
      warning(paste("All specified species are absent of Fauna do Brasil
                  database\n",
                    "Check the species names using the check_names() function"))
      return(NULL)
    }
}

---
title: "3. Using faunabr to flag erroneous records"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{3. Using faunabr to flag erroneous records}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message  = FALSE,
  eval = FALSE,
  fig.width = 8,
  fig.height = 6
)
```

## Loading data
Before you begin, use the `load_faunabr` function to load the data. For more detailed information on obtaining and loading the data, please refer to [1. Getting started with faunabr](getting_started.html)

```{r Get data, results='hide', message=FALSE, warning=FALSE, echo=FALSE}
library(faunabr) #Load package
my_dir <- file.path(file.path(tempdir(), "faunabr"))
dir.create(my_dir)
get_faunabr(output_dir = my_dir, #directory to save the data
            data_version = "latest", #get the most recent version available
            overwrite = T) #Overwrite data, if it exists
```

```{r, message=FALSE, warning=F}
library(terra)
#Load data
bf <- load_faunabr(data_dir = my_dir, #Folder where you stored the data with the function get_faunabr()
                   data_version = "latest",
                   type = "short") #short version
#> Loading version 1.3
```


## Get spatial polygons of species distribution
Fauna do Brasil provides information on the brazilian federal states and countries with confirmed occurrences of the species. The `get_spat_occ` function extracts these information and return Spatial polygons (SpatVectors) representing the distribution of the specie. We can choose getting the Spatialvector of the brazilian federal states and/or countries.

```{r}
#Example species
spp <- c("Panthera onca", "Mazama jucunda")
#Get spatial polygons
spp_spt <- fauna_spat_occ(data = bf, species = spp, state = TRUE,
                          country = TRUE, verbose = TRUE)
#> Getting states of Panthera onca
#> Getting countries of Panthera onca
#> Getting states of Mazama jucunda
#> Getting countries of Mazama jucunda
```

The SpatVectors are stored in a nested list by species:

```{r}
par(mfrow = c(3, 2), mar = c(2, 0, 2, 0)) 
plot(spp_spt$`Panthera onca`$states,
     main = paste0(names(spp_spt)[[1]], " - States"), mar = NA)
plot(spp_spt$`Panthera onca`$countries,
     main = paste0(names(spp_spt)[[1]], " - Countries"), mar = NA)
plot(spp_spt$`Mazama jucunda`$states,
     main = paste0(names(spp_spt)[[2]], " - States"), mar = NA)
plot(spp_spt$`Mazama jucunda`$countries,
     main = paste0(names(spp_spt)[[2]], " - Countries"), mar = NA)
```

```{r IMG01, eval=TRUE, echo=FALSE}
knitr::include_graphics("vignettes_img/IMG01.png")
```

## Filtering occurrence records using distribution information in Fauna do Brasil

Georeferencing errors in online species records can introduce significant bias into ecological and biogeographical research findings. Some R packages, as [CoordinateCleaner](https://besjournals.onlinelibrary.wiley.com/doi/10.1111/2041-210X.13152), help to flagging common spatial errors in biological collection data, for example, records of terrestrial organism that fall in the sea or were assigned to capital and province centroids.
Since the distributions of species in Fauna do Brasil are based on the expertise of taxonomists, it represents a valuable information to add an additional step on checking the validity of occurrence records got from online databases (as GBIF or SpeciesLink). 
The `filter_faunabr` function automatized this flagging. You can use the function to flag and/or remove records that fall outside the brazilian states or countries with confirmed occurrence according to Fauna do Brasil.

As example, let's use the occurrence records of two species. *Panthera onca* is a terrestiral felidae with confirmed occurrences in 22 of the 27 brazilian states (absent in Rio Grande do Norte, Paraíba, Pernambuco, Alagoas, and Sergipe) and 20 countries.
*Chaetomys subspinosus* is an arboreal rodent endemic to Brazil with confirmed occurrences in 5 states (Bahia, Espírito Santo, Minas Gerais, Rio de Janeiro, and Sergipe). Let's plot these information using the function `get_spat_occ`:

```{r}
my_spp <- c("Panthera onca", "Chaetomys subspinosus")
pol_spp <- fauna_spat_occ(data = bf, species = my_spp,
                       state = TRUE, country = TRUE, 
                       verbose = TRUE)
par(mfrow = c(2, 2), mar = c(2, 0, 2, 0))
plot(pol_spp$`Panthera onca`$states,
     main = paste0(names(pol_spp)[[1]], " - States"), mar = NA)
plot(pol_spp$`Panthera onca`$countries,
     main = paste0(names(pol_spp)[[1]], " - Countries"), mar = NA)
plot(pol_spp$`Chaetomys subspinosus`$states,
     main = paste0(names(pol_spp)[[2]], " - States"), mar = NA)
plot(pol_spp$`Chaetomys subspinosus`$countries,
     main = paste0(names(pol_spp)[[2]], " - Countries"), mar = NA)
```
```{r IMG02, eval=TRUE, echo = FALSE}
knitr::include_graphics("vignettes_img/IMG02.png")
```
We downloaded the occurrences of these two species from GBIF using the `plantR::rgbif2()` function. These occurrences are saved as data examples in the package and we can import it:

```{r}
data("occurrences")
head(occurrences)
#>         species         x          y  source
#> 1 Panthera onca -90.38409  17.377023    gbif
#> 2 Panthera onca -90.24368  17.240507    gbif
#> 3 Panthera onca -77.36680   0.287624    gbif
#> 4 Panthera onca -56.61023 -17.239688    gbif
#> 5 Panthera onca -61.04386  -2.387029    gbif
#> 6 Panthera onca -77.28850   0.288757    gbif
```

The input data with records must be a dataframe with at least 3 columns: one informing the species name, one informing the longitude and one informing the latitude.
Let's check if there are records outside of the species' natural ranges considering states and countries:

```{r}
occ_check <- filter_faunabr(data = bf, occ = occurrences,
                    by_state = TRUE, buffer_state = 20,
                    by_country = TRUE, buffer_country = 20,
                    value = "flag&clean", keep_columns = TRUE,
                    verbose = FALSE)
#> Returning list with flagged and cleaned occurrences
```

Since we set *value = "flag&clean"*, the function returned a list with two data.frames: one with all the records flagged if on each test they passed (TRUE) or not (FALSE); and another with only the records that passed in all tests.

Let's use the [mapview package](https://r-spatial.github.io/mapview/) to plot an interactive map of the flagged records:

```{r, warning=FALSE, results='hide'}
#Install mapview if necessary and load package
if(!require(mapview)){
    install.packages("mapview")
}
#Load mapview
library(mapview)
```

Let's check the records of *Panthera onca*, plotting the records and the map got previously. You can see that the green records passed on the test of States, falling in the states with confirmed occurrence of the specie. The red dots are the records that falls outside the states. The gray dots are records that was not testes because they falls outside Brazil.

```{r, fig.width= 9}
#Convert points to spatvector
panthera_occ <- subset(occ_check$flagged,
                       occ_check$flagged$species == "Panthera onca")
panthera_occ <- vect(panthera_occ, geom = c("x", "y"),
                     crs = crs(pol_spp$`Panthera onca`$states))

#Iteractive plot
mapview(pol_spp$`Panthera onca`$states,
        layer.name = "States with confirmed occurrences") +
  mapview(panthera_occ, zcol = "inside_state", layer.name = "Inside state?",
          col.regions = c("red", "green"))
```
```{r IMG03, eval=TRUE, echo = FALSE, fig.pos="H", out.width="90%"}
knitr::include_graphics("vignettes_img/IMG03.png")
```
We can see the same with countries: green dots represents records that passed in the test and green dots represents records that did not passed the test.

```{r, fig.width= 9}
#Iteractive plot
mapview(pol_spp$`Panthera onca`$countries,
        layer.name = "Countries with confirmed occurrences", burst = TRUE, legend = F) +
  mapview(panthera_occ, zcol = "inside_country",
          col.regions = c("red", "green"),
          layer.name = "Inside country?")
```
```{r IMG04, eval=TRUE, echo = FALSE, fig.pos="H", out.width="90%"}
knitr::include_graphics("vignettes_img/IMG04.png")
```

---
title: "2. Selecting and filtering species"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{2. Selecting and filtering species}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  warning = FALSE
)
```

### Selecting a list of species
Selecting a list of target species is a common task in macroecological and conservation studies. For example, a researcher may seek to model the distribution of arboreal mammals that occur in a particular state.

By applying different filters to the database, anyone can obtain a verified taxonomic list of Brazilian species of animals for any Brazilian state or Country. Additionally, the filter can be applied by phylum, class, order, family, genus, life form, habitat, origin, and taxonomic status.
In this vignette, users will learn how to use *faunabr* package to select a list of species based on these features.

## Loading data
Before you begin, use the `load_faunabr` function to load the data. For more detailed information on obtaining and loading the data, please refer to [Getting started with faunabr](https://wevertonbio.github.io/faunabr/articles/getting_started.html)

```{r}
library(faunabr)
#Folder where you stored the data with the function get_faunabr()
#Load data
bf <- load_faunabr(data_dir = my_dir,
                   data_version = "latest",
                   type = "short") #short version
#> Loading version 1.3
```

## Selecting a list of species
One of the primary objectives of this package is to assist in selecting a species list based on taxonomic classification, characteristics and distribution. Specifically, you can filter by:

* phylum, class, order, family and genus;
* lifeForm and habitat;
* brazilian states and countries with confirmed occurrences;
* origin;
* taxonomicStatus.

To explore all available options for each filter, use the `fauna_attributes()` function with the desired attributes. This function will provide a list of data.frames with the available options to use in the `select_fauna()` function.

```{r}
#Get available options to filter by lifeForm, habitat and origin
fauna_at <- fauna_attributes(data = bf,
                             attribute = c("lifeForm", "habitat", "origin"))
head(fauna_at$lifeForm)
#>          lifeForm
#> 1        colonial
#> 2       commensal
#> 3    ectoparasite
#> 4 ectoparasiteIDE
#> 5    endoparasite
#> 6  endoparasiteid

head(fauna_at$habitat)
#>         habitat
#> 1      arboreal
#> 2 cavernicolous
#> 3     fossorial
#> 4    freshwater
#> 5     hyporheic
#> 6        marine

head(fauna_at$origin)
#>         origin
#> 1  cryptogenic
#> 2 domesticated
#> 3   introduced
#> 4     invasive
#> 5       native
```

As an illustration, let's consider the scenario where we aim to retrieve a list of all native arboreal insects with confirmed occurrences in Rio de Janeiro:

```{r}
insects_rj <- select_fauna(data = bf, include_subspecies = FALSE,
                           phylum = "all",
                           class = "Insecta", #Select insecs
                           order = "all",
                           family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "arboreal", filter_habitat = "in",
                           states = "RJ", #Rio de Janeiro
                           filter_states = "in", #Select IN Rio de Janeiro
                           country = "all", filter_country = "in",
                           origin = "native",
                           taxonomicStatus = "valid")
nrow(insects_rj)
#> [1] 114
```

The filter returned 114 species that meet the specified criteria. It's important to note that these selections include species with confirmed occurrences in Rio de Janeiro, and some of them may also have confirmed occurrences in other states:

```{r}
#First 7 unique values of states in the filtered dataset
unique(insects_rj$states)[1:7]
#> [1] "AC;AM;AP;BA;CE;DF;ES;GO;MA;MG;MS;MT;PA;PE;RJ;RN;RO;RR;SE;SP;TO"
#> [2] "BA;MA;MG;MT;PR;RJ;RS;SC;SP"                                    
#> [3] "ES;MG;PE;PR;RJ;RS;SP"                                          
#> [4] "ES;PR;RJ;SP"                                                   
#> [5] "RJ"                                                            
#> [6] "RJ;SP"                                                         
#> [7] "ES;MG;RJ;RS;SC;SP" 
```

If you wish to select species with confirmed occurrences only Rio de Janeiro, and not in any other state, modify the *filter_state* parameter to "only":

```{r}
insects_rj_only <- select_fauna(data = bf, include_subspecies = FALSE,
                           phylum = "all",
                           class = "Insecta", #Select insecs
                           order = "all",
                           family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "arboreal", filter_habitat = "in",
                           states = "RJ", #Rio de Janeiro
                           filter_states = "only", #Select ONLY in Rio de Janeiro
                           country = "all", filter_country = "in",
                           origin = "native",
                           taxonomicStatus = "valid")
nrow(insects_rj_only)
#> [1] 22
unique(insects_rj_only$states)
#> [1] "RJ"
```

Now, the filter has resulted in 22 species, all exclusively confined to Rio de Janeiro.

Furthermore, the package offers the flexibility to combine various filters (please consult `?select_fauna` for comprehensive details). For instance, consider the scenario where we aim to compile a list of native insects with confirmed occurrences in the states of Paraná (PR), Santa Catarina (SC), and Rio Grande do Sul (RS):

```{r}
insects_south <- select_fauna(data = bf, include_subspecies = FALSE,
                           phylum = "all",
                           class = "Insecta", #Select insecs
                           order = "all",
                           family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "all", filter_habitat = "in",
                           states = c("PR", "SC", "RS"), #States from southern Brazil
                           filter_states = "in", #IN any of these states
                           country = "all", filter_country = "in",
                           origin = "native",
                           taxonomicStatus = "valid")
nrow(insects_south)
#> [1] 11461

#First 10 unique values of states in the filtered dataset
unique(insects_south$states)[1:10]
#> [1] "PR;SP"                                                                           
#> [2] "BA;MT;RJ;RS"                                                                     
#> [3] "RS"                                                                              
#> [4] "PR"                                                                              
#> [5] "PR;RS"                                                                           
#> [6] "GO;PA;RO;RR;SC"                                                                  
#> [7] "MG;PR;SP"                                                                        
#> [8] "GO;MG;PR;RS;SC"                                                                  
#> [9] "GO;RJ;SC"                                                                        
#> [10] "AC;AL;AM;AP;BA;CE;DF;ES;GO;MA;MG;MS;MT;PA;PB;PE;PI;PR;RJ;RN;RO;RR;RS;SC;SE;SP;TO"
```

By utilizing *filter_state = "in"*, our selection encompassed species occurring in all three states, as well as those appearing in only two or even one of them, in addition to other states. To impose a more rigorous criterion, selecting solely those species with confirmed occurrences in all three states, we can use *filter_state = "and"*:

```{r}
insects_south_and <- select_fauna(data = bf, include_subspecies = FALSE,
                           phylum = "all",
                           class = "Insecta", #Select insecs
                           order = "all",
                           family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "all", filter_habitat = "in",
                           states = c("PR", "SC", "RS"), #States from southern Brazil
                           filter_states = "and", #in PR AND SC AND RS
                           country = "all", filter_country = "in",
                           origin = "native",
                           taxonomicStatus = "valid")
nrow(insects_south_and)
#> [1] 1924

#First 10 unique values of states in the filtered dataset
unique(insects_south_and$states)[1:10]
#> [1] "GO;MG;PR;RS;SC"                                                                  
#> [2] "AC;AL;AM;AP;BA;CE;DF;ES;GO;MA;MG;MS;MT;PA;PB;PE;PI;PR;RJ;RN;RO;RR;RS;SC;SE;SP;TO"
#> [3] "AC;AM;MG;MS;MT;PA;PE;PR;RO;RS;SC;SP"                                             
#> [4] "AM;BA;CE;DF;ES;GO;MG;MS;MT;PA;PE;PR;RJ;RS;SC;SP"                                 
#> [5] "AC;AM;AP;BA;CE;DF;ES;GO;MG;MS;MT;PA;PE;PR;RJ;RO;RR;RS;SC;SE;SP;TO"               
#> [6] "BA;ES;MG;PR;RJ;RS;SC;SP"                                                         
#> [7] "AM;DF;ES;GO;PR;RJ;RR;RS;SC;SP"                                                   
#> [8] "ES;GO;MG;PR;RJ;RS;SC;SP"                                                         
#> [9] "AC;AM;AP;BA;CE;ES;GO;MA;MG;MT;PA;PR;RJ;RO;RS;SC;SP;TO"                           
#> [10] "GO;MG;MS;PR;RO;RS;SC;SP"
```

Now, our selection consists solely of species with confirmed occurrences in all of the specified states. However, by utilizing the "and" argument, we permit the filter to include species with occurrences in additional states. To confine the filter exclusively to species with confirmed occurrences in all three states, without any occurrences elsewhere, we can use *filter_state = "only"*:

```{r}
insects_south_only <- select_fauna(data = bf, include_subspecies = FALSE,
                           phylum = "all",
                           class = "Insecta", #Select insecs
                           order = "all",
                           family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "all", filter_habitat = "in",
                           states = c("PR", "SC", "RS"), #States from southern Brazil
                           filter_states = "only", #ONLY in PR, SC and RS
                           country = "all", filter_country = "in",
                           origin = "native",
                           taxonomicStatus = "valid")
nrow(insects_south_only)
#> [1] 134

#The unique state in the filtered dataset
unique(insects_south_only$states)
#> [1] "PR;RS;SC"
```
Now, the filter return only 134 species, all of them occurring in PR, SC and RS; with no recorded occurrences in any other states.

## Checking names and subsetting species
### Extracting binomial name
In addition to selecting a species list based on their characteristics, the package also includes a function for subsetting species by name. Please note that this function exclusively operates with binomial names (Genus + specificEpithet), such as *Panthera onca*, and does not support complete scientific names (including Author or infraspecificEpithet), such as *Panthera onca (Linnaeus, 1758)*.

If you have species with complete scientific names, you can extract the binomial names using the function `get_binomial()`. Don't worry about excessive spaces between words; the function will remove any extra spaces from the names.

```{r}
complete_names <- c("Pantera onça (Linnaeus, 1758)",
                    "Zonotrichia capensis subtorquata Swainson, 1837",
                    "Mazama bororo Duarte, 1996",
                    "Mazama jucunda Thomas, 1913",
                    "Arrenurus tumulosus intercursor",
                    "Araucaria angustifolia")
#Panthera onca with typos to illustrate how the next function corrects it
#Araucaria angustifolia (a Plant) was used just as an example that will be used to illustrate the 
#next function
binomial_names <- extract_binomial(species_names = complete_names)
binomial_names
#> [1] "Pantera onça"                         "Zonotrichia capensis"                
#> [3] "Mazama bororo"                        "Mazama jucunda"                      
#> [5] "Arrenurus tumulosus"                  "Araucaria angustifolia" 
```
### Checking the species names
Additionally, you can verify the spelling, nomenclatural status, and taxonomic status of species names using the check_fauna_names() function. If the function is unable to locate the name of a species in the database (due to a typo, for example), it can suggest potential names based on similarities to other entries in the database.
To see how the function works, let's utilize the previously created binomial_names dataset:

```{r}
#Create example
checked_names <- check_fauna_names(data = bf, 
                             species = binomial_names,
                             include_subspecies = FALSE,
                             max_distance = 0.1)
tibble::tibble(checked_names) #print data.frame as tibble
#> # A tibble: 6 × 8
#>   input_name           Spelling Suggested_name Distance taxonomicStatus validName #> family
#>   <chr>                <chr>    <chr>             <dbl> <chr>           <chr>        #> <chr> 
#> 1 Zonotrichia capensis Correct  Zonotrichia c…        0 valid   Zonotrichia… #> Passe…
#> 2 Mazama bororo        Correct  Mazama bororo         0 synonym         Mazama jucu… #> Cervi…
#> 3 Mazama jucunda       Correct  Mazama jucunda        0 valid   Mazama jucu… #> Cervi…
#> 4 Arrenurus (Incertae… Correct  Arrenurus tum…        0 valid   Arrenurus t… #> Arren…
#> 5 Pantera onça         Probabl… Panthera onca         2 valid   Panthera on… #> Felid…
#> 6 Araucaria angustifo… Not_fou… NA                   NA NA              NA           NA 

```
We can see that *Zonotrichia capensis*, *Mazama jucunda*, and *Arrenurus tumulosus* are spelling correctly and are valid names. *Mazama bororo*  is spelling correctly, but it is a synonym of *Mazama jucunda*.
In the case of *Pantera onça*, the spelling appears to be potentially incorrect (as the name wasn't found in the database); however, a similar name, *Panthera onca*, is suggested by the function. The 'Distance' column indicates the Levenshtein edit distance between the input and the suggested name.
The spelling of *Araucaria angustifolia* was flagged as Not_found (as expected, given that *Araucaria angustifolia* is not an animal!). Consequently, the name was not located in the database, and there were no comparable names available.

### Subsetting species

To retrieve species information from the Fauna do Brasil database, employ the `subset_species()` function. For optimal performance, we highly recommend utilizing the `get_binomial()` and `check_names()` functions beforehand. This ensures that you're exclusively working with species present in the Fauna do Brasil database.
To see how the function works, let's use the valid names in *checked_names* created previously:

```{r}
#Get only valid names
valids <- unique(checked_names$validName)
valids <- na.omit(valids) #Remove NA

#Subset species
my_sp <- subset_fauna(data = bf, species = valids,
                      include_subspecies = FALSE)
tibble::tibble(my_sp) #print data.frame as tibble
#> # A tibble: 4 × 19
#>   species       subspecies scientificName validName kingdom phylum class order family genus lifeForm habitat states countryCode #> origin
#>   <chr>         <chr>      <chr>          <chr>        <chr>   <chr>  <chr> <chr> <chr>  <chr> <chr>    <chr>   <chr>  <chr>       #> <chr> 
#> 1 Panthera onca ""         Panthera onca… ""           Animal… Chord… Mamm… Carn… Felid… Pant… "free_l… "terre… AC;AM… argentina;… #> "nati…
#> 2 Arrenurus (I… ""         Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      ""    
#> 3 Mazama jucun… ""         Mazama jucund… ""           Animal… Chord… Mamm… Arti… Cervi… Maza… "herbiv… "terre… BA;ES… brazil      #> "nati…
#> 4 Zonotrichia … ""         Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AL;AP… brazil      #> "nati…
#> # ℹ 4 more variables: taxonomicStatus <chr>, nomenclaturalStatus <chr>, vernacularName <chr>, taxonRank <chr>
```
We can also include subspecies and/or varieties:

```{r}
my_sp2 <- subset_fauna(data = bf, species = valids,
                      include_subspecies = TRUE)
tibble::tibble(my_sp2) #print data.frame as tibble
#> # A tibble: 14 × 19
#>    species      subspecies scientificName validName kingdom phylum class order family genus lifeForm habitat states countryCode #> origin
#>    <chr>        <chr>      <chr>          <chr>        <chr>   <chr>  <chr> <chr> <chr>  <chr> <chr>    <chr>   <chr>  <chr>       #> <chr> 
#>  1 Panthera on… ""         Panthera onca… ""           Animal… Chord… Mamm… Carn… Felid… Pant… "free_l… "terre… AC;AM… argentina;… #> "nati…
#>  2 Arrenurus (… ""         Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      "" #>    
#>  3 Arrenurus (… "Arrenuru… Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      "" #>    
#>  4 Arrenurus (… "Arrenuru… Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      "" #>    
#>  5 Mazama jucu… ""         Mazama jucund… ""           Animal… Chord… Mamm… Arti… Cervi… Maza… "herbiv… "terre… BA;ES… brazil      #> "nati…
#>  6 Zonotrichia… ""         Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AL;AP… brazil      #> "nati…
#>  7 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AP     brazil      #> "nati…
#>  8 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      RR     brazil      #> "nati…
#>  9 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      RR     brazil      #> "nati…
#> 10 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      BA;MA… brazil      #> "nati…
#> 11 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      PA     brazil      #> "nati…
#> 12 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AP;PA… brazil      #> "nati…
#> 13 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      ES;MG… brazil      #> "nati…
#> 14 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      TO     brazil      #> "nati…
#> # ℹ 4 more variables: taxonomicStatus <chr>, nomenclaturalStatus <chr>, vernacularName <chr>, taxonRank <chr>
#> 
```

### Retrieving synonyms
We can retrieve all synonyms of a species list. This can be particularly useful, for example, when searching for records of a species and all its synonyms (as listed in the Fauna do Brasil) in online databases like GBIF. To accomplish this, utilize the function fauna_synonym. To understand how the function works, let's search for the synonyms of three species:
```{r}
spp <- c("Panthera onca", "Mazama jucunda", "Subulo gouzoubira")
spp_syn <- fauna_synonym(data = bf, species = spp)
spp_syn
#>           validName               synonym        taxonomicStatus
#> 49343     Panthera onca                  <NA>             valid
#> 60523    Mazama jucunda         Mazama bororo             synonym
#> 61168 Subulo gouzoubira    Mazama gouazoubira             synonym
```
We can see that *Mazama jucunda* and *Subulo gouzoubira* have synonyms, *Mazama bororo* and *Mazama gouazoubira*, respectively. *Panthera onca* does not have any synonyms.

---
title: "4. Obtaining a Presence-Absence Matrix"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{4. Obtaining a Presence-Absence Matrix}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
  
```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message  = FALSE,
  eval = FALSE,
  fig.width = 8,
  fig.height = 6
)
```

One way of organizing biodiversity data is by using presence-absence matrices (PAMs), where a one represents the presence of species j in cell i, and a zero indicates absence. From a PAM, we can estimate a variety of metrics related to biodiversity patterns, including richness, range size, and composition. For a comprehensive list of biodiversity metrics, refer to the [PAM_indices function in the biosurvey package](https://claununez.github.io/biosurvey/reference/PAM_indices.html).

## Loading data
Before you begin, use the `load_faunabr` function to load the data. For more detailed information on obtaining and loading the data, please refer to [1. Getting started with faunabr](getting_started.html)

```{r Get data, results='hide', message=FALSE, warning=FALSE, echo=FALSE}
my_dir <- file.path(file.path(tempdir(), "faunabr"))
dir.create(my_dir)
get_faunabr(output_dir = my_dir, #directory to save the data
            data_version = "latest", #get the most recent version available
            overwrite = T) #Overwrite data, if it exists
```

```{r, message=FALSE, warning=F}
library(faunabr)
library(terra)
#Folder where you stored the data with the function get_faunabr()
#Load data
bf <- load_faunabr(data_dir = my_dir,
                   data_version = "latest",
                   type = "short") #short version
#> Loading version 1.3
```

## Getting a presence-absence matrix

The `fauna_pam()` function facilitates the utilization of species distribution information in Fauna do Brazil to generate a PAM. Each site represents a brazilian state or a country. In addition to the PAM, the function also provides a summary and a SpatVector containing the number of species in each site.

As an example, lets obtain a PAM consisting of all mammal species natives to Brazil:

```{r}
#Select native species of mammals with confirmed occurrence in Brazil
br_mammals <- select_fauna(data = bf,
                           include_subspecies = FALSE, phylum = "all",
                           class = "Mammalia",
                           order = "all", family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "all", filter_habitat = "in",
                           states = "all", filter_states = "in",
                           country = "BR", filter_country = "in",
                           origin = "all", taxonomicStatus = "valid")

#Get presence-absence matrix in states and countries
pam_mammals <- fauna_pam(data = br_mammals, by_state = TRUE,
                         by_country = FALSE,
                         remove_empty_sites = TRUE,
                         return_richness_summary = TRUE,
                         return_spatial_richness = TRUE,
                         return_plot = TRUE)

#Visualize (as tibble) the PAM for the first 5 species and 7 sites
tibble::tibble(pam_mammals$PAM[1:7, 1:5])
#> # A tibble: 7 × 5
#>   states `Platyrrhinus aurarius` `Kannabateomys amblyonyx` `Callicebus lucifer` `Cerradomys maracajuensis`
#>   <fct>                    <dbl>                     <dbl>                <dbl>                      <dbl>
#> 1 AM                           1                         0                    1                          0
#> 2 ES                           0                         1                    0                          0
#> 3 MG                           0                         1                    0                          1
#> 4 PR                           0                         1                    0                          0
#> 5 RJ                           0                         1                    0                          0
#> 6 RS                           0                         1                    0                          0
#> 7 SC                           0                         1                    0                          0
```
Since return_richness_summary is set to TRUE, the function also returns a data frame containing the number of species per site.

```{r}
#Visualize (as tibble) the richness summary table
tibble::tibble(pam_mammals$Richness_summary[1:7,])
#> # A tibble: 7 × 3
#> states   richness
#>   <fct>     <dbl>
#> 1 AM          225
#> 2 ES          120
#> 3 MG          188
#> 4 PR          116
#> 5 RJ          133
#> 6 RS          105
#> 7 SC          101
```

If return_spatial_richness is set to TRUE, the function will return a SpatVector containing the number of species per site. Additionally, when return_plot is also set to TRUE, the function returns a plot.

```{r IMG09, eval=TRUE, echo = FALSE}
knitr::include_graphics("vignettes_img/IMG05.png")
```

---
title: "4. Obtendo uma matrix de Presença-Ausência (PAM)"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{4. Obtendo uma matrix de Presença-Ausência (PAM)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message  = FALSE,
  eval = FALSE,
  fig.width = 8,
  fig.height = 6
)
```

Uma forma de organizar dados de biodiversidade é usando matrizes de presença-ausência (PAMs), onde 1 representa a presença da espécie j na célula i, e 0 indica ausência. A partir de uma PAM, podemos estimar uma variedade de métricas relacionadas a padrões de biodiversidade, incluindo riqueza, tamanho da área de distribuição e composição. Para uma lista abrangente de métricas de biodiversidade, consulte a função [PAM_indices do pacote biosurvey](https://claununez.github.io/biosurvey/reference/PAM_indices.html).

## Carregando os dados
Antes de começar, utilize a função `load_faunabr()` para carregar os dados. Para informações mais detalhadas sobre como obter e carregar os dados, consulte [Primeiros passos com faunabr](https://wevertonbio.github.io/faunabr/articles/primeiros_passos.html)

```{r}
library(faunabr)
#Carregar dados
bf <- load_faunabr(data_dir = my_dir, #Pasta onde foi salvo o arquivo com get_faunabr()
                   data_version = "latest", #Versão mais recente
                   type = "short") #Versão resumida
#> Loading version 1.3
```

Aqui, vamos utilizar os dados baixados em português (`translate = FALSE` em `get_faunabr()`.

## Obtendo uma matriz de presença-ausência

A função `fauna_pam()` facilita a utilização das informações de distribuição de espécies no Fauna do Brasil para gerar uma PAM. Cada local representa um estado brasileiro ou um país. Além da PAM, a função também fornece um resumo e um SpatVector contendo o número de espécies em cada local.

Como exemplo, vamos obter uma PAM com todas as espécies de mamíferos nativos do Brasil:

```{r}
#Selecionar espécies de mamíferos nativos do Brasil:
br_mammals <- select_fauna(data = bf,
                           include_subspecies = FALSE, phylum = "all",
                           class = "Mammalia",
                           order = "all", family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "all", filter_habitat = "in",
                           states = "all", filter_states = "in",
                           country = "BR", filter_country = "in",
                           origin = "all", taxonomicStatus = "valido")
#PAM em Estados do Brasil
pam_mammals <- fauna_pam(data = br_mammals, by_state = TRUE, 
                         by_country = FALSE,
                         remove_empty_sites = TRUE,
                         return_richness_summary = TRUE,
                         return_spatial_richness = TRUE,
                         return_plot = TRUE)
#Visualizar (em tibble) PAM com as primeiras 5 espécies nos primeiros 5 sites
tibble::tibble(pam_mammals$PAM[1:7, 1:5])
#> # A tibble: 7 × 5
#>   states `Platyrrhinus aurarius` `Kannabateomys amblyonyx` `Callicebus lucifer` `Cerradomys maracajuensis`
#>   <fct>                    <dbl>                     <dbl>                <dbl>                      <dbl>
#> 1 AM                           1                         0                    1                          0
#> 2 ES                           0                         1                    0                          0
#> 3 MG                           0                         1                    0                          1
#> 4 PR                           0                         1                    0                          0
#> 5 RJ                           0                         1                    0                          0
#> 6 RS                           0                         1                    0                          0
#> 7 SC                           0                         1                    0                          0
```
Como *return_richness_summary* está definido como *TRUE*, a função também retorna um dataframe contendo o número de espécies por local.
```{r}
#Visualise (em tibble) a tabela com a riqueza por site
tibble::tibble(pam_mammals$Richness_summary[1:7,])
#> # A tibble: 7 × 3
#> states   richness
#>   <fct>     <dbl>
#> 1 AM          225
#> 2 ES          120
#> 3 MG          188
#> 4 PR          116
#> 5 RJ          133
#> 6 RS          105
#> 7 SC          101
```

Se *return_spatial_richness* estiver definido como *TRUE*, a função retornará um SpatVector contendo o número de espécies por local. Além disso, quando *return_plot* também estiver definido como *TRUE* a função retornará um plot do mapa:
```{r IMG09, eval=TRUE, echo = FALSE}
knitr::include_graphics("vignettes_img/IMG05.png")
```

---
title: "1. Primeiros passos com faunabr"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{1. Primeiros passos com faunabr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = F,
  warning = FALSE
)
```

## Introdução
### O que é a Fauna do Brasil?
O [Catálogo Taxonômico da Fauna do Brasil](https://fauna.jbrj.gov.br/fauna) é resultado do esforço colaborativo de mais de 500 zoólogos especializados em diversos grupos de animais nativos do Brasil. Este banco de dados oferece descrições morfológicas detalhadas e padronizadas, informações sobre nomenclatura, distribuição geográfica e chaves de identificação para animais tanto nativos quanto não nativos encontrados no Brasil.

O pacote *faunabr* oferece um conjunto de funções desenvolvidas para acessar, filtrar e espacializar da Fauna do Brasil.

## Descrição geral das funções do pacote:

+ `get_faunabr()`: baixa a versão mais recente ou uma versão anterior do banco de dados da Fauna do Brasil.
+ `fauna_version()`: verifica se você possui a versão mais recente dos dados da Fauna do Brasil disponível em um diretório.
+ `load_faunabr()`: carrega o banco de dados da Fauna do Brasil para o R.
+ `translate_faunabr()`: traduz os dados de "forma de vida (lifeForm)", "origem (origin)", "habitat", "ranking taxonômico (taxonRank)" e "Status Taxonômico (taxonomicStatus)" entre Português e Inglês.
+ `fauna_attributes()`: exibe todas as opções disponíveis para filtrar espécies de acordo com suas características.
+ `select_fauna()`: filtra espécies com base em suas características e distribuição.
+ `extract_binomial()`: extrai o nome binomial (Gênero + epíteto específico) de um nome científico completo (Gênero + epíteto específico + Autor + Ano)
+ `fauna_synonym()`: obtém os sinônimos para as espécies.
+ `check_fauna_names()`: verifica se os nomes das espécies estão corretos.
+ `subset_fauna()`: obtém um subconjunto de espécies da Fauna do Brasil.
+ `fauna_spat_occ()`: obtém polígonos espaciais (SpatVectors) de espécies com base em sua distribuição.
+ `filter_faunabr()`: remove ou sinaliza registros fora das áreas naturais de ocorrência das espécies.
+ `fauna_pam()`: obtém uma matriz de presença-ausência de espécies com base em sua distribuição.

## Instalação

### Instalar a versão em desenvolvimento do GitHub

Você pode atual a versão do *faunabr* no [github](https://github.com/wevertonbio/faunabr) com:
```{r, results='hide', message=FALSE}
if(!require(remotes)){
    install.packages("remotes")
}

if(!require(faunabr)){
remotes::install_github('wevertonbio/faunabr')}

library(faunabr)
```

## Obtendo dados da Fauna do Brasil
Todos os dados incluídos no Fauna do Brasil (nomenclatura, forma de vida, distribuição, etc.) são armazenados em conjuntos de dados no formato Darwin Core Archive, que são atualizados frequentemente. Antes de baixar os dados disponíveis no Fauna do Brasil, precisamos criar uma pasta para salvar os dados:

```{r}
#Criando uma pasta em um diretório temporário
#Substitua 'file.path(tempdir(), "faunabr")' por um caminho de pasta a ser
#criada em seu computador
my_dir <- file.path(file.path(tempdir(), "faunabr"))
dir.create(my_dir)
```

Agora você pode utilizar a função `get_faunabr` para obter a versão mais recente dos dados:

```{r, results='hide', message=FALSE, warning=FALSE}
get_faunabr(output_dir = my_dir, #Pasta para salvar os dados
            data_version = "latest", #Obter a versão mais recente
            translate = FALSE, #Manter versão original em português
            overwrite = T) #Sobrescrever dados, se eles já existirem
```

A função levará alguns segundos para baixar os dados e alguns minutos para mesclar os conjuntos de dados em um único `data.frame.` Após a conclusão bem-sucedida, você encontrará uma pasta nomeada com a versão do Fauna do Brasil. Esta pasta contém o conjunto de dados bruto baixado (arquivos TXT em português) e um arquivo chamado **CompleteBrazilianFauna.gz**. Este último representa o conjunto de dados final mesclado. Por padrão, os dados são traduzidos para o inglês, mas aqui mantemos a versão original em português definindo `translate = TRUE`.

Você também tem a opção de baixar uma versão mais antiga específica do conjunto de dados da Fauna do Brasil. Para explorar as versões disponíveis, consulte [este link](https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil). Para baixar uma versão específica, basta substituir `latest` pelo número da versão desejada. Por exemplo:

```{r, results='hide', message=FALSE, warning=FALSE}
get_faunabr(output_dir = my_dir, #Pasta para salvar os dados
            data_version = "1.10", #Versão 1.10, publicada em 01-02-2024
            translate = FALSE, #Manter versão original em português
            overwrite = T) #Sobrescrever dados, se eles já existirem
```

Como mencionado anteriormente, você encontrará uma pasta chamada '1.10' dentro do diretório designado.

Para visualizar as versões disponíveis no seu diretório especificado, execute:

```{r}
fauna_version(data_dir = my_dir)
#> You have the following versions of Fauna do Brasil:
#>  1.17
#>  1.10
#>  It includes the latest version:  1.17
```

## Carregando os dados
Para utilizar as outras funções do pacote, você precisa carregar os dados em seu ambiente R. Para isso, utilize a função `load_faunabr()`. Por padrão, a função buscará automaticamente pela versão mais recente disponível no seu diretório. No entanto, você tem a opção de especificar uma versão particular usando o parâmetro *data_version*.
Além disso, você pode escolher entre duas versões dos dados: a versão resumida 'short' (contendo as 21 colunas necessárias para executar as outras funções do pacote) ou a versão completa 'completa' (com todas as 32 colunas originais). A função importa a versão 'short' por padrão.

```{r}
#Short version
bf <- load_faunabr(data_dir = my_dir,
                   data_version = "latest",
                   type = "short") #resumida
#> Loading version 393.401
colnames(bf) #Ver colunas da versão resumida
#>  [1] "species"             "subspecies"          "scientificName"       
#>  [4] "validName"           "kingdom"             "phylum"
#>  [7] "class"               "order"               "family" 
#> [10] "genus"               "lifeForm"            "habitat"
#> [13] "states"              "countryCode"         "origin" 
#> [16] "taxonomicStatus"     "nomenclaturalStatus" "vernacularName"       
#> [19] "taxonRank"           "id"                  "language"
```
Note que a versão completa possui 11 colunas adicionais:

```{r}
#Complete version
bf_complete <- load_faunabr(data_dir = my_dir,
                   data_version = "latest",
                   type = "complete") #completa

colnames(bf_complete) #Ver colunas da versão completa
#>  [1] "id"                       "taxonID"                 
#>  [3] "species"                  "subspecies"       
#>  [5] "scientificName"           "validName"                   
#>  [7] "validNameUsage"           "parentNameUsage"                      
#>  [9] "namePublishedInYear"      "higherClassification"      
#> [11] "kingdom"                  "phylum"                                 
#> [13] "class"                    "order"                                  
#> [15] "family"                   "genus"               
#> [17] "specificEpithet"          "infraspecificEpithet"
#> [19] "taxonRank"                "scientificNameAuthorship"              
#> [21] "taxonomicStatus"          "nomenclaturalStatus" 
#> [23] "vernacularName"           "lifeForm"      
#> [25] "habitat"                  "origin"
#> [27] "states"                   "countryCode"    
#> [29] "modified"                 "bibliographicCitation"               
#> [31] "relationshipOfResource"   "language"                        
```

## Traduzir atributos

Por padrão, os dados baixados do [Catálogo Taxonômico da Fauna do Brasil](https://fauna.jbrj.gov.br/fauna) são traduzidos para o inglês, a menos que você defina `translate = FALSE` na função `get_faunabr()`. No entanto, é possível usar a função `translate_faunabr()` para traduzir os dados carregados entre inglês e português:

```{r}
# Traduzir do português para o inglês
bf_ingles <- translate_faunabr(data = bf, to = "en")
# Ver formas de vida (em ingles)
fauna_attributes(bf_ingles, "lifeForm")
#>  $lifeForm
#>                  lifeForm
#> 1                colonial
#> 2               commensal
#> 3            ectoparasite
#> 4          ectoparasitoid
#> 5            endoparasite
#> 6          endoparasitoid
#> 7                epibiont
#> 8                eusocial
#> 9  free_living_individual
#> 10              herbivore
#> 11        hyperparasitoid
#> 12              inquiline
#> 13            mutualistic
#> 14              polynizer
#> 15               predator
#> 16                sessile

# Traduzir do inglês para o português
bf_portugues <- translate_faunabr(data = bf_ingles, to = "pt_br")
#Ver formas de vida disponíveis em portugues novamente
fauna_attributes(bf_portugues, "lifeForm")
#>  $lifeForm
#>                 lifeForm
#> 1               colonial
#> 2               comensal
#> 3           ectoparasito
#> 4        ectoparasitoide
#> 5           endoparasito
#> 6        endoparasitoide
#> 7              epibionte
#> 8              eussocial
#> 9              herbivoro
#> 10      hiperparasitoide
#> 11             inquilino
#> 12                mutual
#> 13           polinizador
#> 14              predador
#> 15                sessil
#> 16 vida_livre_individual
```

## Salvar dados
Se você deseja salvar os conjuntos de dados para abrir com um editor externo (por exemplo, Excel), você pode salvar o data.frame como um arquivo CSV:
```{r, eval=FALSE, warning=FALSE, message=FALSE, }
write.csv(x = bf,
          file = file.path(my_dir, "BrazilianFauna.csv"),
          row.names = F)
```

---
title: "3. Filtrar pontos de ocorrência com faunabr"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{3. Filtrar pontos de ocorrência com faunabr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message  = FALSE,
  eval = FALSE,
  fig.width = 8,
  fig.height = 6
)
```

## Carregando os dados
Antes de começar, utilize a função load_faunabr para carregar os dados. Para informações mais detalhadas sobre como obter e carregar os dados, consulte [Primeiros passos com faunabr](https://wevertonbio.github.io/faunabr/articles/primeiros_passos.html)

```{r}
library(faunabr)
library(terra)
#Carregar dados
bf <- load_faunabr(data_dir = my_dir, #Pasta onde foi salvo o arquivo com get_faunabr()
                   data_version = "latest", #Versão mais recente
                   type = "short") #Versão resumida
#> Loading version 1.3
```
Aqui, vamos utilizar os dados baixados em português (`translate = FALSE` em `get_faunabr()`).

## Obter polígonos espaciais da distribuição de espécies
Fauna do Brasil fornece informações sobre os estados brasileiros e países com ocorrências confirmadas das espécies. A função `get_spat_occ()` extrai essas informações e retorna polígonos espaciais (SpatVectors) que representam a distribuição da espécie. Podemos escolher obter o SpatVector dos estados e/ou países.

```{r}
#Espécies de exemplo
spp <- c("Panthera onca", "Mazama jucunda")
#Obter polígonos espaciais
spp_spt <- fauna_spat_occ(data = bf, species = spp, state = TRUE,
                          country = TRUE, verbose = TRUE)
#> Getting states of Panthera onca
#> Getting countries of Panthera onca
#> Getting states of Mazama jucunda
#> Getting countries of Mazama jucunda
```

Os SpatVectors são armazenadas em uma lista aninhada por espécies:

```{r}
par(mfrow = c(2, 2), mar = c(2, 0, 2, 0)) 
plot(spp_spt$`Panthera onca`$states,
     main = paste0(names(spp_spt)[[1]], " - States"), mar = NA)
plot(spp_spt$`Panthera onca`$countries,
     main = paste0(names(spp_spt)[[1]], " - Countries"), mar = NA)
plot(spp_spt$`Mazama jucunda`$states,
     main = paste0(names(spp_spt)[[2]], " - States"), mar = NA)
plot(spp_spt$`Mazama jucunda`$countries,
     main = paste0(names(spp_spt)[[2]], " - Countries"), mar = NA)
```

```{r IMG01, eval=TRUE, echo=FALSE}
knitr::include_graphics("vignettes_img/IMG01.png")
```

## Filtrando registros de ocorrência usando informações de distribuição no Fauna do Brasil

Erros de georreferenciamento em registros de espécies online podem introduzir vieses significativos nos resultados de pesquisas ecológicas e biogeográficas. Alguns pacotes R, como o [CoordinateCleaner](https://besjournals.onlinelibrary.wiley.com/doi/10.1111/2041-210X.13152), ajudam a identificar erros comuns em dados de coleções biológicas, como registros de organismos terrestres que caem no mar ou que foram atribuídos a centroides de capitais e estados.
Dado que as distribuições de espécies no Fauna do Brasil são baseadas na expertise de taxonomistas, elas representam informações valiosas para adicionar um passo adicional na verificação da validade de registros de ocorrência obtidos de bancos de dados online (como GBIF ou SpeciesLink).
A função `filter_faunabr()` automatiza essa identificação. Você pode usar a função para sinalizar e/ou remover registros que estejam fora dos estados brasileiros ou países com ocorrências confirmadas de acordo com a Fauna do Brasil.

Como exemplo, vamos usar os registros de ocorrência de duas espécies. *Panthera onca* é um felino terrestre com ocorrências confirmadas em 22 dos 27 estados brasileiros (ausente no Rio Grande do Norte, Paraíba, Pernambuco, Alagoas e Sergipe) e em 20 países.
*Chaetomys subspinosus* é um roedor arbóreo endêmico do Brasil com ocorrências confirmadas em 5 estados (Bahia, Espírito Santo, Minas Gerais, Rio de Janeiro e Sergipe). Vamos plotar mapas com essas informações usando a função `get_spat_occ()`:

```{r}
my_spp <- c("Panthera onca", "Chaetomys subspinosus")
pol_spp <- fauna_spat_occ(data = bf, species = my_spp,
                       state = TRUE, country = TRUE, 
                       verbose = TRUE)
par(mfrow = c(2, 2), mar = c(2, 0, 2, 0))
plot(pol_spp$`Panthera onca`$states,
     main = paste0(names(pol_spp)[[1]], " - States"), mar = NA)
plot(pol_spp$`Panthera onca`$countries,
     main = paste0(names(pol_spp)[[1]], " - Countries"), mar = NA)
plot(pol_spp$`Chaetomys subspinosus`$states,
     main = paste0(names(pol_spp)[[2]], " - States"), mar = NA)
plot(pol_spp$`Chaetomys subspinosus`$countries,
     main = paste0(names(pol_spp)[[2]], " - Countries"), mar = NA)
```
```{r IMG02, eval=TRUE, echo = FALSE}
knitr::include_graphics("vignettes_img/IMG02.png")
```
As ocorrências dessas duas espécies foram baixadas do GBIF usando a função `plantR::rgbif2()`. Essas ocorrências estão salvas como exemplos de dados no pacote (para mais detalhes, veja `?faunabr::occurrences`). Podemos importá-las com:

```{r}
data("occurrences")
head(occurrences)
#>         species         x          y  source
#> 1 Panthera onca -90.38409  17.377023    gbif
#> 2 Panthera onca -90.24368  17.240507    gbif
#> 3 Panthera onca -77.36680   0.287624    gbif
#> 4 Panthera onca -56.61023 -17.239688    gbif
#> 5 Panthera onca -61.04386  -2.387029    gbif
#> 6 Panthera onca -77.28850   0.288757    gbif
```

Os dados de entrada com registros devem ser um dataframe com pelo menos 3 colunas: uma informando o nome da espécie, uma informando a longitude e outra informando a latitude.
Vamos verificar se há registros fora das áreas naturais de ocorrência das espécies considerando estados e países:

```{r}
occ_check <- filter_faunabr(data = bf, occ = occurrences,
                    by_state = TRUE, buffer_state = 20,
                    by_country = TRUE, buffer_country = 20,
                    value = "flag&clean", keep_columns = TRUE,
                    verbose = FALSE)
#> Returning list with flagged and cleaned occurrences
```

Como configuramos *value = "flag&clean"*, a função retornou uma lista com dois dataframes: um com todos os registros sinalizados se passaram (TRUE) ou não (FALSE) em cada teste; e outro apenas com os registros que passaram em todos os testes.

Vamos usar o pacote [mapview](https://r-spatial.github.io/mapview/) para plotar um mapa interativo dos registros sinalizados:

```{r, warning=FALSE, results='hide'}
#Instala pacote (se necessário)
if(!require(mapview)){
    install.packages("mapview")
}
#Carregar pacote
library(mapview)
library(terra)
```

Vamos verificar os registros de *Panthera onca*, plotando os registros e o mapa obtido anteriormente. Os pontos verdes representam os registros que passaram no teste dos estados, ou seja, registros dentro dos estados com ocorrência confirmada da espécie. Os pontos vermelhos são os registros que caem fora dos estados. Os pontos cinzas são registros que não foram testados porque estão fora do Brasil.

```{r, fig.width= 9}
#Converter pontos para spatvector
panthera_occ <- subset(occ_check$flagged,
                       occ_check$flagged$species == "Panthera onca")
panthera_occ <- vect(panthera_occ, geom = c("x", "y"),
                     crs = crs(pol_spp$`Panthera onca`$states))

#Plot interativo
mapview(pol_spp$`Panthera onca`$states,
        layer.name = "States with confirmed occurrences") +
  mapview(panthera_occ, zcol = "inside_state", layer.name = "Inside state?",
          col.regions = c("red", "green"))
```
```{r IMG03, eval=TRUE, echo = FALSE, fig.pos="H", out.width="90%"}
knitr::include_graphics("vignettes_img/IMG03.png")
```
Podemos observar o mesmo com os países: os pontos verdes representam registros que passaram no teste e os pontos vermelhos representam registros que não passaram no teste.

```{r, fig.width= 9}
#Plot interativo
mapview(pol_spp$`Panthera onca`$countries,
        layer.name = "Countries with confirmed occurrences", burst = TRUE, legend = F) +
  mapview(panthera_occ, zcol = "inside_country",
          col.regions = c("red", "green"),
          layer.name = "Inside country?")
```

```{r IMG04, eval=TRUE, echo = FALSE, fig.pos="H", out.width="90%"}
knitr::include_graphics("vignettes_img/IMG04.png")
```

---
title: "1. Getting started with faunabr"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{1. Getting started with faunabr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = F,
  warning = FALSE
)
```

## Introduction
### What is Fauna do Brasil?
The [Catálogo Taxonômico da Fauna do Brasil](https://fauna.jbrj.gov.br/fauna) is the result of the collaborative efforts of over 500 zoologists, each specializing in various animal groups native to Brazil. This comprehensive database offers detailed and standardized morphological descriptions, nomenclatural information, geographic distribution, and identification keys for both native and non-native animals found in Brazil.

The faunabr package provides a suite of functions designed to retrieve, filter, and spatialize data from the Fauna do Brasil dataset.

## Overview of the functions:

+ `get_faunabr()`: download the latest or an older version of Fauna do Brasil database.
+ `fauna_version()`: check if you have the latest version of Fauna do Brasil data available in a directory.
+ `load_faunabr()`: load Fauna do Brasil database.
+ `translate_faunabr()`: translates information in the "lifeForm", "origin", "habitat", "taxonRank" and "taxonomicStatus" columns between Portuguese and English.
+ `fauna_attributes()`: display all the options available to filter species by its characteristics.
+ `select_fauna()`: filter species based on its characteristics and distribution.
+ `extract_binomial()`: extract the binomial name (Genus + specific epithet) from a Scientific Name.
+ `fauna_synonym()`: Retrieve synonyms for species.
+ `check_fauna_names()`: check if the species names are correct.
+ `subset_fauna()`: subset a list of species from Fauna do Brasil.
+ `fauna_spat_occ()`: get Spatial polygons (SpatVectors) of species based on its distribution.
+ `filter_faunabr()`: removes or flags records outside of the species' natural ranges.
+ `fauna_pam()`: get a presence-absence matrix of species based on its distribution.

## Installation

### Install development version from GitHub

You can install the released version of *faunabr* from [github](https://github.com/wevertonbio/faunabr) with:
```{r, results='hide', message=FALSE}
if(!require(remotes)){
    install.packages("remotes")
}

if(!require(faunabr)){
remotes::install_github('wevertonbio/faunabr')}

library(faunabr)
```

## Get data from Fauna do Brasil
All data included in the Fauna do Brasil (i.e., nomenclature, life form, and distribution) are stored in Darwin Core Archive data sets, which is updated often. Before downloading the data available in the Fauna do Brasil, we need to create a folder to save the data:

```{r}
#Creating a folder in a temporary directory
#Replace 'file.path(tempdir(), "faunabr")' by a path folder to be create in 
#your computer
my_dir <- file.path(file.path(tempdir(), "faunabr"))
dir.create(my_dir)
```

You can now utilize the `get_faunabr` function to retrieve the most recent version of the data:

```{r, results='hide', message=FALSE, warning=FALSE}
get_faunabr(output_dir = my_dir, #directory to save the data
            data_version = "latest", #get the most recent version available
            translate = TRUE, #translate data to English
            overwrite = T) #Overwrite data, if it exists
```

The function will take a few seconds to download the data and a few minutes to merge the datasets into a single data.frame. Upon successful completion, you will find a folder named with the version of Fauna do Brasil. This folder contains the downloaded raw dataset (TXT files in Portuguese) and a file named **CompleteBrazilianFauna.gz**. The latter represents the finalized dataset. By default, the data is translated into English, but you can keep the original Portuguese version by setting `translate = FALSE`.

You also have the option to download an older, specific version of the Fauna do Brasil dataset. To explore the available versions, please refer to [this link](https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil). For downloading a particular version, simply replace 'latest' with the desired version number. For example:

```{r, results='hide', message=FALSE, warning=FALSE}
get_faunabr(output_dir = my_dir, #directory to save the data
            data_version = "1.10", #Version 1.10, published on 2024-02-01
            overwrite = T) #Overwrite data, if it exists
```

As previously mentioned, you will find a folder named '1.10' within the designated directory.

To view the available versions in your specified directory, run:

```{r}
fauna_version(data_dir = my_dir)
#> You have the following versions of Fauna do Brasil:
#>  1.17
#>  1.10
#>  It includes the latest version:  1.17
```

## Loading the data
In order to use the other functions of the package, you need to load the data into your environment. To achieve this, utilize the `load_faunabr()` function. By default, the function will automatically search for the latest available version in your directory. However, you have the option to specify a particular version using the *data_version* parameter.
Additionally, you can choose between two versions of the data: the 'short' version (containing the 19 columns required for run the other functions of the package) or the 'complete' version (with all original 31 columns). The function imports the 'short' version by default.

```{r}
#Short version
bf <- load_faunabr(data_dir = my_dir,
                   data_version = "latest",
                   type = "short") #short
#> Loading version 393.401
colnames(bf) #See variables from short version
#>  [1] "species"             "subspecies"          "scientificName"       
#>  [4] "validName"           "kingdom"             "phylum"
#>  [7] "class"               "order"               "family" 
#> [10] "genus"               "lifeForm"            "habitat"
#> [13] "states"              "countryCode"         "origin" 
#> [16] "taxonomicStatus"     "nomenclaturalStatus" "vernacularName"          
#> [19] "taxonRank"           "id"                  "language"
```

Note that the complete version has 12 more columns:

```{r}
#Complete version
bf_complete <- load_faunabr(data_dir = my_dir,
                   data_version = "latest",
                   type = "complete") #complete

colnames(bf_complete) #See variables from complete version
#>  [1] "id"                       "taxonID"                 
#>  [3] "species"                  "subspecies"       
#>  [5] "scientificName"           "validName"                   
#>  [7] "validNameUsage"           "parentNameUsage"                             
#>  [9] "namePublishedInYear"      "higherClassification"      
#> [11] "kingdom"                  "phylum"                                   
#> [13] "class"                    "order"                                   
#> [15] "family"                   "genus"               
#> [17] "specificEpithet"          "infraspecificEpithet"
#> [19] "taxonRank"                "scientificNameAuthorship"                  
#> [21] "taxonomicStatus"          "nomenclaturalStatus" 
#> [23] "vernacularName"           "lifeForm"      
#> [25] "habitat"                  "origin"
#> [27] "states"                   "countryCode"    
#> [29] "modified"                 "bibliographicCitation"                              
#> [31] "relationshipOfResource"   "language"                             
```

### Translate information
By default, the data downloaded from [Catálogo Taxonômico da Fauna do Brasil](https://fauna.jbrj.gov.br/fauna) is translated into English, unless you set `translate = FALSE` in `get_faunabr()` function. However, you can use the `translate_faunabr()` function to translate the loaded data between English and Portuguese:
```{r}
#Translate from English to Portuguese
bf_portuguese <- translate_faunabr(data = bf, to = "pt_br")
# See lifeforms (in portuguese)
fauna_attributes(bf_portuguese, "lifeForm")
#>  $lifeForm
#>                 lifeForm
#> 1               colonial
#> 2               comensal
#> 3           ectoparasito
#> 4        ectoparasitoide
#> 5           endoparasito
#> 6        endoparasitoide
#> 7              epibionte
#> 8              eussocial
#> 9              herbivoro
#> 10      hiperparasitoide
#> 11             inquilino
#> 12                mutual
#> 13           polinizador
#> 14              predador
#> 15                sessil
#> 16 vida_livre_individual

#Translate from Portuguese to English
bf_english <- translate_faunabr(data = fauna_portuguese, to = "en")
#See life forms(in English)
fauna_attributes(bf_english, "lifeForm")
#>  $lifeForm
#>                  lifeForm
#> 1                colonial
#> 2               commensal
#> 3            ectoparasite
#> 4          ectoparasitoid
#> 5            endoparasite
#> 6          endoparasitoid
#> 7                epibiont
#> 8                eusocial
#> 9  free_living_individual
#> 10              herbivore
#> 11        hyperparasitoid
#> 12              inquiline
#> 13            mutualistic
#> 14              polynizer
#> 15               predator
#> 16                sessile

```

## Save data

If you want to save the datasets to open with an external editor (e.g. Excel), you can save the data.frame as a CSV file:
```{r, eval=FALSE, warning=FALSE, message=FALSE, }
write.csv(x = bf,
          file = file.path(my_dir, "BrazilianFauna.csv"),
          row.names = F)
```

---
title: "2. Selecionar e filtrar espécies"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{2. Selecionar e filtrar espécies}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  warning = FALSE
)
```

### Selecionar uma lista de espécies
Selecionar uma lista de espécies-alvo é uma tarefa comum em estudos macroecológicos e de conservação. Por exemplo, um pesquisador pode desejar modelar a distribuição de mamíferos arborícolas que ocorrem em um estado específico.

Aplicando diferentes filtros ao banco de dados, podemos obter uma lista verificada das espécies de animais brasileiros para qualquer estado brasileiro ou país. Além disso, o filtro pode ser aplicado para selecionar espécies por filo, classe, ordem, família, gênero, forma de vida, habitat, origem e status taxonômico.
Neste tutorial, vamos ver como utilizar o pacote *faunabr* para selecionar uma lista de espécies com base nessas características.

## Carregando os dados
Antes de começar, utilize a função `load_faunabr()` para carregar os dados. Para informações mais detalhadas sobre como obter e carregar os dados, consulte [Primeiros passos com faunabr](https://wevertonbio.github.io/faunabr/articles/primeiros_passos.html)

```{r}
library(faunabr)
#Carregar dados
bf <- load_faunabr(data_dir = my_dir, #Pasta onde foi salvo o arquivo com get_faunabr()
                   data_version = "latest", #Versão mais recente
                   type = "short") #Versão resumida
#> Loading version 1.3
```

Aqui, vamos utilizar os dados baixados em português (`translate = FALSE` em `get_faunabr()`).

## Como selecionar uma lista de espécies?

Um dos principais objetivos deste pacote é auxiliar na seleção de uma lista de espécies com base na classificação taxonômica, características e distribuição. Especificamente, você pode filtrar por:

* filo, classe, ordem, família e gênero;
* forma de vida e habitat;
* estados brasileiros e países com ocorrências confirmadas;
* origem;
* status taxonômico.

Para explorar todas as opções disponíveis para cada filtro, utilize a função `fauna_attributes()` com os atributos desejados. Esta função fornecerá uma lista de data.frames com as opções disponíveis para usar na função `select_fauna()`:

```{r}
#Obter as opções disponíveis para filtrar por forma de vida, habitat e origem
fauna_at <- fauna_attributes(data = bf,
                             attribute = c("lifeForm", "habitat", "origin"))
head(fauna_at$lifeForm)
#>          lifeForm
#> 1        colonial
#> 2       comensal
#> 3    ectoparasito
#> 4 ectoparasitoide
#> 5    endoparasito
#> 6  endoparasitoide

head(fauna_at$habitat)
#> habitat
#> 1          agua_doce
#> 2 aguas_subterraneas
#> 3            arboreo
#> 4        cavernicola
#> 5          fossorial
#> 6         hiporreico

head(fauna_at$origin)
#>         origin
#> 1 criptogenica
#> 2      exotica
#> 3       nativa
```

Como exemplo, vamos considerar o cenário em que desejamos obter uma lista de todos os insetos arbóreos nativos com ocorrências confirmadas no Rio de Janeiro:

```{r}
insects_rj <- select_fauna(data = bf, include_subspecies = FALSE,
                           phylum = "all",
                           class = "Insecta", #Insetos
                           order = "all",
                           family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "arboreo", filter_habitat = "in",
                           states = "RJ", #Rio de Janeiro
                           filter_states = "in", #Que ocorrem NO RJ
                           country = "all", filter_country = "in",
                           origin = "nativa",
                           taxonomicStatus = "valido")
nrow(insects_rj)
#> [1] 114
```

O filtro retornou 114 espécies que atendem aos critérios especificados. É importante notar que essas seleções incluem espécies com ocorrências confirmadas no Rio de Janeiro, e algumas delas também podem ter ocorrências confirmadas em outros estados.

```{r}
#Primeiros 7 valores únicos de Estados onde as espécies ocorrem
unique(insects_rj$states)[1:7]
#> [1] "AC;AM;AP;BA;CE;DF;ES;GO;MA;MG;MS;MT;PA;PE;RJ;RN;RO;RR;SE;SP;TO"
#> [2] "BA;MA;MG;MT;PR;RJ;RS;SC;SP"                                    
#> [3] "ES;MG;PE;PR;RJ;RS;SP"                                          
#> [4] "ES;PR;RJ;SP"                                                   
#> [5] "RJ"                                                            
#> [6] "RJ;SP"                                                         
#> [7] "ES;MG;RJ;RS;SC;SP" 
```

Se você deseja selecionar espécies com ocorrências confirmadas apenas no Rio de Janeiro, e não em nenhum outro estado, modifique o parâmetro *filter_state* para "only":

```{r}
insects_rj_only <- select_fauna(data = bf, include_subspecies = FALSE,
                           phylum = "all",
                           class = "Insecta", #Insetos
                           order = "all",
                           family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "arboreo", filter_habitat = "in",
                           states = "RJ", #Rio de Janeiro
                           filter_states = "only", #Com ocorrência APENAS no RJ
                           country = "all", filter_country = "in",
                           origin = "nativa",
                           taxonomicStatus = "valido")
nrow(insects_rj_only)
#> [1] 22
unique(insects_rj_only$states)
#> [1] "RJ"
```

Agora, o filtro retornou 22 espécies, todas exclusivamente confinadas ao Rio de Janeiro.

Além disso, o pacote oferece a flexibilidade de combinar vários filtros (consulte `?select_fauna` para mais detalhes). Por exemplo, considere o cenário em que desejamos compilar uma lista de insetos nativos com ocorrências confirmadas nos estados do Paraná (PR), Santa Catarina (SC) e Rio Grande do Sul (RS):

```{r}
insects_south <- select_fauna(data = bf, include_subspecies = FALSE,
                           phylum = "all",
                           class = "Insecta", #Insetos
                           order = "all",
                           family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "all", filter_habitat = "in",
                           states = c("PR", "SC", "RS"), #Estados do Sul do Brasil
                           filter_states = "in", #EM qualquer um dos estados
                           country = "all", filter_country = "in",
                           origin = "nativa",
                           taxonomicStatus = "valido")
nrow(insects_south)
#> [1] 11461

#Primeiros 10 valores únicos de Estados onde as espécies ocorrem
unique(insects_south$states)[1:10]
#> [1] "PR;SP" 
#> [2] "BA;MT;RJ;RS" 
#> [3] "RS"   
#> [4] "PR"   
#> [5] "PR;RS"  
#> [6] "GO;PA;RO;RR;SC"
#> [7] "MG;PR;SP"    
#> [8] "GO;MG;PR;RS;SC"
#> [9] "GO;RJ;SC"       
#> [10] "AC;AL;AM;AP;BA;CE;DF;ES;GO;MA;MG;MS;MT;PA;PB;PE;PI;PR;RJ;RN;RO;RR;RS;SC;SE;SP;TO"
```

Como utilizamos *filter_state = "in"*, nossa seleção englobou espécies que ocorrem nos três estados, mas também aquelas presentes apenas em dois ou mesmo em um deles, além de outros estados. Para impor um critério mais rigoroso, selecionando apenas as espécies com ocorrências confirmadas em todos os três estados, podemos usar *filter_state = "and"*:

```{r}
insects_south_and <- select_fauna(data = bf, include_subspecies = FALSE,
                           phylum = "all",
                           class = "Insecta", #Insetos
                           order = "all",
                           family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "all", filter_habitat = "in",
                           states = c("PR", "SC", "RS"), 
                           filter_states = "and", #PR & SC & RS
                           country = "all", filter_country = "in",
                           origin = "nativa",
                           taxonomicStatus = "valido")
nrow(insects_south_and)
#> [1] 1924

#Primeiros 10 valores únicos de Estados onde as espécies ocorrem
unique(insects_south_and$states)[1:10]
#> [1] "GO;MG;PR;RS;SC"  
#> [2] "AC;AL;AM;AP;BA;CE;DF;ES;GO;MA;MG;MS;MT;PA;PB;PE;PI;PR;RJ;RN;RO;RR;RS;SC;SE;SP;TO"
#> [3] "AC;AM;MG;MS;MT;PA;PE;PR;RO;RS;SC;SP" 
#> [4] "AM;BA;CE;DF;ES;GO;MG;MS;MT;PA;PE;PR;RJ;RS;SC;SP" 
#> [5] "AC;AM;AP;BA;CE;DF;ES;GO;MG;MS;MT;PA;PE;PR;RJ;RO;RR;RS;SC;SE;SP;TO"
#> [6] "BA;ES;MG;PR;RJ;RS;SC;SP"
#> [7] "AM;DF;ES;GO;PR;RJ;RR;RS;SC;SP"  
#> [8] "ES;GO;MG;PR;RJ;RS;SC;SP"     
#> [9] "AC;AM;AP;BA;CE;ES;GO;MA;MG;MT;PA;PR;RJ;RO;RS;SC;SP;TO" 
#> [10] "GO;MG;MS;PR;RO;RS;SC;SP"
```

Agora, nossa seleção consistiu exclusivamente em espécies com ocorrências confirmadas em todos os estados especificados. No entanto, ao utilizar o argumento *"and"*, permitimos que o filtro incluísse espécies com ocorrências em estados adicionais. Para limitar o filtro a espécies com ocorrências confirmadas exclusivamente nos três estados, sem ocorrências em outros lugares, podemos usar *filter_state = "only"*:

```{r}
insects_south_only <- select_fauna(data = bf, include_subspecies = FALSE,
                           phylum = "all",
                           class = "Insecta", #Insetos
                           order = "all",
                           family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "all", filter_habitat = "in",
                           states = c("PR", "SC", "RS"), 
                           filter_states = "only", #Apenas no PR & SC & RS
                           country = "all", filter_country = "in",
                           origin = "nativa",
                           taxonomicStatus = "valido")
nrow(insects_south_only)
#> [1] 134

#Valor único de Estados onde as espécies ocorrem
unique(insects_south_only$states)
#> [1] "PR;RS;SC"
```
Agora, o filtro retornou apenas 134 espécies, todas ocorrendo em PR, SC e RS; sem registros de ocorrências em qualquer outro estado.

## Checagem de nomes científicos e subseleção de espécies

### Extraindo o nome binomial
Além de selecionar uma lista de espécies com base em suas características, o pacote também inclui uma função para subselecionar espécies por nome. A função funciona exclusivamente com nomes binomiais (Gênero + Epíteto específico), como *Panthera onca*, e não funciona com nomes científicos completos (incluindo Autor ou Epíteto infraespecífico), como *Panthera onca (Linnaeus, 1758)*.

Se você tiver uma lista de espécies com nomes científicos completos, pode extrair os nomes binomiais usando a função `get_binomial()`. Não se preocupe com espaços excessivos entre as palavras, a função removerá quaisquer espaços extras dos nomes.

```{r}
complete_names <- c("Pantera onca (Linnaeus, 1758)",
                    "Zonotrichia capensis subtorquata Swainson, 1837",
                    "Mazama bororo Duarte, 1996",
                    "Mazama jucunda Thomas, 1913",
                    "Arrenurus tumulosus intercursor",
                    "Araucaria angustifolia")
#Panthera onca com erros de digitação para ilustrar como a próxima função os corrige
#Araucaria angustifolia (uma planta) foi usada apenas como um exemplo que será utilizado para ilustrar a próxima função
binomial_names <- extract_binomial(species_names = complete_names)
binomial_names
#> [1] "Pantera onca"                         "Zonotrichia capensis" 
#> [3] "Mazama bororo"                        "Mazama jucunda" 
#> [5] "Arrenurus tumulosus"                  "Araucaria angustifolia" 
```
### Checando se os nomes das espécies estão corretos
Vamos agora verificar a ortografia, o status nomenclatural e o status taxonômico dos nomes das espécies usando a função `check_fauna_names()`. Se a função não conseguir localizar o nome de uma espécie no banco de dados (devido a um erro tipográfico, por exemplo), ela pode sugerir nomes potenciais com base em similaridades com outras entradas no banco de dados.
Para ver como a função funciona, vamos utilizar o dataset *binomial_names* criado anteriormente:

```{r}
#Checar nomes
checked_names <- check_fauna_names(data = bf, 
                             species = binomial_names,
                             include_subspecies = FALSE,
                             max_distance = 0.1)
tibble::tibble(checked_names) #Exibir tabela como tibble
#> # A tibble: 6 x 9
#>   input_name               Spelling           Suggested_name       Distance taxonomicStatus nomenclaturalStatus validName   family        matches
#>   <chr>                    <chr>              <chr>                   <dbl> <chr>           <chr>               <chr>          <chr>         <chr>  
#> 1 "Zonotrichia capensis"   Correct            Zonotrichia capensis        0 valido    NA                  NA             Passerellidae single 
#> 2 "Mazama bororo"          Correct            Mazama bororo               0 sinonimo        NA                  Mazama jucunda Cervidae      single 
#> 3 "Mazama jucunda"         Correct            Mazama jucunda              0 valido     NA                  NA             Cervidae      single 
#> 4 "Arrenurus tumulosus"    Correct            Arrenurus tumulosus         0 valido     NA                  NA             Arrenuridae   single 
#> 5 "Pantera onca"           Probably_incorrect Panthera onca               2 valido     NA                  NA             Felidae       single 
#> 6 "Araucaria angustifolia" Not_found          NA                         NA NA              NA                  NA             NA            none   
```

Podemos ver que *Zonotrichia capensis*, *Mazama jucunda* e *Arrenurus tumulosus* estão escritos corretamente e são nomes validos. *Mazama bororo* está escrito corretamente, mas é um sinônimo de *Mazama jucunda*.
No caso de *Pantera onca*, a ortografia parece estar potencialmente incorreta (já que o nome não foi encontrado no banco de dados); no entanto, um nome semelhante, *Panthera onca*, é sugerido pela função. A coluna 'Distance' indica a distância de edição de Levenshtein entre a entrada e o nome sugerido.
*Araucaria angustifolia* foi marcada como Not_found (como esperado, dado que *Araucaria angustifolia* não é um animal!). Consequentemente, o nome não foi localizado no banco de dados e não havia nomes comparáveis disponíveis.

### Subselecionando espécies

Para obter informações sobre espécies do banco de dados Fauna do Brasil a partir de uma lista de espécies pré-definida, utilize a função `subset_species()`. Para obter um desempenho ideal, recomendamos fortemente o uso das funções `get_binomial()` e `check_names()` previamente. Isso garante que você esteja trabalhando exclusivamente com espécies presentes no banco de dados da Fauna do Brasil.
Para ver como a função funciona, vamos usar os nomes validos em *checked_names* criados anteriormente:

```{r}
#Obter apenas nomes validos
valids <- unique(checked_names$validName)
valids <- na.omit(valids) #Remove NAs

#Subselecionar especies
my_sp <- subset_fauna(data = bf, species = valids,
                      include_subspecies = FALSE)
tibble::tibble(my_sp)
# A tibble: 4 x 21
#>   species           subspecies scientificName validName kingdom phylum class order family genus lifeForm habitat states countryCode origin
#>   <chr>             <chr>      <chr>          <chr>        <chr>   <chr>  <chr> <chr> <chr>  <chr> <chr>    <chr>   <chr>  <chr>       <chr> 
#> 1 Panthera onca     NA         Panthera onca~ NA           Animal~ Chord~ Mamm~ Carn~ Felid~ Pant~ "vida_l~ "terre~ AC;AM~ AR;BO;BR;B~ nativa
#> 2 Arrenurus tumulo~ NA         Arrenurus (In~ NA           Animal~ Arthr~ Arac~ Trom~ Arren~ Arre~ ""       ""      NA     NA          NA    
#> 3 Mazama jucunda    NA         Mazama jucund~ NA           Animal~ Chord~ Mamm~ Arti~ Cervi~ Maza~ "herbiv~ "terre~ BA;ES~ BR          nativa
#> 4 Zonotrichia cape~ NA         Zonotrichia c~ NA           Animal~ Chord~ Aves  Pass~ Passe~ Zono~ ""       ""      NA     NA          nativa  
# i 5 more variables: nomenclaturalStatus <chr>, vernacularName <chr>, taxonRank <chr>, id <int>, language <chr>
```
Também podemos incluir subespécies e/ou variedades:

```{r}
my_sp2 <- subset_fauna(data = bf, species = valids,
                      include_subspecies = TRUE)
tibble::tibble(my_sp2) #print data.frame as tibble
#> # A tibble: 14 × 19
#>    species      subspecies scientificName validName kingdom phylum class order family genus lifeForm habitat states countryCode #> origin
#>    <chr>        <chr>      <chr>          <chr>        <chr>   <chr>  <chr> <chr> <chr>  <chr> <chr>    <chr>   <chr>  <chr>       #> <chr> 
#>  1 Panthera on… ""         Panthera onca… ""           Animal… Chord… Mamm… Carn… Felid… Pant… "free_l… "terre… AC;AM… argentina;… #> "nati…
#>  2 Arrenurus (… ""         Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      "" #>    
#>  3 Arrenurus (… "Arrenuru… Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      "" #>    
#>  4 Arrenurus (… "Arrenuru… Arrenurus (In… ""           Animal… Arthr… Arac… Trom… Arren… Arre… ""       ""      NA     brazil      "" #>    
#>  5 Mazama jucu… ""         Mazama jucund… ""           Animal… Chord… Mamm… Arti… Cervi… Maza… "herbiv… "terre… BA;ES… brazil      #> "nati…
#>  6 Zonotrichia… ""         Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AL;AP… brazil      #> "nati…
#>  7 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AP     brazil      #> "nati…
#>  8 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      RR     brazil      #> "nati…
#>  9 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      RR     brazil      #> "nati…
#> 10 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      BA;MA… brazil      #> "nati…
#> 11 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      PA     brazil      #> "nati…
#> 12 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      AP;PA… brazil      #> "nati…
#> 13 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      ES;MG… brazil      #> "nati…
#> 14 Zonotrichia… "Zonotric… Zonotrichia c… ""           Animal… Chord… Aves  Pass… Passe… Zono… "free_l… ""      TO     brazil      #> "nati…
#> # ℹ 4 more variables: taxonomicStatus <chr>, nomenclaturalStatus <chr>, vernacularName <chr>, taxonRank <chr>
#> 
```

### Obtendo sinônimos

Podemos obter todos os sinônimos de uma lista de espécies. Isso pode ser particularmente útil, por exemplo, ao buscar registros de uma espécie e todos os seus sinônimos (conforme listado na Fauna do Brasil) em bancos de dados online como o GBIF. Para isso, utilize a função `fauna_synonym()`. Como exemplo, vamos buscar os sinônimos de três espécies:
```{r}
spp <- c("Panthera onca", "Mazama jucunda", "Subulo gouzoubira")
spp_syn <- fauna_synonym(data = bf, species = spp)
spp_syn
#>               validName               synonym   taxonomicStatus
#> 49343     Panthera onca                  <NA>    valido
#> 60523    Mazama jucunda         Mazama bororo    sinonimo
#> 61168 Subulo gouzoubira    Mazama gouazoubira    sinonimo
```
Podemos ver que *Mazama jucunda* possui o sinônimo *Mazama bororo*; enquanto *Subulo gouzoubira* possui o sinônimo *Mazama gouazoubira*. *Panthera onca* não possui nenhum sinônimo listado na base de dados.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{states}
\alias{states}
\title{SpatVector of the federal states of Brazil}
\format{
A \code{SpatVector} with 27 geometries and 3 attributes:
\describe{
\item{abbrev_state}{State acronym}
\item{name_state}{State's full name}
\item{name_region}{The region to which the state belongs}
}
}
\usage{
states
}
\description{
A simplified and packed SpatVector of the polygons of the federal states of
Brazil. The spatial data was originally obtained from
\code{geobr::read_state}. Borders have been simplified by removing vertices
of borders using \code{terra::simplifyGeom}. It's necessary unpack the
Spatvectos using \code{terra::unwrap}

@usage data(states)
states <- terra::unwrap(states)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fauna_by_vernacular.R
\name{fauna_by_vernacular}
\alias{fauna_by_vernacular}
\title{Search for taxa using vernacular names}
\usage{
fauna_by_vernacular(data, names, exact = FALSE)
}
\arguments{
\item{data}{(data.frame) the data.frame imported with the
\code{\link{load_faunabr}} function or generated with the function
\code{\link{select_fauna}}.}

\item{names}{(character) vernacular name ("Nome comum") of the species to be
searched}

\item{exact}{(logic) if TRUE, the function will search only for exact
matches. For example, if names = "veado-mateiro" and exact = TRUE, the function
will return only the species popularly known as "veado-mateiro". On the other
hand, if names = "veado-mateiro" and exact = FALSE, the function will return
other results as "Veado-mateiro-pequeno". Default = FALSE}
}
\value{
a data.frame with the species with vernacular names that match the
input names
}
\description{
Search for taxa using vernacular names
}
\examples{
data("fauna_data") #Load Fauna do Brasil data
#Search for species whose vernacular name is 'veado-mateiro'
veado_exact <- fauna_by_vernacular(data = fauna_data,
                                   names = "veado-mateiro",
                                   exact = TRUE)
veado_exact
#Search for species whose vernacular name is 'veado_mateiro', allowing non-exact
#matches
veado_not_exact <- fauna_by_vernacular(data = fauna_data,
                                       names = "veado-mateiro",
                                      exact = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{world_fauna}
\alias{world_fauna}
\title{SpatVector of the world countries}
\format{
A \code{SpatVector} with 258 geometries and 1 attribute:
\describe{
\item{name}{The name of the country (argentina, brazil, colombia, etc.)}
}
}
\usage{
world_fauna
}
\description{
A simplified and packed SpatVector of the world country polygons. The
spatial data was originally obtained from \code{rnaturalearth::ne_countries}.
Borders have been simplified by removing vertices of borders using
\code{terra::simplifyGeom}.It's necessary unpack the Spatvectos using
\code{terra::unwrap}

@usage data(world_fauna)
biomes <- terra::unwrap(world_fauna)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{fauna_data}
\alias{fauna_data}
\title{Catálogo Taxonômico da Fauna do Brasil database - Version 1.17}
\format{
A \code{data.frame} with 9558 rows and 19 variables:
\describe{
\item{species}{Species names}
\item{subspecies}{Subspecies names}
\item{scientificName}{Complete scientific name of the species}
\item{validName}{Valid name of the species (NA when the name in
species is already a valid name)}
\item{kingdom}{Kingdom to which species belongs (Animalia)}
\item{phylum}{Phylum to which species belongs}
\item{class}{Class to which species belongs}
\item{order}{Order to which species belongs}
\item{family}{Family to which species belongs}
\item{genus}{Genus to which species belongs}
\item{lifeForm}{Life form of the species (e.g: free_living_individual,
colonial, sessile, etc.)}
\item{habitat}{Habitat type of the species (e.g., terrestrial, arboreal,
freshwater, etc.)}
\item{states}{Federal states with confirmed occurrences of the species}
\item{countryCode}{Countries with confirmed occurrences of the species}
\item{origin}{Indicates whether the species is native, introduced,
domesticated, cryptogenic or invasive}
\item{taxonomicStatus}{Indicates the level of recognition and acceptance
of the species (valid or synonym)}
\item{nomenclaturalStatus}{Indicates the legitimacy and validity of the
species name (original_combination, changed_combination, etc.)}
\item{vernacularName}{Locally or culturally used name for the species}
\item{taxonRank}{Taxonomic rank (Species, Genus, Family, Order, etc). This
data contains only Species}
}
}
\usage{
data(fauna_data)
}
\description{
A dataset containing a subset of the Catálogo Taxonômico da Fauna do Brasil
database (version 1.17)
}
\references{
Brazilian Zoology Group. Catálogo Taxonômico da Fauna do Brasil. Available at:
https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{map_translation}
\alias{map_translation}
\title{Helpers for translating data}
\format{
A \code{list} with 5 data.frames ("lifeForm", "origin", "habitat",
"taxonRank", and "taxonomicStatus"). Each data.frame has 2 columns:
\describe{
\item{pt_br}{The attribute in Brazilian Portuguese.}
\item{en}{The attribute in English.}
}
}
\usage{
data(map_translation)
}
\description{
A list of data.frames used by \code{faunabr::translate_faunabr()} function.
\code{faunabr::map_translation}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fauna_discrepancies.R
\name{fauna_discrepancies}
\alias{fauna_discrepancies}
\title{Resolve discrepancies between species and subspecies information}
\usage{
fauna_discrepancies(data)
}
\arguments{
\item{data}{(data.frame) the data.frame imported with the
\code{\link{load_faunabr}} function.}
}
\value{
a data.frame with the discrepancies solved
}
\description{
Resolve discrepancies between species and subspecies information
}
\details{
In the original dataset, discrepancies may exist between species and
subspecies information. An example of a discrepancy is when
species occurs only in one state (e.g., SP), but a subspecies or variety
of the same species occurs in another states (e.g., SP and RJ). This function
rectifies such discrepancies by considering distribution (states and countries)
life form, and habitat. For instance, if a subspecies is recorded in a specific
state, it implies that the species also occurs in that state
}
\examples{
data("fauna_data") #Load fauna e Funga do Brasil data
#Check if discrepancies were solved in the dataset
attr(fauna_data, "solved_discrepancies")
#Solve discrepancies
fauna_solved <- fauna_discrepancies(fauna_data)
#Check if discrepancies were solved in the dataset
attr(fauna_solved, "solved_discrepancies")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translate_faunabr.R
\name{translate_faunabr}
\alias{translate_faunabr}
\title{Translate information in Brazilian Fauna database}
\usage{
translate_faunabr(data, map_list = NULL, to = "en")
}
\arguments{
\item{data}{(data.frame) the data.frame imported with the
\code{\link{load_faunabr}} function.}

\item{map_list}{(list) A list of data.frames used for translation. The default
is NULL, which means it uses \code{faunabr::map_translation}. If not NULL, its
structure (list names and data.frame column names) must be identical
to \code{faunabr::map_translation}.}

\item{to}{(character) The target language for translation. Available options
are "en" to translate from Portuguese to English, and "pt_br" to translate
from English to Portuguese. The default is "en".}
}
\value{
A data.frame with the values in the "lifeForm", "origin", "habitat",
"taxonRank", and "taxonomicStatus" columns translated.
}
\description{
This function translates information in the "lifeForm", "origin", "habitat",
"taxonRank", and "taxonomicStatus" columns between Portuguese and English.
}
\examples{
data("fauna_data") #Load data example (in English)
#Translate to Portuguese
fauna_portugues <- translate_faunabr(data = fauna_data, to = "pt_br")
# See attributes of lifeForm in Portuguese
fauna_attributes(fauna_portugues, attribute = "lifeForm")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{occurrences}
\alias{occurrences}
\title{Records of animal species}
\format{
A \code{data.frame} with 2798 rows and 3 variables:
\describe{
\item{species}{Species names (Panthera onca and Chaetomys subspinosus)}
\item{x}{Longitude}
\item{y}{Latitude}
\item{source}{record downloaded from GBIF or fake data}
}
}
\usage{
data(occurrences)
}
\description{
A dataset containing records of 2 species downloaded from GBIF, with
additional fake data. The records were obtained with \code{plantR::rgbif2}
}
\references{
GBIF, 2024. florabr R package: Records of plant species. https://doi.org/10.15468/DD.QPGEB7
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_faunabr.R
\name{load_faunabr}
\alias{load_faunabr}
\title{Load Brazilian Fauna database}
\usage{
load_faunabr(data_dir, data_version = "latest",
                    type = "short", verbose = TRUE, encoding = "UTF-8")
}
\arguments{
\item{data_dir}{(character) the same directory used to save the data
downloaded from Brazilian Fauna using the \link{get_faunabr} function.}

\item{data_version}{(character) the version of Brazilian Fauna database to
be loaded. It can be "latest", which will load the latest version
available; or another specified version, for example "1.2".
Default = "latest".}

\item{type}{(character) it determines the number of columns that will be
loaded. It can be "short" or "complete". Default = "short". See details.}

\item{verbose}{(logical) Whether to display messages during function
execution. Set to TRUE to enable display, or FALSE to run silently.
Default = TRUE.}

\item{encoding}{(character) the declared encodings for special characters.
Character strings in R can be declared to be encoded in "latin1" or "UTF-8".
Default: "UTF-8".}
}
\value{
A data.frame with the specified version (Default is the latest
available) of the Brazilian Fauna database. This data.frame is necessary to
run most of the functions of the package.
}
\description{
Load Brazilian Fauna database
}
\details{
The parameter type accepts two arguments. If type = short, it will load a
data.frame with the 20 columns needed to run the other functions of the
package: species, subspecies, scientificName, validName, kingdom, phylum,
class, order, family, genus, lifeForm, habitat, states, countryCode, origin,
taxonomicStatus, nomenclaturalStatus, vernacularName, and taxonRank.
If type = complete, it will load a data.frame with all 31 variables available
in Brazilian Fauna database.
}
\examples{
\dontrun{
#Creating a folder in a temporary directory
#Replace 'file.path(tempdir(), "faunabr")' by a path folder to be create in
#your computer
my_dir <- file.path(file.path(tempdir(), "faunabr"))
dir.create(my_dir)
#Download, merge and save data
get_fauna(output_dir = my_dir, data_version = "latest", overwrite = TRUE,
            verbose = TRUE)
#Load data
df <- load_faunabr(data_dir = my_dir, data_version = "latest",
                   type = "short")
}
}
\references{
Brazilian Zoology Group. Catálogo Taxonômico da Fauna do Brasil. Available at:
https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fauna_spat_occ.R
\name{fauna_spat_occ}
\alias{fauna_spat_occ}
\title{Get Spatial polygons (SpatVectors) of species based on its distribution
(states and countrys) according to Fauna do Brasil}
\usage{
fauna_spat_occ(data, species, state = TRUE, country = TRUE,
                      spat_state = NULL, spat_country = NULL, verbose = TRUE)
}
\arguments{
\item{data}{(data.frame) the data.frame imported with the
\code{\link{load_faunabr}} function.}

\item{species}{(character) one or more species names (only genus and
specific epithet, eg. "Panthera onca")}

\item{state}{(logical) get SpatVector of states with occurrence of the
species? Default = TRUE}

\item{country}{(logical) get SpatVector of countrys with occurrence of the
species? Default = TRUE}

\item{spat_state}{(SpatVector) a SpatVector of the Brazilian states. By
default, it uses the SpatVector provided by geobr::read_state(). It can be
another Spatvector, but the structure must be identical to
'faunabr::states', with a column called "abbrev_state" identifying the states
codes.}

\item{spat_country}{(SpatVector) a SpatVector of the world countries. By
default, it uses the SpatVector provided by rnaturalearth::ne_countries. It
can be another Spatvector, but the structure must be identical to
'faunabr::world_fauna', with a column called "country_code" identifying the
country codes.}

\item{verbose}{(logical) Whether to display species being filtered during
function execution. Set to TRUE to enable display, or FALSE to run silently.
Default = TRUE.}
}
\value{
A list with SpatVectors of states and/or countrys for each
specie.
}
\description{
Get Spatial polygons (SpatVectors) of species based on its distribution
(states and countrys) according to Fauna do Brasil
}
\examples{
library(terra)
data("fauna_data")
spp <- c("Panthera onca", "Mazama jucunda")
#Get states, countrys and intersection states-countrys of species
spp_spt <- fauna_spat_occ(data = fauna_data, species = spp, state = TRUE,
                          country = TRUE, verbose = TRUE)
#Plot states with confirmed occurrence of Panthera onca and Mazama jucunda
plot(spp_spt$`Panthera onca`$states)
plot(spp_spt$`Mazama jucunda`$states)
#Plot countries with confirmed occurrence of Panthera onca and Mazama jucunda
plot(spp_spt$`Panthera onca`$countries)
plot(spp_spt$`Mazama jucunda`$countries)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{country_codes}
\alias{country_codes}
\title{Country Codes and Names}
\format{
A \code{data.frame} with 244 rows and 2 variables:
\describe{
\item{map_name}{Country names as defined in \code{faunabr::world_fauna}.}
\item{country_code}{Country codes used in the Catálogo Taxonômico da Fauna
do Brasil.}
}
}
\usage{
data(country_codes)
}
\description{
A dataset containing country codes used in the Catálogo Taxonômico da Fauna
do Brasil along with their corresponding country names, as defined in
\code{faunabr::world_fauna}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fauna_version.R
\name{fauna_version}
\alias{fauna_version}
\title{Check if you have the latest version of Fauna do Brasil data available}
\usage{
fauna_version(data_dir)
}
\arguments{
\item{data_dir}{the directory where the data should be located.}
}
\value{
A message informing whether you have the latest version of Fauna do
Brasil available in the data_dir
}
\description{
This function checks if you have the latest version of the Fauna do Brasil
data available in a specified directory.
}
\examples{
#Check if there is a version of Fauna do Brasil data available in the
#current directory
fauna_version(data_dir = getwd())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fauna_pam.R
\name{fauna_pam}
\alias{fauna_pam}
\title{Get a presence-absence matrix}
\usage{
fauna_pam(data, by_state = TRUE, by_country= FALSE,
               remove_empty_sites = TRUE,
               return_richness_summary = TRUE,
               return_spatial_richness = TRUE,
               return_plot = TRUE)
}
\arguments{
\item{data}{(data.frame) a data.frame imported with the
\code{\link{load_faunabr}} function or generated by either
\code{\link{select_fauna}} or \code{\link{subset_fauna}} functions}

\item{by_state}{(logical) get occurrences by State. Default = TRUE}

\item{by_country}{(logical) get occurrences by countries. Default = FALSE}

\item{remove_empty_sites}{(logical) remove empty sites (sites without any
species) from final presence-absence matrix. Default = TRUE}

\item{return_richness_summary}{(logical) return a data.frame with the number
of species in each site. Default = TRUE}

\item{return_spatial_richness}{(logical) return a SpatVector with the number
of species in each site. Default = TRUE}

\item{return_plot}{(logical) plot map with the number of species in each
site.
Only works if return_spatial_richness = TRUE. Default = TRUE}
}
\value{
If return_richness_summary and/or return_spatial_richness is set to
TRUE, return a list with:
\itemize{
\item PAM: the presence-absence matrix (PAM)
\item Richness_summary: a data.frame with the number of species in each site
\item Spatial_richness: a SpatVector with the number of species in each site
(by State and/or country)
}

If return_richness_summary and return_spatial_richness is set to FALSE,
return a presence-absence matrix
}
\description{
Get a presence-absence matrix of species based on its distribution
(brazilian states and/or countries) according to Fauna do Brasil.
}
\examples{
#Test function
data("fauna_data") #Load fauna e Funga do Brasil data
#Select native species of mammals with occurrence only in Brazil
br_mammals <- select_fauna(data = fauna_data,
                           include_subspecies = FALSE, phylum = "all",
                           class = "Mammalia",
                           order = "all", family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "all", filter_habitat = "in",
                           states = "all", filter_states = "in",
                           country = "BR", filter_country = "only",
                           origin = "all", taxonomicStatus = "valid")
#Get presence-absence matrix in states
pam_mammals <- fauna_pam(data = br_mammals, by_state = TRUE,
                         by_country = FALSE,
                         remove_empty_sites = TRUE,
                         return_richness_summary = TRUE,
                         return_spatial_richness = TRUE,
                         return_plot = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fauna_attributes.R
\name{fauna_attributes}
\alias{fauna_attributes}
\title{Get available attributes to filter species}
\usage{
fauna_attributes(data, attribute)
}
\arguments{
\item{data}{(data.frame) a data.frame imported with the
\code{\link{load_faunabr}} function or a data.frame generated with the
\code{\link{select_fauna}} function.}

\item{attribute}{(character) the type of characteristic. Accept more than one
option. See detail to see the options.}
}
\value{
a list of data.frames with the available options to use in the
\code{\link{select_fauna}} function.
}
\description{
This function displays all the options available to filter species by its
characteristics
}
\details{
The attribute argument accepts the following options: phylum, class, family,
genus, lifeForm, habitat, states, country, origin, and taxonomicstatus. These options
represent different characteristics of species that can be used for filtering.
}
\examples{
data("fauna_data") #Load data example
# Get available states, countries and lifeForms to filter species
d <- fauna_attributes(data = fauna_data,
                    attribute = c("country", "lifeform", "states"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fauna_synonym.R
\name{fauna_synonym}
\alias{fauna_synonym}
\title{Retrieve synonyms for species}
\usage{
fauna_synonym(data, species,
                   include_subspecies = TRUE)
}
\arguments{
\item{data}{(data.frame) the data.frame imported with the
\code{\link{load_faunabr}} function}

\item{species}{(character) names of the species}

\item{include_subspecies}{(logical) include subspecies that are synonyms of
the species? Default = TRUE}
}
\value{
A data.frame containing unique synonyms of the specified species
along with relevant information on taxonomic status.
}
\description{
Retrieve synonyms for species
}
\examples{
data("fauna_data") #Load Flora e Funga do Brasil data
#Species to extract synonyms
spp <- c("Panthera onca", "Mazama jucunda", "Subulo gouzoubira")
spp_synonyms <- fauna_synonym(data = fauna_data, species = spp,
                              include_subspecies = FALSE)
spp_synonyms

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_fauna.R
\name{select_fauna}
\alias{select_fauna}
\title{Selection of species based on its characteristics and distribution}
\usage{
select_fauna(data, include_subspecies = FALSE, phylum = "all",
                      class = "all", order = "all", family = "all",
                      genus = "all",
                      lifeForm = "all", filter_lifeForm = "in",
                      habitat = "all", filter_habitat = "in",
                      states = "all", filter_states = "in",
                      country = "all", filter_country = "in",
                      origin = "all", taxonomicStatus = "valid")
}
\arguments{
\item{data}{(data.frame) the data.frame imported with the
\code{\link{load_faunabr}} function.}

\item{include_subspecies}{(logical) include subspecies?
Default = FALSE}

\item{phylum}{(character) The phyla for filtering the dataset. It can
be included more than one phylum. Default = "all".}

\item{class}{(character) The classes for filtering the dataset. It can
be included more than one class. Default = "all".}

\item{order}{(character) The orders for filtering the dataset. It can
be included more than one order. Default = "all".}

\item{family}{(character) The families for filtering the dataset. It can
be included more than one family. Default = "all".}

\item{genus}{(character) The genus for filtering the dataset. It can
be included more than one genus. Default = "all".}

\item{lifeForm}{(character) The life forms for filtering the dataset. It can
be included more than one lifeForm. Default = "all"}

\item{filter_lifeForm}{(character) The type of filtering for life forms. It
can be "in", "only", "not_in" and "and". See details for more about this
argument.}

\item{habitat}{(character) The life habitat for filtering the dataset. It can
be included more than one habitat. Default = "all"}

\item{filter_habitat}{(character) The type of filtering for habitat. It
can be "in", "only", "not_in" and "and". See details for more about this
argument.}

\item{states}{(character) The states for filtering the dataset. It can
be included more than one state. Default = "all".}

\item{filter_states}{(character) The type of filtering for states. It
can be "in", "only", "not_in" and "and". See Details for more about this
argument.}

\item{country}{(character) The country or countries with confirmed
occurrences for filtering the dataset. It can be included more than one
country. Default = "all".}

\item{filter_country}{(character) The type of filtering for country. It can
be "in", "only", "not_in" and "and". See details for more about this argument.}

\item{origin}{(character) The origin for filtering the dataset. It can
be "native", "introduced", "cryptogenic", "domesticaded" and "invasora".
Default = "all".}

\item{taxonomicStatus}{(character) The taxonomic status for filtering the
dataset. It can be "valid", "synonym" or "all".
Default = "valid".}
}
\value{
A new dataframe with the filtered species.
}
\description{
select_fauna allows filter species based on its
characteristics and distribution available in Brazilian Fauna
}
\details{
It's possible to choose 4 ways to filter by lifeform, by habitat,
by states and by country:
"in": selects species that have any occurrence of the determined values. It
allows multiple matches. For example, if country = c("brazil", argentina") and
filter_country = "in", it will select all species that occur in Brazil and/or
Argentina, some of which may also occur in other countries.

"only": selects species that have only occurrence of the determined values.
It allows only single matches. For example, if
country = c("brazil", argentina") and filter_country = "in", it will select
all species that occur exclusively in both countries, without any occurrences
in other countries.

"not_in": selects species that don't have occurrence of the determined
values. It allows single and multiple matches. For example,
if country = c("brazil", argentina") and filter_country = "not_in", it will select
all species without occurrences in Brazil and Argentina.

"and": selects species that have occurrence in all determined values. It
allows single and multiple matches. For example,
if country = c("brazil", argentina") and filter_country = "and", it will select
all species that occurs only in both countries,including species that occurs
in other countries too.

To get the complete list of arguments available for phylum, class, order,
family, genus, lifeForm, habitat, states, country and origins, use
the function \code{\link{fauna_attributes}}
}
\examples{
data("fauna_data") #Load data example
#Select endemic and native species of birds (Aves) with confirmed occurrence
#in Brazil or Argentina
aves_br_ar <- select_fauna(data = fauna_data, include_subspecies = FALSE,
                           phylum = "all", class = "Aves",
                           order = "all",
                           family = "all",
                           genus = "all",
                           lifeForm = "all", filter_lifeForm = "in",
                           habitat = "all", filter_habitat = "in",
                           states = "all", filter_states = "in",
                           country = c("BR", "AR"),
                           filter_country = "in",
                           origin = "native",
                           taxonomicStatus = "valid")
}
\references{
Brazilian Zoology Group. Catálogo Taxonômico da Fauna do Brasil. Available at:
https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_faunabr.R
\name{get_faunabr}
\alias{get_faunabr}
\title{Download the latest version of Catálogo Taxonômico da Fauna do Brasil}
\usage{
get_faunabr(output_dir, data_version = "latest",
                 solve_discrepancies = TRUE, translate = TRUE,
                 overwrite = TRUE, verbose = TRUE)
}
\arguments{
\item{output_dir}{(character) a directory to save the data downloaded from
Fauna do Brasil}

\item{data_version}{(character) Version of the Fauna do Brasil database to
download. Use "latest" to get the most recent version, which is updated
frequently. Alternatively, specify an older version (e.g.,
data_version = "1.2").Default value is "latest".}

\item{solve_discrepancies}{Resolve inconsistencies between species and
subspecies  information. When set to TRUE (default), species
information is updated based on unique data from subspecies.
For example, if a subspecies occurs in a certain state, it implies that the
species also occurs in that state.}

\item{translate}{(logical) whether to translate the original dataset
("lifeForm", "origin", "habitat", and "taxonRank") from Portuguese to English.
Default is TRUE.}

\item{overwrite}{(logical) If TRUE, data is overwritten. Default = TRUE.}

\item{verbose}{(logical) Whether to display messages during function
execution. Set to TRUE to enable display, or FALSE to run silently.
Default = TRUE.}
}
\value{
The function downloads the latest version of the Catálogo Taxonômico da Fauna
do Brasil database from the official source. It then merges the information
into a single data.frame, containing details on species, taxonomy, occurrence,
and other relevant data.
The merged data.frame is then saved as a file in the specified output
directory. The data is saved in a format that allows easy loading using the
\code{\link{load_faunabr}} function for further analysis in R.
}
\description{
This function downloads the latest or an older version of Catálogo Taxonômico
da Fauna do Brasil database, merges the information into a single data.frame,
and saves this data.frame in the specified directory.
}
\examples{
\dontrun{
#Creating a folder in a temporary directory
#Replace 'file.path(tempdir(), "faunaabr")' by a path folder to be create in
#your computer
my_dir <- file.path(file.path(tempdir(), "faunabr"))
dir.create(my_dir)
#Download, merge and save data
get_faunabr(output_dir = my_dir)
}
}
\references{
Brazilian Zoology Group. Catálogo Taxonômico da Fauna do Brasil. Available at:
https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_binomial.R
\name{extract_binomial}
\alias{extract_binomial}
\title{Extract the binomial name (Genus + specific epithet) from a Scientific Name}
\usage{
extract_binomial(species_names)
}
\arguments{
\item{species_names}{(character) Scientific names to be converted to
binomial names}
}
\value{
A vector with the binomial names (Genus + specific epithet).
}
\description{
Extract the binomial name (Genus + specific epithet) from a Scientific Name
}
\examples{
spp <- c("Panthera onca (Linnaeus, 1758)",
"Zonotrichia capensis subtorquata Swainson, 1837",
"Paraganaspis egeria Díaz & Gallardo, 1996",
"Arrenurus tumulosus intercursor")
spp_new <- extract_binomial(species_names = spp)
spp_new

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_fauna_names.R
\name{check_fauna_names}
\alias{check_fauna_names}
\title{Check species names}
\usage{
check_fauna_names(data, species, max_distance = 0.1,
                         include_subspecies = TRUE)
}
\arguments{
\item{data}{(data.frame) the data.frame imported with the
\code{\link{load_faunabr}} function.}

\item{species}{(character) names of the species to be checked.}

\item{max_distance}{(numeric) Maximum distance (as a fraction) allowed for
searching suggestions when the name is misspelled. It can be any value
between 0 and 1. The higher the value, the more suggestions are returned.
For more details, see \code{\link[base:agrep]{agrep}}. Default = 0.1.}

\item{include_subspecies}{whether include subspecies when checking names.
Default = TRUE.}
}
\value{
a data.frame with the following columns:
\itemize{
\item input_name: the species names informed in species argument
\item Spelling: indicates if the species name is Correct (a perfect match with a
species name in the Flora e Funga do Brasil), Probably_incorrect
(partial match), or Not_found (no match with any species).
\item Suggested name: If Spelling is Correct, it is the same as the input_name.
If Spelling is Probably_correct, one or more suggested names are listed,
found according to the maximum distance. If Spelling is "Not_found", the value
is NA.
\item Distance: The integer Levenshtein edit distance. It represents the number
of single-character edits (insertions, deletions, or substitutions) required
to transform the input_name into the Suggested_name.
\item taxonomicStatus: the taxonomic status of the species name ("valid" or
"synonym").
\item nomenclaturalStatus: the nomenclatural status of the species name. This
information is not available for all species.
\item validName: If the species name is not valid or incorrect, the
valid name of the specie. If the species name is valid and correct,
the same as input_name and Suggested_name.
\item family: the family of the specie.
}
}
\description{
check_fauna_names checks if the species names are correct and searches
for suggestions if the name is misspelled or not found in the Fauna do
Brasil database
}
\examples{
data("fauna_data")
spp <- c("Pantera onça", "Mazama bororo", "Mazama jucunda",
          "Araucaria angustifolia")
check_fauna_names(data = fauna_data, species = spp)
}
\references{
Brazilian Zoology Group. Catálogo Taxonômico da Fauna do Brasil. Available at:
https://ipt.jbrj.gov.br/jbrj/resource?r=catalogo_taxonomico_da_fauna_do_brasil
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_faunabr.R
\name{filter_faunabr}
\alias{filter_faunabr}
\title{Identify records outside natural ranges according to Fauna do Brasil}
\usage{
filter_faunabr(data, occ, species = "species", long = "x", lat = "y",
                      by_state = TRUE, buffer_state = 20, by_country = TRUE,
                      buffer_country = 20, value = "flag&clean",
                      keep_columns = TRUE, spat_state = NULL,
                      spat_country = NULL, verbose = TRUE)
}
\arguments{
\item{data}{(data.frame) the data.frame imported with the
\code{\link{load_faunabr}} function.}

\item{occ}{(data.frame) a data.frame with the records of the species.}

\item{species}{(character) column name in occ with species names.
Default = "species"}

\item{long}{(character) column name in occ with longitude data. Default = "x"}

\item{lat}{(character) column name in occ with latitude data. Default = "y"}

\item{by_state}{(logical) filter records by state? Default = TRUE}

\item{buffer_state}{(numeric) buffer (in km) around the polygons of the
states of occurrence of the specie. Default = 20.}

\item{by_country}{(logical) filter records by country? Default = TRUE}

\item{buffer_country}{(numeric) buffer (in km) around the polygons of the
countries of occurrence of the specie. Default = 20.}

\item{value}{(character) Defines output values. See Value section.
Default = "flag&clean".}

\item{keep_columns}{(logical) if TRUE, keep all the original columns of the
input occ. If False, keep only the columns species, long and lat.
Default = TRUE}

\item{spat_state}{(SpatVector) a SpatVector of the Brazilian states. By
default, it uses the SpatVector provided by geobr::read_state(). It can be
another Spatvector, but the structure must be identical to
'faunabr::states', with a column called "abbrev_state" identifying the states
codes.}

\item{spat_country}{(SpatVector) a SpatVector of the world countries. By
default, it uses the SpatVector provided by rnaturalearth::ne_countries. It
can be another Spatvector, but the structure must be identical to
'faunabr::world_fauna', with a column called "country_code" identifying the
country codes.}

\item{verbose}{(logical) Whether to display species being filtered during
function execution. Set to TRUE to enable display, or FALSE to run silently.
Default = TRUE.}
}
\value{
Depending on the 'value' argument. If value = "flag", it returns the
same data.frame provided in data with additional columns indicating if the
record falls inside the natural range of the specie (TRUE) or outside
(FALSE).
If value = "clean", it returns a data.frame with only the records that passes
all the tests (TRUE for all the filters). If value = "flag&clean" (Default),
it returns a list with two data.frames: one with the flagged records and one
with the cleaned records.
}
\description{
This function removes or flags records outside of the species'
natural ranges according to information provided by the Fauna do Brasil database
}
\details{
If by_state = TRUE and/or by_country = TRUE, the function takes polygons
representing the states and/or countrys with confirmed occurrences of the
specie, draws a buffer around the polygons, and tests if the records of the
species fall inside it.
}
\examples{
data("fauna_data") #Load fauna e Funga do Brasil data
data("occurrences") #Load occurrences
pts <- subset(occurrences, species == "Panthera onca")
fd <- filter_faunabr(data = fauna_data,
                     occ = pts, long = "x", lat = "y", species = "species",
                     by_state = TRUE, buffer_state = 20,
                     by_country = TRUE, buffer_country = 20,
                     value = "flag&clean", keep_columns = TRUE,
                     verbose = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset_fauna.R
\name{subset_fauna}
\alias{subset_fauna}
\title{Extract a subset of species from Fauna do Brasil database}
\usage{
subset_fauna(data, species, include_subspecies = FALSE)
}
\arguments{
\item{data}{(data.frame) the data.frame imported with the
\code{\link{load_faunabr}} function.}

\item{species}{(character) names of the species to be extracted from Fauna do
Brasil database.}

\item{include_subspecies}{(logical) include subspecies? Default = FALSE}
}
\value{
A data.frame with the selected species.
}
\description{
Returns a data.frame with a subset of species from Fauna do Brasil database
}
\examples{
data("fauna_data") #Load data example
#Species to extract from database
spp <- c("Panthera onca", "Mazama jucunda", "Subulo gouzoubira")
spp_subset <- subset_fauna(data = fauna_data, species = spp,
                      include_subspecies = FALSE)
spp_subset
}
